/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.bigdata.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import javax.validation.Valid;

import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.bigdata.entity.VisualMap;
import org.springblade.bigdata.vo.VisualMapVO;
import org.springblade.bigdata.service.IVisualMapService;
import org.springblade.core.boot.ctrl.BladeController;

/**
 * 可视化地图配置表 控制器
 *
 * @author BladeX
 * @since 2021-03-01
 */
@RestController
@AllArgsConstructor
@RequestMapping("bigdata/visualmap")
@Api(value = "可视化地图配置表", tags = "可视化地图配置表接口")
public class VisualMapController extends BladeController {

	private final IVisualMapService visualMapService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入visualMap")
	public R<VisualMap> detail(VisualMap visualMap) {
		VisualMap detail = visualMapService.getOne(Condition.getQueryWrapper(visualMap));
		return R.data(detail);
	}

	/**
	 * 分页 可视化地图配置表
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入visualMap")
	public R<IPage<VisualMap>> list(VisualMap visualMap, Query query) {
		IPage<VisualMap> pages = visualMapService.page(Condition.getPage(query), Condition.getQueryWrapper(visualMap));
		return R.data(pages);
	}

	/**
	 * 自定义分页 可视化地图配置表
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入visualMap")
	public R<IPage<VisualMapVO>> page(VisualMapVO visualMap, Query query) {
		IPage<VisualMapVO> pages = visualMapService.selectVisualMapPage(Condition.getPage(query), visualMap);
		return R.data(pages);
	}

	/**
	 * 新增 可视化地图配置表
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入visualMap")
	public R save(@Valid @RequestBody VisualMap visualMap) {
		return R.status(visualMapService.save(visualMap));
	}

	/**
	 * 修改 可视化地图配置表
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入visualMap")
	public R update(@Valid @RequestBody VisualMap visualMap) {
		return R.status(visualMapService.updateById(visualMap));
	}

	/**
	 * 新增或修改 可视化地图配置表
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入visualMap")
	public R submit(@Valid @RequestBody VisualMap visualMap) {
		return R.status(visualMapService.saveOrUpdate(visualMap));
	}

	
	/**
	 * 删除 可视化地图配置表
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(visualMapService.removeByIds(Func.toLongList(ids)));
	}

	
}
