/**
 * 
 */
package org.springblade.cube.controller;

import java.util.Date;
import java.util.List;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.api.R;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.model.CubeWorkspace;
import org.springblade.cube.service.CubeDataService;
import org.springblade.cube.service.CubeWorkspaceService;
import org.springblade.system.client.DataPermClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * @author wookvn
 *
 */
@RestController
@RequestMapping("/cube/workspace")
@Api(value = "数据集工作空间接口", tags = "数据集工作空间相关接口")
public class CubeWorkspaceController extends BladeController {
	
	@Autowired
	private CubeWorkspaceService cubeWorkspaceService;
	
	@Autowired
	private CubeDataService cubeDataService;
	
	@Autowired
	private DataPermClient dataPermClient;
	
	/**
	 * 
	 * @param name
	 * @return
	 */
	@GetMapping("/list")
	@ApiOperation(value = "分页数据列表")
	public R<List<CubeWorkspace>> list(String name){
		//判断是否超级权限组「拥有所有权限」
		Boolean checkSuperPermRet = dataPermClient.checkSuperPermission(getUser().getUserId());
		LambdaQueryWrapper<CubeWorkspace> queryWrapper = Wrappers.<CubeWorkspace>lambdaQuery();
		if(StrUtil.isNotBlank(name)) {
			queryWrapper.like(CubeWorkspace::getWsName, name);
		}
		List<CubeWorkspace> dataList = cubeWorkspaceService.list(queryWrapper.orderByDesc(CubeWorkspace::getSortNum).orderByDesc(CubeWorkspace::getId));
		if(checkSuperPermRet != null && checkSuperPermRet) {
			return R.data(dataList);
		}else {
			//查询有权限的数据集分类ID
			List<Integer> workspaceIdList = cubeDataService.workspaceIdList(getUser().getUserId());
			if(CollectionUtil.isNotEmpty(workspaceIdList)) {
				//1.获取这些分类的父级ID
				dataList.stream().filter(f -> workspaceIdList.contains(f.getId()) && f.getParentId() != 0).forEach(t -> workspaceIdList.add(t.getParentId()));
			}
			//2.删除不在权限内的数据集分类，并且创建人不是自己的用户
			dataList.removeIf(y -> !workspaceIdList.contains(y.getId()) && y.getCreateUserId().longValue() != getUser().getUserId());
			return R.data(dataList);
		}
	}
	
	/**
	 * 
	 * @return
	 */
	@PostMapping("/delete")
	@ApiOperation(value = "删除数据")
	public R<String> delete(@RequestParam Integer workspaceId){
		CubeWorkspace dbWorkspace = cubeWorkspaceService.getById(workspaceId);
		if(dbWorkspace != null && dbWorkspace.getSystemStatus() == 1) {
			return R.fail("系统分类不能删除");
		}
		long wsCount = cubeWorkspaceService.count(Wrappers.<CubeWorkspace>lambdaQuery().eq(CubeWorkspace::getParentId, workspaceId));
		if(wsCount > 0) {
			return R.fail("此分类下有子分类，不能删除");
		}
		long count = cubeDataService.count(Wrappers.<CubeData>lambdaQuery().eq(CubeData::getWorkspaceId, workspaceId));
		if(count > 0) {
			return R.fail("有数据集正在使用，不能删除");
		}
		return R.status(cubeWorkspaceService.removeById(workspaceId));
	}
	
	/**
	 * 
	 * @return
	 */
	@PostMapping("/saveOrUpdate")
	@ApiOperation(value = "添加或修改数据")
	public R<String> saveOrUpdate(@RequestBody CubeWorkspace cubeWorkspace){
		if(cubeWorkspace.getId() == null) {
			cubeWorkspace.setCreateBy(getUser().getNickName());
			cubeWorkspace.setCreateUserId(getUser().getUserId());
			cubeWorkspace.setCreateTime(new Date());
			//查询显示状态的数据集【静态数据不显示，但是workspaceId=0】
			long count = cubeDataService.count(Wrappers.<CubeData>lambdaQuery().eq(CubeData::getWorkspaceId, cubeWorkspace.getParentId()).eq(CubeData::getDisplayStatus, 1));
			if(count > 0) {
				return R.fail("你选择的父级分类下有数据集，不能新建子分类。");
			}
		}else {
			CubeWorkspace dbWorkspace = cubeWorkspaceService.getById(cubeWorkspace.getId());
			if(dbWorkspace != null && dbWorkspace.getSystemStatus() == 1) {
				return R.fail("系统分类不能修改");
			}
		}
		return R.status(cubeWorkspaceService.saveOrUpdate(cubeWorkspace));
	}

	@GetMapping("/list-tenantid")
	@ApiOperation(value = "根据租户id查询数据集分类")
	public R<List<CubeWorkspace>> getWorkspaceByTenantId(String tenantId){
		return R.data(cubeWorkspaceService.list(Wrappers.<CubeWorkspace>lambdaQuery()
				.eq(CubeWorkspace::getTenantId,tenantId)));
	}
}
