/**
 *
 */
package org.springblade.cube.model;

import java.io.Serializable;

import org.springblade.core.mp.base.TenantEntity;
import org.springblade.cube.enums.AggregationTypeEnum;
import org.springblade.cube.enums.ColumnAttrEnum;
import org.springblade.cube.enums.DimensionTypeEnum;
import org.springblade.datasource.enums.DateFormatEnum;
import org.springblade.datasource.enums.NumberFormatEnum;
import org.springblade.datasource.enums.ViewColumnTypeEnum;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.NullSerializer;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

/**
 * @author wookvn
 *
 */
@Data
@ApiModel("数据集_表结构")
@TableName("cube_table_column")
public class CubeTableColumn extends TenantEntity implements Serializable {

    /**
     *
     */
    private static final long serialVersionUID = 1L;

    @TableId(type = IdType.AUTO)
    private Integer id;

    @ApiModelProperty("数据集ID")
    private Integer cubeDataId;

    @ApiModelProperty("表名称")
    private String tableName;

    @ApiModelProperty("表别名")
    private String tableAliasName;

    @ApiModelProperty("字段显示名")
    private String columnTitle;

    @ApiModelProperty("字段名")
    private String columnName;

    @ApiModelProperty("字段注释")
    private String columnComment;

    @JsonSerialize(nullsUsing = NullSerializer.class)
    @ApiModelProperty("旧字段数据展示类型")
    private ViewColumnTypeEnum oldViewDataType;

    @JsonSerialize(nullsUsing = NullSerializer.class)
    @ApiModelProperty("字段数据展示类型")
    private ViewColumnTypeEnum viewDataType;

    @JsonSerialize(nullsUsing = NullSerializer.class)
    @ApiModelProperty("字段维度/指标类型")
    private DimensionTypeEnum dimensionType;

    @JsonSerialize(nullsUsing = NullSerializer.class)
    @ApiModelProperty("字段属性(NORMAL-标准字段/VIRTUAL-虚拟字段)")
    private ColumnAttrEnum columnAttr;

    @JsonSerialize(nullsUsing = NullSerializer.class)
    @ApiModelProperty("数值类型展示格式")
    private NumberFormatEnum numberDataFormat;

    @JsonSerialize(nullsUsing = NullSerializer.class)
    @ApiModelProperty("日期类型展示格式")
    private DateFormatEnum timeDataFormat;

    @JsonSerialize(nullsUsing = NullSerializer.class)
    @ApiModelProperty("指标的聚合函数")
    private AggregationTypeEnum aggregationType;

    @ApiModelProperty("虚拟字段公式")
    private String columnFormula;

    @ApiModelProperty("是否隐藏字段(0-否/1-是)")
    private Integer hiddenStatus;

    @ApiModelProperty("排序号")
    private Integer sortNum;

    @ApiModelProperty("虚拟字段SQL[系统设置]")
    private String columnSql;

    @ApiModelProperty("虚拟字段是否包含聚合函数[1-是/0-否]")
    private Integer virtualAggregationStatus;

    @ApiModelProperty("虚拟字段是否仅用于指标列查询[1-是/0-否]")
    private Integer virtualOnlyZhibiaoStatus;

    @ApiModelProperty("字段别名[后端赋值]")
    @TableField(exist = false)
    private String columnAliasName;
    @TableField(exist = false)
    @ApiModelProperty(value = "是否主键")
    private Integer primaryKeyStatus;
    @TableField(exist = false)
    @ApiModelProperty(value = "是否自增")
    private Integer autoIncrementStatus = 0;
}
