/**
 *
 */
package org.springblade.cube.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.apache.commons.collections4.map.HashedMap;
import org.springblade.bi.client.form.SubTableQueryParam;
import org.springblade.bi.client.form.param.JoinColumnItem;
import org.springblade.bi.client.form.param.MasterTableColumnItem;
import org.springblade.bi.client.form.param.TableLinkJoinItem;
import org.springblade.bi.client.form.param.TableLinkJoinParam;
import org.springblade.bi.client.model.CubeBrowserSqlModel;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.exception.ServiceException;
import org.springblade.core.tool.utils.Func;
import org.springblade.cube.dto.BrowserParam;
import org.springblade.cube.dto.BrowserRequest;
import org.springblade.cube.dto.CubeTableName;
import org.springblade.cube.enums.JoinTypeEnum;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.model.CubeTableColumn;
import org.springblade.cube.utils.CubeConstants;
import org.springblade.cube.utils.CubeUtils;
import org.springblade.datasource.context.DialectContext;
import org.springblade.datasource.enums.DateFormatEnum;
import org.springblade.datasource.enums.DsTypeEnum;
import org.springblade.datasource.enums.FunctionMethodEnum;
import org.springblade.datasource.enums.NumberFormatEnum;
import org.springblade.datasource.enums.ViewColumnTypeEnum;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.service.IReportDatasourceService;
import org.springblade.visual.dto.ComponentFilter;
import org.springblade.visual.dto.ComponentFilterRelation;
import org.springblade.visual.dto.CubePermParseDto;
import org.springblade.visual.dto.PreviewBaseDto;
import org.springblade.visual.enums.CommonFunctionEnum;
import org.springblade.visual.service.QuerySqlService;
import org.springblade.visual.utils.Tuple2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.lang.Tuple;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;

/**
 * @author wookvn
 */
@Component
public class CubeDataBrowserService {

    @Autowired
    private QuerySqlService querySqlService;

    @Autowired
    private CubeDataService cubeDataService;

    @Autowired
    private CubeTableColumnService cubeTableColumnService;

    @Autowired
    private IReportDatasourceService reportDatasourceService;

    @Autowired
    private CubeDataPermissionService cubeDataPermissionService;

    @Autowired
    private DialectContext dialectContext;

    /**
     * @param cubeId
     * @param browserRequest
     * @param bladeUser
     * @return
     */
    public CubeBrowserSqlModel buildSql(Integer cubeId, BrowserRequest browserRequest, BladeUser bladeUser) {
        // 获取分页model和查询参数model
        List<BrowserParam> paramList = browserRequest.getParamList();
        Query query = browserRequest.getPageQuery();

        CubeData cubeData = cubeDataService.getById(cubeId);
        if (cubeData == null) {
            throw new ServiceException("数据集不存在");
        }
        if(cubeData.getWorkspaceId() != 0){  //临时处理：数据集的分类ID=0不校验权限组
            // 检查数据集是否有权限
            boolean checkCubePermRet = cubeDataPermissionService.checkCubePermForUser(cubeId, cubeData.getCreateUserId(), bladeUser);
            if (!checkCubePermRet) {
                throw new ServiceException("数据集没有访问权限");
            }
        }

        ReportDatasource reportDatasource = reportDatasourceService.getById(cubeData.getDatasourceId());
        if (reportDatasource == null) {
            throw new ServiceException("数据源不存在");
        }
        DsTypeEnum dsTypeEnum = DsTypeEnum.resolve(reportDatasource.getDsType());
        String fromSql = cubeData.getQuerySql();
        // 构造Select模块
        List<CubeTableColumn> columnList = cubeTableColumnService
                .list(Wrappers.<CubeTableColumn>lambdaQuery().eq(CubeTableColumn::getCubeDataId, cubeId).eq(CubeTableColumn::getTenantId, bladeUser.getTenantId()));
        // 初始化别名
        for (int i = 0; i < columnList.size(); i++) {
            columnList.get(i).setColumnAliasName(CubeConstants.COLUMN_ALIAS_NAME_PREFIX + i);
        }
        // 查询SQL
        List<String> selectSqlList = new ArrayList<String>();
        // 条件SQL
        List<String> conditionSqlList = new ArrayList<String>();

        Map<String, List<Object>> extraParams = querySqlService.buildGlobalParams(null, bladeUser);
        // 行/列 权限
        CubePermParseDto cubePermParseDto = cubeDataPermissionService.parseCubePermission(cubeId, bladeUser);
        Set<Integer> excludeColumnIdSet = Optional.ofNullable(cubePermParseDto.getExcludeColumnIdList()).orElse(new HashSet<>());
        for (CubeTableColumn column : columnList) {
            // 计算字段包含聚合函数 OR 列权限包含该列，不做查询
            if (Optional.<Integer>ofNullable(column.getVirtualAggregationStatus()).orElse(0) == 1 || excludeColumnIdSet.contains(column.getId())) {
                continue;
            }
            // 处理全局参数变量
            String columnName = CubeUtils.formatVirtualSqlFragment(column.getColumnSql(), extraParams);
            String selectColumnName = columnName, queryColumnName = columnName;
            // 格式化日期
            if (column.getTimeDataFormat() != null && column.getViewDataType() == ViewColumnTypeEnum.DATETIME) {
                queryColumnName = dialectContext.getDialect(dsTypeEnum).formatDateTime(column.getTimeDataFormat(),
                        queryColumnName);
                selectColumnName = column.getTimeDataFormat() != DateFormatEnum.yyyy_MM_ddhh$mm$ss ? queryColumnName
                        : selectColumnName;
            }
            // 格式化数字
            if (column.getNumberDataFormat() != null && column.getViewDataType() == ViewColumnTypeEnum.NUMBER) {
                // 数字格式化函数
                selectColumnName = String.format(column.getNumberDataFormat().getMethod(), selectColumnName);
                // select显示拼接百分号
                if (NumberFormatEnum.isPercentFormat(column.getNumberDataFormat())) {
                    selectColumnName = dialectContext.getDialect(dsTypeEnum).formatMethodFunction(
                            FunctionMethodEnum.NUMBER_JOINT_PERCENT, selectColumnName,
                            column.getNumberDataFormat().getOracleFmFormat());
                }
            }
            selectSqlList.add(selectColumnName + " as \"" + column.getColumnAliasName() + "\"");
            // 是否包含在查询参数中
            BrowserParam browserParam = CollectionUtil.isNotEmpty(paramList) ? paramList.stream().filter(item -> item.getCubeColumnId().equals(column.getId())).findFirst().orElse(null) : null;
            if (browserParam != null) {
                String conditionSql = browserParam.getFunction().handle(queryColumnName, column.getViewDataType(),
                        browserParam.getValues(), dsTypeEnum);
                if (StrUtil.isNotEmpty(conditionSql)) {
                    conditionSqlList.add(conditionSql);
                }
            }
        }
        StringBuilder querySqlBuilder = new StringBuilder("select ").append(String.join(",", selectSqlList))
                .append(" from ").append(fromSql);
        // 解析数据集全局过滤器
        if (StrUtil.isNotBlank(cubeData.getFilterJson())) {
            PreviewBaseDto cubePreviewDto = JSON.parseObject(cubeData.getFilterJson(), PreviewBaseDto.class);
            // 解析SQL
            Tuple2<String, String> whereHavingSqlTuple = querySqlService.parseFilters(
                    JSON.parseArray(cubePreviewDto.getFilterList(), ComponentFilter.class),
                    JSON.parseObject(cubePreviewDto.getFilterRelation(), ComponentFilterRelation.class), dsTypeEnum,
                    extraParams);
            if (StrUtil.isNotBlank(whereHavingSqlTuple.getT1())) {
                conditionSqlList.add(whereHavingSqlTuple.getT1());
            }
        }

        if (StrUtil.isNotBlank(cubePermParseDto.getFilterSql())) {
            conditionSqlList.add(cubePermParseDto.getFilterSql());
        }
        // 拼接查询条件
        if (!conditionSqlList.isEmpty()) {
            querySqlBuilder.append(" where " + String.join(" AND ", conditionSqlList));
        }

        // ID和别名Map
        Map<Integer, String> columnDict = columnList.stream().collect(Collectors.toMap(CubeTableColumn::getId, CubeTableColumn::getColumnAliasName));

        // 设置OrderBy
        Integer[] ascArr = Func.toIntArray(query.getAscs());
        query.setAscs(Stream.of(ascArr).filter(x -> x > 0).map(columnDict::get).filter(Objects::nonNull).collect(Collectors.joining(",")));
        Integer[] descArr = Func.toIntArray(query.getDescs());
        query.setDescs(Stream.of(descArr).filter(x -> x > 0).map(columnDict::get).filter(Objects::nonNull).collect(Collectors.joining(",")));
        return new CubeBrowserSqlModel(querySqlBuilder.toString(), columnDict);
    }

    /**
     * @param cubeId
     * @param browserRequest
     * @param tableLinkJoinParam 处理有主表数据集，加上子表关联关系的情况
     * @param bladeUser
     * @return
     */
    public CubeBrowserSqlModel buildSql(Integer cubeId, BrowserRequest browserRequest, TableLinkJoinParam tableLinkJoinParam, List<SubTableQueryParam> subTableQueryParams, BladeUser bladeUser) {
        // 获取分页model和查询参数model
        List<BrowserParam> paramList = browserRequest.getParamList();
        Query query = browserRequest.getPageQuery();

        CubeData cubeData = cubeDataService.getById(cubeId);
        if (cubeData == null) {
            throw new ServiceException("数据集不存在");
        }
        if(cubeData.getWorkspaceId() != 0) {  //临时处理：数据集的分类ID=0不校验权限组
            // 检查数据集是否有权限
            boolean checkCubePermRet = cubeDataPermissionService.checkCubePermForUser(cubeId, cubeData.getCreateUserId(), bladeUser);
            if (!checkCubePermRet) {
                throw new ServiceException("数据集没有访问权限");
            }
        }
        ReportDatasource reportDatasource = reportDatasourceService.getById(cubeData.getDatasourceId());
        if (reportDatasource == null) {
            throw new ServiceException("数据源不存在");
        }
        DsTypeEnum dsTypeEnum = DsTypeEnum.resolve(reportDatasource.getDsType());
        // 构造Select模块
        List<CubeTableColumn> columnList = cubeTableColumnService.list(Wrappers.<CubeTableColumn>lambdaQuery().eq(CubeTableColumn::getCubeDataId, cubeId).eq(CubeTableColumn::getTenantId, bladeUser.getTenantId()));
        //字段ID和字段别名的映射关系
        Map<Integer, String> masterTableColumnAliasMap = tableLinkJoinParam.getTableColumns().stream().collect(Collectors.toMap(MasterTableColumnItem::getCubeColumnId, MasterTableColumnItem::getColumnAlias));
        // 查询SQL
        List<String> selectSqlList = new ArrayList<String>();
        // 条件SQL
        List<String> conditionSqlList = new ArrayList<String>();

        Map<String, List<Object>> extraParams = querySqlService.buildGlobalParams(null, bladeUser);
        // 行/列 权限
        CubePermParseDto cubePermParseDto = cubeDataPermissionService.parseCubePermission(cubeId, bladeUser);
        Set<Integer> excludeColumnIdSet = Optional.ofNullable(cubePermParseDto.getExcludeColumnIdList()).orElse(new HashSet<>());

        Map<Integer, String> masterTableColumnMap = new HashedMap<>();
        for (CubeTableColumn column : columnList) {
            masterTableColumnMap.put(column.getId(), column.getColumnName());
            //获取别名
            String tableColumnAlias = masterTableColumnAliasMap.get(column.getId());
            // 计算字段包含聚合函数 OR 列权限包含该列，不做查询
            if (StrUtil.isBlank(tableColumnAlias) || Optional.<Integer>ofNullable(column.getVirtualAggregationStatus()).orElse(0) == 1 || excludeColumnIdSet.contains(column.getId())) {
                continue;
            }
            // 处理全局参数变量
            String columnName = CubeUtils.formatVirtualSqlFragment(column.getColumnSql(), extraParams);
            String selectColumnName = columnName, queryColumnName = columnName;
            // 格式化日期
            if (column.getTimeDataFormat() != null && column.getViewDataType() == ViewColumnTypeEnum.DATETIME) {
                queryColumnName = dialectContext.getDialect(dsTypeEnum).formatDateTime(column.getTimeDataFormat(), queryColumnName);
                selectColumnName = column.getTimeDataFormat() != DateFormatEnum.yyyy_MM_ddhh$mm$ss ? queryColumnName : selectColumnName;
            }
            // 格式化数字
            if (column.getNumberDataFormat() != null && column.getViewDataType() == ViewColumnTypeEnum.NUMBER) {
                // 数字格式化函数
                selectColumnName = String.format(column.getNumberDataFormat().getMethod(), selectColumnName);
                // select显示拼接百分号
                if (NumberFormatEnum.isPercentFormat(column.getNumberDataFormat())) {
                    selectColumnName = dialectContext.getDialect(dsTypeEnum).formatMethodFunction(FunctionMethodEnum.NUMBER_JOINT_PERCENT, selectColumnName,
                            column.getNumberDataFormat().getOracleFmFormat());
                }
            }
            selectSqlList.add(selectColumnName + " as \"" + tableColumnAlias + "\"");
            // 是否包含在查询参数中
            BrowserParam browserParam = CollectionUtil.isNotEmpty(paramList) ? paramList.stream().filter(item -> item.getCubeColumnId().equals(column.getId())).findFirst().orElse(null) : null;
            if (browserParam != null) {
                String conditionSql = browserParam.getFunction().handle(queryColumnName, column.getViewDataType(), browserParam.getValues(), dsTypeEnum);
                if (StrUtil.isNotEmpty(conditionSql)) {
                    conditionSqlList.add(conditionSql);
                }
            }
        }
        //拼接from表
        List<CubeTableName> tableLinkJoinList = JSON.parseArray(cubeData.getCubeTableNames(), CubeTableName.class);
        CubeTableName cubeTableName = tableLinkJoinList.get(0);
        Tuple tableSqlTuple = _buildSelectAndFromSql(cubeTableName, masterTableColumnMap, tableLinkJoinParam);
        String fromSql = tableSqlTuple.get(0), subSelectSql = tableSqlTuple.get(1);
        //selectSql拼接上子表的查询SQL
        selectSqlList.add(subSelectSql);
        StringBuilder querySqlBuilder = new StringBuilder("select ").append(String.join(",", selectSqlList)).append(" from ").append(fromSql);
        //子表查询条件构建
        if (CollectionUtil.isNotEmpty(subTableQueryParams)) {
            //获取子表别名
            Map<String, TableLinkJoinItem> subTableMap = tableLinkJoinParam.getTableLinkJoins().stream().collect(Collectors.toMap(TableLinkJoinItem::getTableName, Function.identity()));
            for (SubTableQueryParam subTableQueryParam : subTableQueryParams) {
                String subQueryColumnName = subTableMap.get(subTableQueryParam.getTableName()).getTableAlias() + "." + subTableQueryParam.getColumnName();
                ViewColumnTypeEnum typeEnum = subTableQueryParam.getColumnType() != null ? ViewColumnTypeEnum.resolve(subTableQueryParam.getColumnType()) : ViewColumnTypeEnum.STRING;
                String subConditionSql = getCommonFunctionEnum(subTableQueryParam.getFunction())
                        .handle(subQueryColumnName, typeEnum, subTableQueryParam.getValues(), dsTypeEnum);
                conditionSqlList.add(subConditionSql);
            }
        }
        // 解析数据集全局过滤器
        if (StrUtil.isNotBlank(cubeData.getFilterJson())) {
            PreviewBaseDto cubePreviewDto = JSON.parseObject(cubeData.getFilterJson(), PreviewBaseDto.class);
            // 解析SQL
            Tuple2<String, String> whereHavingSqlTuple = querySqlService.parseFilters(
                    JSON.parseArray(cubePreviewDto.getFilterList(), ComponentFilter.class),
                    JSON.parseObject(cubePreviewDto.getFilterRelation(), ComponentFilterRelation.class), dsTypeEnum,
                    extraParams);
            if (StrUtil.isNotBlank(whereHavingSqlTuple.getT1())) {
                conditionSqlList.add(whereHavingSqlTuple.getT1());
            }
        }

        if (StrUtil.isNotBlank(cubePermParseDto.getFilterSql())) {
            conditionSqlList.add(cubePermParseDto.getFilterSql());
        }
        // 拼接查询条件
        if (!conditionSqlList.isEmpty()) {
            querySqlBuilder.append(" where " + String.join(" AND ", conditionSqlList));
        }

        Map<String, String> extColumnDict = new HashMap<>();
        tableLinkJoinParam.getTableLinkJoins().stream().map(TableLinkJoinItem::getTableColumns).forEach(item -> {
            item.forEach(ft -> extColumnDict.put(ft.getVirColumnId(), ft.getColumnAlias()));
        });
        masterTableColumnAliasMap.forEach((columnId, columnAlias) -> extColumnDict.put(columnId.toString(), columnAlias));

        // 设置OrderBy
        Integer[] ascArr = Func.toIntArray(query.getAscs());
        query.setAscs(Stream.of(ascArr).filter(x -> x > 0).map(masterTableColumnAliasMap::get).filter(Objects::nonNull).collect(Collectors.joining(",")));
        Integer[] descArr = Func.toIntArray(query.getDescs());
        query.setDescs(Stream.of(descArr).filter(x -> x > 0).map(masterTableColumnAliasMap::get).filter(Objects::nonNull).collect(Collectors.joining(",")));

        CubeBrowserSqlModel cubeBrowserSqlModel = new CubeBrowserSqlModel();
        cubeBrowserSqlModel.setSql(querySqlBuilder.toString());
        cubeBrowserSqlModel.setExtColumnDict(extColumnDict);
        return cubeBrowserSqlModel;
    }

    public CommonFunctionEnum getCommonFunctionEnum(String functionEnum) {
        for (CommonFunctionEnum commonFunctionEnum : CommonFunctionEnum.values()) {
            if (commonFunctionEnum.toString().equals(functionEnum)) {
                return commonFunctionEnum;
            }
        }
        return null;
    }

    /**
     * 构造FromSql和select语句
     *
     * @param cubeTableName
     * @param masterTableColumnMap
     * @param tableLinkJoinParam
     * @return
     */
    private Tuple _buildSelectAndFromSql(CubeTableName cubeTableName, Map<Integer, String> masterTableColumnMap, TableLinkJoinParam tableLinkJoinParam) {
        StringBuilder fromSqlBuilder = new StringBuilder();
        fromSqlBuilder.append(cubeTableName.getTableName() + " " + cubeTableName.getTableAliasName());
        List<TableLinkJoinItem> tableLinkJoins = tableLinkJoinParam.getTableLinkJoins();
        List<String> subSelectColumnList = new ArrayList<>();
        for (TableLinkJoinItem tableLinkJoinItem : tableLinkJoins) {
            String tableAlias = RandomUtil.randomString("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ", 5);
            String tableName = tableLinkJoinItem.getTableName();
            tableLinkJoinItem.setTableAlias(tableAlias);
            JoinTypeEnum joinType = JoinTypeEnum.valueOf(tableLinkJoinItem.getJoinType());
            //left join subtable subAlias on masterAlias.id = subAlias.sid
            fromSqlBuilder.append(joinType.getSql() + tableName + " " + tableAlias + " on ");
            List<String> sqlList = new ArrayList<>();
            for (JoinColumnItem joinColumnItem : tableLinkJoinItem.getJoinColumns()) {
                String masterColumn = cubeTableName.getTableAliasName() + "." + masterTableColumnMap.get(joinColumnItem.getMasterColumnId());
                String subColumn = tableAlias + "." + joinColumnItem.getJoinColumnName();
                sqlList.add(masterColumn + " = " + subColumn);
            }
            fromSqlBuilder.append(String.join(" AND ", sqlList));
            //子表字段select语句
            tableLinkJoinItem.getTableColumns().stream().map(item -> tableAlias + "." + item.getColumnName() + " as \"" + item.getColumnAlias() + "\"").forEach(subSelectColumnList::add);
        }
        return new Tuple(fromSqlBuilder.toString(), String.join(",", subSelectColumnList));
    }
}
