package org.springblade.cube.service;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.springblade.bi.client.form.AggCubeCreateForm;
import org.springblade.bi.client.form.enums.LinkTypeEnum;
import org.springblade.bi.client.form.param.JoinColumnItem;
import org.springblade.bi.client.form.param.MixLinkJoinItem;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.exception.ServiceException;
import org.springblade.cube.dto.CubeTableName;
import org.springblade.cube.dto.TableLinkJoin;
import org.springblade.cube.enums.AggregationTypeEnum;
import org.springblade.cube.enums.ColumnAttrEnum;
import org.springblade.cube.enums.DimensionTypeEnum;
import org.springblade.cube.enums.JoinTypeEnum;
import org.springblade.cube.enums.TableTypeEnum;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.model.CubeTableColumn;
import org.springblade.datasource.context.DialectContext;
import org.springblade.datasource.enums.DateFormatEnum;
import org.springblade.datasource.enums.NumberFormatEnum;
import org.springblade.datasource.enums.ViewColumnTypeEnum;
import org.springblade.datasource.model.TableColumnDto;
import org.springblade.datasource.model.TableViewDto;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.service.IReportDatasourceService;
import org.springblade.visual.service.QuerySqlService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 数据集创建 - 主要给低代码提供业务支撑
 * - 支持单数据集+物理表的多级关联关系 来创建聚合数据集
 */
@Component
public class MixCubeDataService {

    @Autowired
    private QuerySqlService querySqlService;

    @Autowired
    private CubeDataService cubeDataService;

    @Autowired
    private CubeTableColumnService cubeTableColumnService;

    @Autowired
    private IReportDatasourceService reportDatasourceService;

    @Autowired
    private CubeDataPermissionService cubeDataPermissionService;

    @Autowired
    private DialectContext dialectContext;

    @Autowired
    private CubeWorkspaceService cubeWorkspaceService;

    /**
     * 添加或修改聚合数据集
     * @param aggCubeCreateForm
     * @return
     */
    public Integer doSaveOrUpdate(AggCubeCreateForm aggCubeCreateForm) {
        //1.根据主数据集ID查询数据集信息
        CubeData masterCubeData = cubeDataService.getById(aggCubeCreateForm.getMasterCubeId());
        _throwNpeExp(masterCubeData, "未查询到主数据集");
        ReportDatasource reportDatasource = reportDatasourceService.getById(aggCubeCreateForm.getDatasourceId());
        _throwNpeExp(reportDatasource, "未查询到数据源");
        //取主数据集的第一个表为主表
        String masterTableName = JSONUtil.toList(masterCubeData.getCubeTableNames(), CubeTableName.class).get(0).getTableName();
        //聚合数据集-主表名称-表单ID-年月日
        String yyyyMMdd = DateUtil.format(new Date(), "yyyyMMdd");
        String cubeName = String.format("聚合数据集-%s-%s-%s", masterTableName, aggCubeCreateForm.getModelName(), yyyyMMdd);

        List<MixLinkJoinItem> linkJoins = aggCubeCreateForm.getLinkJoins();
        List<CubeTableColumn> cubeTableColumnList = new ArrayList<>();
        List<TableLinkJoin> tableLinkJoinList = new ArrayList<>();
        List<CubeTableName> cubeTableNameList = new ArrayList<>();

        //创建或更新数据集
        CubeData cubeData = new CubeData();
        cubeData.setId(aggCubeCreateForm.getAggCubeId());
        cubeData.setCubeName(cubeName);
        cubeData.setRemark(aggCubeCreateForm.getModelId());
        cubeData.setWorkspaceId(cubeWorkspaceService.getIdOfModel(true));
        cubeData.setDisplayStatus(1);
        cubeData.setPublicStatus(0);
        cubeData.setDatasourceId(aggCubeCreateForm.getDatasourceId());
        if(aggCubeCreateForm.getAggCubeId() == null) {
            cubeData.setCreateBy(SecureUtil.getNickName());
            cubeData.setCreateUserId(SecureUtil.getUserId());
            cubeData.setCreateTime(new Date());
        }
        if(CollectionUtil.isNotEmpty(linkJoins)) { //关联关系的表单
            // 格式化处理数据结果
            _normalizeLinkJoin(reportDatasource, linkJoins, cubeTableColumnList, tableLinkJoinList, cubeTableNameList);

            //对tableLinkJoinList进行排序处理 - 主表关联放到第一位
            tableLinkJoinList.sort(Comparator.comparing(TableLinkJoin::getLevel));

            //聚合数据集需要的一些核心字段值
            String cubeTableNames = JSONUtil.toJsonStr(cubeTableNameList), querySql = _buildQuerySql(tableLinkJoinList), linksJoin = JSONUtil.toJsonStr(tableLinkJoinList);

            cubeData.setCubeTableNames(cubeTableNames);
            cubeData.setQuerySql(querySql);
            cubeData.setLinksJoin(linksJoin);
            cubeData.setTableType(TableTypeEnum.LOGIC);
        }else{ //单数据集表单，不存在关联关系
            cubeTableColumnList = cubeTableColumnService.listByCubeId(aggCubeCreateForm.getMasterCubeId());
            cubeData.setCubeTableNames(masterCubeData.getCubeTableNames());
            cubeData.setQuerySql(masterCubeData.getQuerySql());
            cubeData.setLinksJoin(masterCubeData.getLinksJoin());
            cubeData.setTableType(TableTypeEnum.PHYSICS);
        }
        return _doSubmit(aggCubeCreateForm.getAggCubeId(), cubeData, cubeTableColumnList);
    }

    /**
     * 格式化关联关系
     * @param mixLinkJoins
     * @param cubeTableColumnList
     * @param tableLinkJoins
     * @param cubeTableNameList
     */
    private void _normalizeLinkJoin(ReportDatasource reportDatasource, List<MixLinkJoinItem> mixLinkJoins, List<CubeTableColumn> cubeTableColumnList, List<TableLinkJoin> tableLinkJoins, List<CubeTableName> cubeTableNameList) {
        Set<Integer> cubeIdSet = new HashSet<>();
        mixLinkJoins.forEach(item -> {
            if(item.getMasterCubeId() != null){
                cubeIdSet.add(item.getMasterCubeId());
            }
            if(item.getSubCubeId() != null){
                cubeIdSet.add(item.getSubCubeId());
            }
        });
        //查询数据集和数据集字段列表
        List<CubeData> cubeDataList = cubeDataService.listByIds(cubeIdSet);
        Map<Integer, CubeData> cubeDataMap = cubeDataList.stream().collect(Collectors.toMap(CubeData::getId, Function.identity()));
        List<CubeTableColumn> cubeTableColumns = cubeTableColumnService.listByCubeIds(new ArrayList<>(cubeIdSet));
        Map<Integer, CubeTableColumn> cubeTableColumnMap = cubeTableColumns.stream().collect(Collectors.toMap(CubeTableColumn::getId, Function.identity()));
        Map<Integer, List<CubeTableColumn>> cubeTableColumnListMap = cubeTableColumns.stream().collect(Collectors.groupingBy(CubeTableColumn::getCubeDataId));

        Map<String, String> tableAliasMap = new HashMap<>();
        for(MixLinkJoinItem linkJoinItem : mixLinkJoins) {
            if(linkJoinItem.getLinkType() == LinkTypeEnum.CUBE_CUBE) {
                TableLinkJoin tableLinkJoin = _handleCubeToCube(linkJoinItem, tableAliasMap, cubeTableColumnListMap, cubeDataMap, cubeTableColumnMap,
                        cubeTableColumnList, cubeTableNameList);
                tableLinkJoins.add(tableLinkJoin);
            }else if(linkJoinItem.getLinkType() == LinkTypeEnum.CUBE_TABLE){
                TableLinkJoin tableLinkJoin = _handleCubeToTable(reportDatasource, linkJoinItem, tableAliasMap, cubeTableColumnListMap, cubeDataMap, cubeTableColumnMap,
                        cubeTableColumnList, cubeTableNameList);
                tableLinkJoins.add(tableLinkJoin);
            }else if(linkJoinItem.getLinkType() == LinkTypeEnum.TABLE_CUBE) {
                TableLinkJoin tableLinkJoin = _handleTableToCube(reportDatasource, linkJoinItem, tableAliasMap, cubeTableColumnListMap, cubeDataMap, cubeTableColumnMap,
                        cubeTableColumnList, cubeTableNameList);
                tableLinkJoins.add(tableLinkJoin);
            }
        }
    }

    @Transactional
    private Integer _doSubmit(Integer oldCubeId, CubeData cubeData, List<CubeTableColumn> cubeTableColumnList) {
        boolean ret = cubeDataService.saveOrUpdate(cubeData);
        int cubeDataId = cubeData.getId();
        if(ret) { //保存数据集成功，开始操作字段列表
            //先清除掉旧数据，再添加新数据
            if(oldCubeId != null){
                cubeTableColumnService.remove(Wrappers.<CubeTableColumn>lambdaQuery().eq(CubeTableColumn::getCubeDataId, oldCubeId));
            }
            //对字段数据进行清洗处理
            Map<String, Integer> columnTitleMap = new HashMap<>();
            cubeTableColumnList.forEach(item -> {
                item.setId(null);
                item.setCubeDataId(cubeDataId);
                Integer activeNum = columnTitleMap.get(item.getColumnTitle());
                if(activeNum != null) {
                    item.setColumnTitle(item.getColumnTitle() + activeNum);
                }
                columnTitleMap.put(item.getColumnTitle(), activeNum == null ? 1 : activeNum + 1);
            });
            cubeTableColumnService.saveBatch(cubeTableColumnList);
        }
        return cubeDataId;
    }

    /**
     * 构造FromSQL
     * @param tableLinkJoinList
     * @return
     */
    private String _buildQuerySql(List<TableLinkJoin> tableLinkJoinList) {
        StringBuilder fromSqlBuffer = new StringBuilder();
        for (int i = 0; i < tableLinkJoinList.size(); i++) {
            TableLinkJoin tableLinkJoin = tableLinkJoinList.get(i);
            if (i == 0) {
                fromSqlBuffer.append(tableLinkJoin.getSource()).append(" ").append(tableLinkJoin.getSourceId());
            }
            fromSqlBuffer.append(tableLinkJoin.getJoinType().getSql())
                    .append(tableLinkJoin.getTarget()).append(" ").append(tableLinkJoin.getTargetId());
            // 拼接ON
            fromSqlBuffer.append(" ON ");
            int j = 1, size = tableLinkJoin.getJoinKeys().size();
            for (TableLinkJoin.JoinKey joinKey : tableLinkJoin.getJoinKeys()) {
                fromSqlBuffer.append(tableLinkJoin.getSourceId()).append(".").append(joinKey.getSourceKey());
                fromSqlBuffer.append("=");
                fromSqlBuffer.append(tableLinkJoin.getTargetId()).append(".").append(joinKey.getTargetKey());
                fromSqlBuffer.append(j++ != size ? " AND " : "");
            }
        }
        return fromSqlBuffer.toString();
    }

    private TableLinkJoin _handleCubeToCube(MixLinkJoinItem linkJoinItem, Map<String, String> tableAliasMap,
                                            Map<Integer, List<CubeTableColumn>> cubeTableColumnListMap,
                                   Map<Integer, CubeData> cubeDataMap, Map<Integer, CubeTableColumn> cubeTableColumnMap,
                                   List<CubeTableColumn> cubeTableColumnList, List<CubeTableName> cubeTableNameList) {
        Integer masterCubeId = linkJoinItem.getMasterCubeId();
        Integer subCubeId = linkJoinItem.getSubCubeId();
        //主数据集的关联表数据信息
        CubeData cubeData = cubeDataMap.get(masterCubeId);
        _throwNpeExp(cubeData, "未找到数据集信息|cubeId=" + masterCubeId);
        CubeTableName masterCubeTableName = JSONUtil.toList(cubeData.getCubeTableNames(), CubeTableName.class).get(0);
        //子数据集的关联表数据信息
        CubeData subCubeData = cubeDataMap.get(subCubeId);
        _throwNpeExp(subCubeData, "未找到数据集信息|cubeId=" + subCubeId);
        CubeTableName subCubeTableName = JSONUtil.toList(subCubeData.getCubeTableNames(), CubeTableName.class).get(0);
        String masterTableName  = masterCubeTableName.getTableName(),
                masterTableAlias = _getTableAlias(masterTableName, tableAliasMap),
                subTableName     = subCubeTableName.getTableName(),
                subTableAlias    = _getTableAlias(subTableName, tableAliasMap);
        masterCubeTableName.setTableAliasName(masterTableAlias);
        subCubeTableName.setTableAliasName(subTableAlias);
        //处理数据集的表和字段信息
        if(!cubeTableNameList.stream().anyMatch(item -> item.getTableName().equals(masterTableName))){
            cubeTableNameList.add(masterCubeTableName);
            List<CubeTableColumn> cubeTableColumns = cubeTableColumnListMap.get(masterCubeId);
            _throwNpeExp(cubeTableColumns, "未找到数据集的字段列表|cubeId=" + masterCubeId);
            cubeTableColumns.forEach(item -> {
                item.setId(null);item.setCubeDataId(null);
                item.setTableAliasName(masterTableAlias);
                item.setColumnSql(masterTableAlias + "." + item.getColumnName());
            });
            cubeTableColumnList.addAll(cubeTableColumns);
        }
        if(!cubeTableNameList.stream().anyMatch(item -> item.getTableName().equals(subTableName))){
            cubeTableNameList.add(subCubeTableName);
            List<CubeTableColumn> cubeTableColumns = cubeTableColumnListMap.get(subCubeId);
            _throwNpeExp(cubeTableColumns, "未找到数据集的字段列表|cubeId=" + subCubeId);
            cubeTableColumns.forEach(item -> {
                item.setId(null);item.setCubeDataId(null);
                item.setTableAliasName(subTableAlias);
                item.setColumnSql(subTableAlias + "." + item.getColumnName());
            });
            cubeTableColumnList.addAll(cubeTableColumns);
        }
        TableLinkJoin tableLinkJoin = new TableLinkJoin();
        tableLinkJoin.setSource(masterTableName);
        tableLinkJoin.setSourceId(masterTableAlias);
        tableLinkJoin.setTarget(subTableName);
        tableLinkJoin.setTargetId(subTableAlias);
        tableLinkJoin.setLevel(linkJoinItem.getJoinLevel());
        tableLinkJoin.setJoinType(JoinTypeEnum.valueOf(linkJoinItem.getJoinType()));
        List<TableLinkJoin.JoinKey> joinKeys = new ArrayList<>();
        //处理关联关系
        for(JoinColumnItem joinColumnItem : linkJoinItem.getJoinColumns()){
            Integer masterColumnId = joinColumnItem.getMasterColumnId();
            Integer joinColumnId = joinColumnItem.getJoinColumnId();
            CubeTableColumn masterCubeTableColumn = cubeTableColumnMap.get(masterColumnId);
            _throwNpeExp(masterCubeTableColumn, "未找到主数据集的关联字段|columnId=" + masterColumnId);
            CubeTableColumn subCubeTableColumn = cubeTableColumnMap.get(joinColumnId);
            _throwNpeExp(subCubeTableColumn, "未找到子数据集的关联字段|columnId=" + joinColumnId);
            TableLinkJoin.JoinKey joinKey = new TableLinkJoin.JoinKey();
            joinKey.setSourceKey(masterCubeTableColumn.getColumnName());
            joinKey.setTargetKey(subCubeTableColumn.getColumnName());
            joinKeys.add(joinKey);
        }
        tableLinkJoin.setJoinKeys(joinKeys);
        return tableLinkJoin;
    }

    private TableLinkJoin _handleCubeToTable(ReportDatasource ds, MixLinkJoinItem linkJoinItem, Map<String, String> tableAliasMap,
                                             Map<Integer, List<CubeTableColumn>> cubeTableColumnListMap,
                                             Map<Integer, CubeData> cubeDataMap, Map<Integer, CubeTableColumn> cubeTableColumnMap,
                                             List<CubeTableColumn> cubeTableColumnList, List<CubeTableName> cubeTableNameList){
        Integer masterCubeId = linkJoinItem.getMasterCubeId();
        String subTableName = linkJoinItem.getSubTableName();
        //主数据集的关联表数据信息
        CubeData cubeData = cubeDataMap.get(masterCubeId);
        _throwNpeExp(cubeData, "未找到数据集信息|cubeId=" + masterCubeId);
        CubeTableName masterCubeTableName = JSONUtil.toList(cubeData.getCubeTableNames(), CubeTableName.class).get(0);
        //查询子表信息
        List<TableViewDto> tableViewDtos = dialectContext.getDialect(ds.getDsTypeSyl()).listTableViewsByDatasource(ds, subTableName);
        _throwNpeExp(tableViewDtos, "未查询到表信息|tableName=" + subTableName);
        TableViewDto tableViewDto = tableViewDtos.get(0);
        CubeTableName subCubeTableName = new CubeTableName();
        subCubeTableName.setTableName(subTableName);
        subCubeTableName.setTableComment(tableViewDto.getTableComments());
        String masterTableName  = masterCubeTableName.getTableName(),
                masterTableAlias = _getTableAlias(masterTableName, tableAliasMap),
                subTableAlias    = _getTableAlias(subTableName, tableAliasMap);
        masterCubeTableName.setTableAliasName(masterTableAlias);
        subCubeTableName.setTableAliasName(subTableAlias);
        //处理数据集的表和字段信息
        if(!cubeTableNameList.stream().anyMatch(item -> item.getTableName().equals(masterTableName))){
            cubeTableNameList.add(masterCubeTableName);
            List<CubeTableColumn> cubeTableColumns = cubeTableColumnListMap.get(masterCubeId);
            _throwNpeExp(cubeTableColumns, "未找到数据集的字段列表|cubeId=" + masterCubeId);
            cubeTableColumns.forEach(item -> {
                item.setId(null);item.setCubeDataId(null);
                item.setTableAliasName(masterTableAlias);
                item.setColumnSql(masterTableAlias + "." + item.getColumnName());
            });
            cubeTableColumnList.addAll(cubeTableColumns);
        }
        //处理物理表和字段信息
        if(!cubeTableNameList.stream().anyMatch(item -> item.getTableName().equals(subTableName))){
            cubeTableNameList.add(subCubeTableName);
            List<TableColumnDto> tableColumnDtos = dialectContext.getDialect(ds.getDsTypeSyl()).listTableColumnsByTableViewName(ds, subTableName);
            _throwNpeExp(tableColumnDtos, "未查询到物理表的字段列表|tableName=" + subTableName);
            cubeTableColumnList.addAll(_tableToCubeColumn(tableColumnDtos, subTableName, subTableAlias));
        }
        TableLinkJoin tableLinkJoin = new TableLinkJoin();
        tableLinkJoin.setSource(masterTableName);
        tableLinkJoin.setSourceId(masterTableAlias);
        tableLinkJoin.setTarget(subTableName);
        tableLinkJoin.setTargetId(subTableAlias);
        tableLinkJoin.setLevel(linkJoinItem.getJoinLevel());
        tableLinkJoin.setJoinType(JoinTypeEnum.valueOf(linkJoinItem.getJoinType()));
        List<TableLinkJoin.JoinKey> joinKeys = new ArrayList<>();
        //处理关联关系
        for(JoinColumnItem joinColumnItem : linkJoinItem.getJoinColumns()){
            Integer masterColumnId = joinColumnItem.getMasterColumnId();
            CubeTableColumn masterCubeTableColumn = cubeTableColumnMap.get(masterColumnId);
            _throwNpeExp(masterCubeTableColumn, "未找到主数据集的关联字段|columnId=" + masterColumnId);
            TableLinkJoin.JoinKey joinKey = new TableLinkJoin.JoinKey();
            joinKey.setSourceKey(masterCubeTableColumn.getColumnName());
            joinKey.setTargetKey(joinColumnItem.getJoinColumnName());
            joinKeys.add(joinKey);
        }
        tableLinkJoin.setJoinKeys(joinKeys);
        return tableLinkJoin;
    }

    private TableLinkJoin _handleTableToCube(ReportDatasource ds, MixLinkJoinItem linkJoinItem, Map<String, String> tableAliasMap,
                                             Map<Integer, List<CubeTableColumn>> cubeTableColumnListMap,
                                             Map<Integer, CubeData> cubeDataMap, Map<Integer, CubeTableColumn> cubeTableColumnMap,
                                             List<CubeTableColumn> cubeTableColumnList, List<CubeTableName> cubeTableNameList) {
        String masterTableName = linkJoinItem.getMasterTableName();
        Integer subCubeId = linkJoinItem.getSubCubeId();
        //查询主表信息
        List<TableViewDto> tableViewDtos = dialectContext.getDialect(ds.getDsTypeSyl()).listTableViewsByDatasource(ds, masterTableName);
        _throwNpeExp(tableViewDtos, "未查询到表信息|tableName=" + masterTableName);
        TableViewDto tableViewDto = tableViewDtos.get(0);
        CubeTableName masterCubeTableName = new CubeTableName();
        masterCubeTableName.setTableName(masterTableName);
        masterCubeTableName.setTableComment(tableViewDto.getTableComments());
        //处理子数据集信息
        CubeData subCubeData = cubeDataMap.get(subCubeId);
        _throwNpeExp(subCubeData, "未找到数据集信息|cubeId=" + subCubeId);
        CubeTableName subCubeTableName = JSONUtil.toList(subCubeData.getCubeTableNames(), CubeTableName.class).get(0);
        String masterTableAlias = _getTableAlias(masterTableName, tableAliasMap),
                subTableName     = subCubeTableName.getTableName(),
                subTableAlias    = _getTableAlias(subTableName, tableAliasMap);
        masterCubeTableName.setTableAliasName(masterTableAlias);
        subCubeTableName.setTableAliasName(subTableAlias);
        //处理主物理表
        if(!cubeTableNameList.stream().anyMatch(item -> item.getTableName().equals(masterTableName))) {
            cubeTableNameList.add(masterCubeTableName);
            List<TableColumnDto> tableColumnDtos = dialectContext.getDialect(ds.getDsTypeSyl()).listTableColumnsByTableViewName(ds, masterTableName);
            _throwNpeExp(tableColumnDtos, "未查询到物理表的字段列表|tableName=" + masterTableName);
            cubeTableColumnList.addAll(_tableToCubeColumn(tableColumnDtos, masterTableName, masterTableAlias));
        }
        //处理子数据集
        if(!cubeTableNameList.stream().anyMatch(item -> item.getTableName().equals(subTableName))){
            cubeTableNameList.add(subCubeTableName);
            List<CubeTableColumn> cubeTableColumns = cubeTableColumnListMap.get(subCubeId);
            _throwNpeExp(cubeTableColumns, "未找到数据集的字段列表|cubeId=" + subCubeId);
            cubeTableColumns.forEach(item -> {
                item.setId(null);item.setCubeDataId(null);
                item.setTableAliasName(subTableAlias);
                item.setColumnSql(subTableAlias + "." + item.getColumnName());
            });
            cubeTableColumnList.addAll(cubeTableColumns);
        }
        TableLinkJoin tableLinkJoin = new TableLinkJoin();
        tableLinkJoin.setSource(masterTableName);
        tableLinkJoin.setSourceId(masterTableAlias);
        tableLinkJoin.setTarget(subTableName);
        tableLinkJoin.setTargetId(subTableAlias);
        tableLinkJoin.setLevel(linkJoinItem.getJoinLevel());
        tableLinkJoin.setJoinType(JoinTypeEnum.valueOf(linkJoinItem.getJoinType()));
        List<TableLinkJoin.JoinKey> joinKeys = new ArrayList<>();
        //处理关联关系
        for(JoinColumnItem joinColumnItem : linkJoinItem.getJoinColumns()){
            Integer subJoinColumnId = joinColumnItem.getJoinColumnId();
            CubeTableColumn subCubeTableColumn = cubeTableColumnMap.get(subJoinColumnId);
            _throwNpeExp(subCubeTableColumn, "未找到数据集的关联字段|columnId=" + subJoinColumnId);
            TableLinkJoin.JoinKey joinKey = new TableLinkJoin.JoinKey();
            joinKey.setSourceKey(joinColumnItem.getJoinColumnName());
            joinKey.setTargetKey(subCubeTableColumn.getColumnName());
            joinKeys.add(joinKey);
        }
        tableLinkJoin.setJoinKeys(joinKeys);
        return tableLinkJoin;
    }

    private String _getTableAlias(String tableName, Map<String, String> tableAliasMap) {
        String tableAlias = tableAliasMap.get(tableName);
        if(StrUtil.isBlank(tableAlias)) {
            tableAlias = RandomUtil.randomString("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ", 5);
            tableAliasMap.put(tableName, tableAlias);
        }
        return tableAlias;
    }

    private void _throwNpeExp(Object obj, String message) {
        if(obj == null) {
            throw new ServiceException(message);
        }
    }

    private void _throwNpeExp(List<?> list, String message) {
        if(list == null || list.isEmpty()) {
            throw new ServiceException(message);
        }
    }

    private List<CubeTableColumn> _tableToCubeColumn(List<TableColumnDto> reportColumnList, String tableName, String tableAlias){
        List<CubeTableColumn> cubeTableColumnList = new ArrayList<CubeTableColumn>();
        int sortNum = 0;
        for (TableColumnDto columnDto : reportColumnList) {
            CubeTableColumn cubeTableColumn = new CubeTableColumn();
            cubeTableColumn.setTableName(tableName);
            cubeTableColumn.setAggregationType(AggregationTypeEnum.SUM);
            cubeTableColumn.setColumnAttr(ColumnAttrEnum.NORMAL);
            cubeTableColumn.setColumnComment(columnDto.getColumnComments());
            cubeTableColumn.setColumnName(columnDto.getColumnName());
            cubeTableColumn.setColumnTitle(StrUtil.isEmpty(columnDto.getColumnComments()) ? columnDto.getColumnName() : columnDto.getColumnComments());
            cubeTableColumn.setViewDataType(columnDto.getColumnType());
            cubeTableColumn.setOldViewDataType(columnDto.getColumnType());
            cubeTableColumn.setDimensionType(columnDto.getColumnType() == ViewColumnTypeEnum.NUMBER ? DimensionTypeEnum.ZHIBIAO : DimensionTypeEnum.WEIDU);
            cubeTableColumn.setHiddenStatus(0);
            cubeTableColumn.setSortNum(sortNum);
            cubeTableColumn.setTableAliasName(tableAlias);
            cubeTableColumn.setColumnSql(tableAlias + "." + columnDto.getColumnName());
            cubeTableColumn.setNumberDataFormat(NumberFormatEnum.AUTO);
            cubeTableColumn.setTimeDataFormat(DateFormatEnum.yyyy_MM_ddhh$mm$ss);
            cubeTableColumnList.add(cubeTableColumn);
            sortNum += 100;
        }
        return cubeTableColumnList;
    }
}
