/**
 * 
 */
package org.springblade.cube.utils;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.springblade.core.constant.TokenConstant;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.support.Kv;
import org.springblade.core.tool.utils.Func;
import org.springblade.datasource.enums.DateFormatEnum;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

/**
 * @author wookvn
 *
 */
public class CubeConstants {
	
	public final static int DEFAULT_PREVIEW_DATA_SIZE = 100;
	
	public final static String DEFAULT_SHOW_CHAR = "'-'";

	public final static String COLUMN_NAME_PREFIX = "COL_";
	
	public final static String COLUMN_ALIAS_NAME_PREFIX = "COL_ALIAS_";
	
	public final static String COLUMN_USER_ID = "JHBI_USER_ID";
	
	public final static String COLUMN_PRIMARY_KEY = "ID";
	
	public final static String[] SUPPORTS_FILE = {"xls", "xlsx", "csv"};
	
	public final static int DEFAULT_READ_EXCEL_SIZE = 100;
	
	public final static String INSERT_INTO_SQL_FORMAT = "insert into %s %s values %s";
	
	public final static String SELECT_SQL_FORMAT = "select %s from %s where %s %s";
	
	public final static Pattern COLUMN_FORMULA_PATTERN = Pattern.compile("\\[.+?\\]");
	
	public final static String[] AGGREGATION_FUNCTIONS = {"sum(", "count(", "min(", "max(", "avg(", "stddev(", "median("};

	public final static String MODEL_MERGE_CUBE_WORKSPACE_NAME = "聚合表单";
	public final static String MODEL_SINGLE_CUBE_WORKSPACE_NAME = "单表表单";

	/**
	 * 支持数字格式
	 */
	public static String NUMBER_FORAMT_PATTERNS = "￥|¥|$|%|,|\\s*";
	
	/**
	 * 支持的日期格式
	 */
	public static String[] DATE_FORMAT_PATTERNS = {
            "yyyy-MM-dd", "yyyy-MM-dd HH:mm:ss", "yyyy-MM-dd HH:mm", "yyyy-MM", 
            "yyyy/MM/dd", "yyyy/MM/dd HH:mm:ss", "yyyy/MM/dd HH:mm", "yyyy/MM",
            "yyyy.MM.dd", "yyyy.MM.dd HH:mm:ss", "yyyy.MM.dd HH:mm", "yyyy.MM",
            "yyyy年MM月dd日", "yyyy年MM月dd日 HH点mm分ss秒", "yyyy年MM月dd日 HH点mm分", "yyyy年MM月",
            "MM-dd", "MM/dd", "MM月dd日"};
	
	/**
	 * 创建或更新视图语句
	 */
	public final static String CREATE_REPLACE_VIEW_SQL_FORMAT = "create or replace view %s AS %s";
//	public final static String CREATE_REPLACE_VIEW_SQL_FORMAT = "create view %s AS %s";

	/**
	 * 数据集全局参数变量
	 */
	public final static String CUBE_GLOBAL_SYS_USER_ID_PARAM_KEY = "jhbiSysUser";
	public final static String CUBE_GLOBAL_SYS_CURRENT_TIME_PARAM_KEY = "jhbiSysCurrentTime";
	public final static String CUBE_GLOBAL_SYS_USER_ACCOUNT_PARAM_KEY = "jhbiSysUserName";
	public final static String CUBE_GLOBAL_SYS_USER_REAL_NAME_PARAM_KEY = "jhbiSysUserRealName";
	public final static String CUBE_GLOBAL_SYS_USER_MOBILE_PARAM_KEY = "jhbiSysUserMobile";
	public final static String CUBE_GLOBAL_SYS_USER_DEPT_PARAM_KEY = "jhbiSysUserDept";
	public final static String CUBE_GLOBAL_SYS_USER_THIRD_DEPT_PARAM_KEY = "jhbiSysThirdUserDept";
	public final static String CUBE_GLOBAL_SYS_USER_THIRD_POST_PARAM_KEY = "jhbiSysThirdUserPost";
	public final static String CUBE_GLOBAL_SYS_USER_THIRD_PARAM_KEY = "jhbiSysThirdUser";
	
	public static Map<String, List<Object>> mergeSysUserParams(BladeUser bladeUser, Map<String, List<Object>> extraParams){
		if(bladeUser != null) {
			extraParams.put(CUBE_GLOBAL_SYS_USER_ID_PARAM_KEY, ListUtil.toList(bladeUser.getUserId()));
			extraParams.put(CUBE_GLOBAL_SYS_USER_ACCOUNT_PARAM_KEY, ListUtil.toList(bladeUser.getAccount()));
			if(StrUtil.isNotBlank(bladeUser.getMobile())) {
				extraParams.put(CUBE_GLOBAL_SYS_USER_MOBILE_PARAM_KEY, ListUtil.toList(bladeUser.getMobile()));
			}
			if(StrUtil.isNotBlank(bladeUser.getNickName())) {
				extraParams.put(CUBE_GLOBAL_SYS_USER_REAL_NAME_PARAM_KEY, ListUtil.toList(bladeUser.getNickName()));
			}
			if(StrUtil.isNotBlank(bladeUser.getDeptId())) {
				extraParams.put(CUBE_GLOBAL_SYS_USER_DEPT_PARAM_KEY, Stream.of(bladeUser.getDeptId().split(",")).map(Long::valueOf).collect(Collectors.toList()));
			}
			Kv detail = Optional.ofNullable(bladeUser.getDetail()).orElse(Kv.create());
			String thirdUserId = (String)detail.get(TokenConstant.THIRD_USER_ID);
			String thirdPostId = (String)detail.get(TokenConstant.THIRD_POST_ID);
			String thirdDeptId = (String)detail.get(TokenConstant.THIRD_DEPT_ID);
			if(Func.isNotBlank(thirdDeptId)) {
				extraParams.put(CUBE_GLOBAL_SYS_USER_THIRD_DEPT_PARAM_KEY, Stream.of(thirdDeptId.split(",")).collect(Collectors.toList()));
			}
			if(Func.isNotBlank(thirdPostId)){
				extraParams.put(CUBE_GLOBAL_SYS_USER_THIRD_POST_PARAM_KEY, Stream.of(thirdPostId.split(",")).collect(Collectors.toList()));
			}
			if(Func.isNotBlank(thirdUserId)) {
				extraParams.put(CUBE_GLOBAL_SYS_USER_THIRD_PARAM_KEY, ListUtil.toList(thirdUserId));
			}
		}
		return extraParams;
	}
	
	public static Map<String, List<Object>> mergeSysCurrentTimeParams(DateFormatEnum dateFormat, Map<String, List<Object>> extraParams){
		if(dateFormat != null) {
			extraParams.put(CUBE_GLOBAL_SYS_CURRENT_TIME_PARAM_KEY, ListUtil.toList(dateFormat == DateFormatEnum.timestamp ? System.currentTimeMillis() : DateUtil.format(LocalDateTime.now(), dateFormat.getFormat())));
		}
		return extraParams;
	}
	
	public static Map<String, List<Object>> mergeAllSysParams(BladeUser bladeUser, DateFormatEnum dateFormat, Map<String, List<Object>> extraParams){
		return mergeSysCurrentTimeParams(dateFormat, mergeSysUserParams(bladeUser, extraParams));
	}
}
