package org.springblade.microapp.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.api.R;
import org.springblade.microapp.entity.MicroAppShortcutMenu;
import org.springblade.microapp.service.MicroAppShortcutMenuService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.List;

@RestController
@RequestMapping("/micro-app/shortcut-menu")
@Api(value = "应用快捷菜单接口", tags = "应用快捷菜单相关接口")
public class MicroAppShortcutMenuController extends BladeController {

    @Autowired
    private MicroAppShortcutMenuService microAppShortcutMenuService;

    /**
     * 分页查询
     */
    @GetMapping("/list")
    @ApiOperation("查询快捷菜单")
    public R<IPage<MicroAppShortcutMenu>> menuList(Query query){

        Page<MicroAppShortcutMenu> page = microAppShortcutMenuService.page(new Page<>(query.getCurrent(), query.getSize()));

        return R.data(page);
    }

    /**
     * 保存或更新快捷菜单
     */
    @PostMapping("/save-update")
    @ApiOperation("保存或更新快捷菜单")
    public R<Boolean> saveOrUpdate(@RequestBody MicroAppShortcutMenu appMenu){
        BladeUser user = getUser();
        appMenu.setCreateUserId(user.getUserId());
        appMenu.setCreateUser(user.getNickName());
        appMenu.setUpdateTime(LocalDateTime.now());
        return R.data(microAppShortcutMenuService.saveOrUpdate(appMenu));
    }

    /**
     * 删除快捷菜单
     */
    @PostMapping("/remove")
    @ApiOperation("删除快捷菜单")
    public R<Boolean> deleteMenu(List<Integer> ids){

        return R.data(microAppShortcutMenuService.removeByIds(ids));
    }
}
