/**
 * 
 */
package org.springblade.microapp.service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.microapp.dto.MicroAppPermDoo;
import org.springblade.microapp.dto.MicroAppPermDto;
import org.springblade.microapp.entity.MicroApp;
import org.springblade.microapp.entity.MicroAppDataPermission;
import org.springblade.microapp.mapper.MicroAppDataPermissionMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

/**
 * @author wookvn
 *
 */
@Service
public class MicroAppDataPermissionService extends ServiceImpl<MicroAppDataPermissionMapper, MicroAppDataPermission> {

	@Autowired
	private MicroAppService microAppService;
	
	/**
	 * 
	 * @param appId
	 * @return
	 */
	public List<JSONObject> listDataPerm(Integer appId) {
		return baseMapper.selectDataPermList(appId);
	}

	/**
	 * 
	 * @param mircoAppPermDto
	 * @return
	 */
	@Transactional(rollbackFor = Exception.class)
	public boolean doSubmit(MicroAppPermDto mircoAppPermDto, BladeUser bladeUser) {
		// 删除权限组下的应用
		baseMapper.delete(Wrappers.<MicroAppDataPermission>lambdaQuery().eq(MicroAppDataPermission::getDataPermId,
				mircoAppPermDto.getDataPermId()));
		// 批量添加
		if (CollectionUtil.isNotEmpty(mircoAppPermDto.getAppIdList())) {
			//根据ID更新应用的公开状态为权限组状态
			microAppService.update(Wrappers.<MicroApp>lambdaUpdate().set(MicroApp::getPublicStatus, 2).in(MicroApp::getAppId, mircoAppPermDto.getAppIdList()));
			List<MicroAppDataPermission> dataPermList = new ArrayList<>(mircoAppPermDto.getAppIdList().size());
			mircoAppPermDto.getAppIdList().forEach(appId -> {
				MicroAppDataPermission dataPerm = new MicroAppDataPermission();
				dataPerm.setAppId(appId);
				dataPerm.setDataPermId(mircoAppPermDto.getDataPermId());
				dataPerm.setCreateTime(new Date());
				dataPerm.setCreateUserId(bladeUser.getUserId());
				dataPerm.setCreateBy(bladeUser.getNickName());
				dataPermList.add(dataPerm);
			});
			saveBatch(dataPermList);
		}
		return true;
	}

	/**
	 * 
	 * @param mircoAppPermDoo
	 * @return
	 */
	@Transactional(rollbackFor = Exception.class)
	public boolean doPermform(MicroAppPermDoo mircoAppPermDoo, BladeUser bladeUser) {
		// 删除应用相关的权限组列表
		baseMapper.delete(Wrappers.<MicroAppDataPermission>lambdaQuery().eq(MicroAppDataPermission::getAppId,
				mircoAppPermDoo.getAppId()));
		// 批量添加
		if (CollectionUtil.isNotEmpty(mircoAppPermDoo.getDataPermIdList())) {
			//根据ID更新应用的公开状态为权限组状态
			microAppService.update(Wrappers.<MicroApp>lambdaUpdate().set(MicroApp::getPublicStatus, 2).eq(MicroApp::getAppId, mircoAppPermDoo.getAppId()));
			List<MicroAppDataPermission> dataPermList = new ArrayList<>(mircoAppPermDoo.getDataPermIdList().size());
			mircoAppPermDoo.getDataPermIdList().forEach(dataPermId -> {
				MicroAppDataPermission dataPerm = new MicroAppDataPermission();
				dataPerm.setAppId(mircoAppPermDoo.getAppId());
				dataPerm.setDataPermId(dataPermId);
				dataPerm.setCreateTime(new Date());
				dataPerm.setCreateUserId(bladeUser.getUserId());
				dataPerm.setCreateBy(bladeUser.getNickName());
				dataPermList.add(dataPerm);
			});
			saveBatch(dataPermList);
		}
		return true;
	}
}
