/**
 * 
 */
package org.springblade.microapp.service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.springblade.core.secure.BladeUser;
import org.springblade.microapp.entity.MicroApp;
import org.springblade.microapp.entity.MicroAppDataPermission;
import org.springblade.microapp.entity.MicroAppMenu;
import org.springblade.microapp.mapper.MicroAppMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cn.hutool.core.collection.CollectionUtil;

/**
 * @author wookvn
 *
 */
@Service
public class MicroAppService extends ServiceImpl<MicroAppMapper, MicroApp> {

	@Autowired
	private MicroAppMenuService appMenuService;
	
	@Autowired
	private MicroAppDataPermissionService microAppDataPermissionService;
	
	/**
	 * 
	 * @param microApp
	 * @return
	 */
	@Transactional(rollbackFor = Exception.class)
	public Integer doSaveOrUpdate(MicroApp microApp, BladeUser bladeUser) {
		boolean ret = this.saveOrUpdate(microApp);
		if(ret && CollectionUtil.isNotEmpty(microApp.getDataPermIdList())) {
			List<MicroAppDataPermission> dataPermList = new ArrayList<>();
			microApp.getDataPermIdList().forEach(dataPermId -> {
				MicroAppDataPermission dataPerm = new MicroAppDataPermission();
				dataPerm.setAppId(microApp.getAppId());
				dataPerm.setDataPermId(dataPermId);
				dataPerm.setCreateTime(new Date());
				dataPerm.setCreateUserId(bladeUser.getUserId());
				dataPerm.setCreateBy(bladeUser.getNickName());
				dataPermList.add(dataPerm);
			});
			microAppDataPermissionService.saveBatch(dataPermList);
		}
		return microApp.getAppId();
	}
	
	
	/**
	 * 
	 * @param appId
	 * @return
	 */
	@Transactional(rollbackFor = Exception.class)
	public boolean doRemove(List<Integer> appIds) {
		appIds.forEach(appId -> {
			if(baseMapper.deleteById(appId) > 0) {
				appMenuService.remove(Wrappers.<MicroAppMenu>lambdaQuery().eq(MicroAppMenu::getAppId, appId));
				microAppDataPermissionService.remove(Wrappers.<MicroAppDataPermission>lambdaQuery().eq(MicroAppDataPermission::getAppId, appId));
			}
		});
		return true;
	}
	
}
