/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.report.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.report.entity.IndexCategory;
import org.springblade.report.service.IIndexCategoryService;
import org.springblade.report.vo.IndexCategoryVO;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * 报表指标分类 控制器
 *
 * @author BladeX
 * @since 2021-04-15
 */
@RestController
@AllArgsConstructor
@RequestMapping("/indexcategory")
@Api(value = "报表指标分类", tags = "报表指标分类接口 IndexCategoryController")
public class IndexCategoryController extends BladeController {

	private final IIndexCategoryService indexCategoryService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入indexCategory")
	public R<IndexCategory> detail(IndexCategory indexCategory) {
		IndexCategory detail = indexCategoryService.getOne(Condition.getQueryWrapper(indexCategory));
		return R.data(detail);
	}

	/**
	 * 分页 报表指标分类
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "列表", notes = "传入indexCategory")
	public R<List<IndexCategory>> list(IndexCategory indexCategory) {
		QueryWrapper queryWrapper = Condition.getQueryWrapper(indexCategory);
		queryWrapper.orderByAsc("order_index");
		queryWrapper.orderByDesc("create_time");
		List<IndexCategory> list = indexCategoryService.list(queryWrapper);
		return R.data(list);
	}

	/**
	 * 自定义分页 报表指标分类
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入indexCategory")
	public R<IPage<IndexCategoryVO>> page(IndexCategoryVO indexCategory, Query query) {
		IPage<IndexCategoryVO> pages = indexCategoryService.selectIndexCategoryPage(Condition.getPage(query), indexCategory);
		return R.data(pages);
	}

	/**
	 * 新增 报表指标分类
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入indexCategory")
	public R save(@Valid @RequestBody IndexCategory indexCategory) {
		return R.status(indexCategoryService.save(indexCategory));
	}

	/**
	 * 修改 报表指标分类
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入indexCategory")
	public R update(@Valid @RequestBody IndexCategory indexCategory) {
		return R.status(indexCategoryService.updateById(indexCategory));
	}

	/**
	 * 新增或修改 报表指标分类
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入indexCategory")
	public R submit(@Valid @RequestBody IndexCategory indexCategory) {
		return R.status(indexCategoryService.saveOrUpdate(indexCategory));
	}


	/**
	 * 删除 报表指标分类
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(indexCategoryService.removeByIds(Func.toLongList(ids)));
	}


}
