/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.report.controller;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Query;
import org.springblade.core.redis.cache.BladeRedis;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.api.R;
import org.springblade.datasource.context.DialectContext;
import org.springblade.datasource.dialect.SqlServerGmetryDialect;
import org.springblade.datasource.model.TableColumnDto;
import org.springblade.datasource.model.TableViewDto;
import org.springblade.report.dto.oracle.ReportViewDTO;
import org.springblade.report.dto.response.TableCategoryDTO;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.entity.TableCategory;
import org.springblade.report.entity.TableCategoryRelation;
import org.springblade.report.service.IReportBaseService;
import org.springblade.report.service.IReportDatasourceService;
import org.springblade.report.service.ITableCategoryRelationService;
import org.springblade.report.service.ITableCategoryService;
import org.springblade.report.service.normal.ReportDatasourcePermissionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

/**
 * 产品报表基础信息 控制器
 *
 * @author BladeX
 * @since 2021-03-10
 */
@RestController
@RequestMapping("report/baseInfo")
@Api(value = "数据库表基础信息", tags = "数据库表基础信息 ReportBaseController")
public class ReportBaseController extends BladeController {

//	@Resource
//	private IReportBaseInfoService reportBaseInfoService;

    @Resource
    private IReportBaseService reportBaseService;

    @Resource
    private IReportDatasourceService reportDatasourceService;

//	@Autowired
//	private IReportDatasourcePermService reportDatasourcePermService;
//
//	@Autowired
//	private IReportDatasourceTablePermService reportDatasourceTablePermService;
//	@Autowired
//	private DataPermClient dataPermClient;

    @Resource
    private BladeRedis bladeRedis;

    @Autowired
    private DialectContext dialectContext;

    @Autowired
    private ReportDatasourcePermissionService reportDatasourcePermissionService;

    @Autowired
    private ITableCategoryRelationService tableCategoryRelationService;

    @Autowired
    private ITableCategoryService tableCategoryService;

    /**
     * 获取报表视图
     */
    @GetMapping("/listViews")
    @ApiOperationSupport(order = 1)
    @ApiOperation(value = "获取报表视图", notes = "")
    @Deprecated
    public R<List<ReportViewDTO>> list() {
        List<ReportViewDTO> list = reportBaseService.selectReportViewList();
        return R.data(list);
    }

    /**
     * 获取报表视图
     */
    @GetMapping("/listViewsByDataSource")
    @ApiOperationSupport(order = 1)
    @ApiOperation(value = "根据数据源获取报表视图", notes = "")
//	@Cacheable(cacheNames = "baseInfo", key = "#datasourceId")
    public R<IPage<TableCategoryDTO>> list(HttpServletRequest request,
                                           @ApiParam("数据源Id") @RequestParam Integer datasourceId,
                                           @ApiParam("分类id") @RequestParam(required = false) Integer classifyId,
                                           @ApiParam("表名") @RequestParam(required = false) String tableName,
                                           @ApiParam("分页参数") Query query) throws ClassNotFoundException, SQLException {

        return R.data(getTableView(datasourceId, classifyId, tableName, query, getUser()));
    }


    /**
     * 获取报表视图的字段信息
     */
    @GetMapping("/listColumnsByDatasource")
    @ApiOperationSupport(order = 2)
    @ApiOperation(value = "根据数据源获取报表视图的字段信息", notes = "")
//	@Cacheable(cacheNames = "baseInfo", key = "#reportViewDTO")
    public R<List<TableColumnDto>> listColumnsByDatasource(ReportViewDTO reportViewDTO, HttpServletRequest request) throws ClassNotFoundException, SQLException {
        String viewName = reportViewDTO.getTableViewName();
        if (StrUtil.isAllBlank(viewName)) {
            return R.fail("视图名称不可以为空");
        }
        ReportDatasource datasource = reportDatasourceService.getById(reportViewDTO.getDatasourceId());
        if (null == datasource) {
            return R.fail("数据源id错误");
        }
//		List<ReportColumnDTO> list = reportBaseInfoService.selectReportColumnList(datasource, viewName);
        List<TableColumnDto> list = dialectContext.getDialect(datasource.getDsTypeSyl()).listTableColumnsByTableViewName(datasource, viewName);
        return R.data(list);
    }


    @GetMapping("removeBaseInfoCache")
    @ApiOperation(value = "清除baseInfo缓存", notes = "")
    public R removeBaseInfoCache(Integer datasourceId) {
        if (null != datasourceId) {
            bladeRedis.del(bladeRedis.keys("baseInfo::" + datasourceId));
        } else {
            bladeRedis.del(bladeRedis.keys("baseInfo*"));
        }
        return R.success("清除缓存完成");
    }

    public IPage<TableCategoryDTO> getTableView(Integer datasourceId, Integer classifyId, String tableName, Query query, BladeUser user) {

        IPage<TableCategoryDTO> page = new Page<>();
        page.setRecords(new ArrayList<>());
        if (query.getSize() != null) {
            page.setSize(query.getSize());
        } else {
            page.setSize(Integer.MAX_VALUE);
        }
        if (query.getCurrent() != null) {
            page.setCurrent(query.getCurrent());
        } else {
            page.setCurrent(1);
        }
        page.setTotal(0);
        page.setPages(0);

        ReportDatasource datasource = reportDatasourceService.getById(datasourceId);
        if (null == datasource) {
            return page;
        }
        List<TableViewDto> list = dialectContext.getDialect(datasource.getDsTypeSyl()).listTableViewsByDatasource(datasource);
        if (classifyId == null && StrUtil.isNotEmpty(tableName)) {
//			list = dialectContext.getDialect(datasource.getDsTypeSyl()).listTableViewsByDatasource(datasource,tableName);
            List<TableViewDto> collect1 = list.stream().filter(tableViewDto -> tableViewDto.getTableViewName().contains(tableName)).collect(Collectors.toList());
            List<TableViewDto> collect = list.stream().filter(tableViewDto -> tableViewDto.getTableComments() != null && tableViewDto.getTableComments().contains(tableName)).collect(Collectors.toList());
            if (CollectionUtil.isEmpty(collect1)) {
                list = collect;
            } else {
                list = collect1;
                if (CollectionUtil.isNotEmpty(collect)) {
                    list.addAll(collect);
                }
            }
        }
        if (CollectionUtil.isNotEmpty(list)) { //处理权限组
            List<TableViewDto> finalList = list;
            reportDatasourcePermissionService.dealTablePermission(user.getUserId(), datasourceId,
                    permTableNameSet -> {
                        if (permTableNameSet.isEmpty()) {
                            finalList.clear();
                        } else {
                            finalList.removeIf(it -> !permTableNameSet.contains(it.getTableViewName()));
                        }
                    });
        }

        if ((classifyId == null || classifyId < 1)) {
            if (CollectionUtil.isEmpty(list)) {
                return null;
            }
            // 对查询结果进行分页
            List<TableViewDto> subList = list;
            if (query.getSize() == null || query.getSize() > list.size()) {
                page.setSize(list.size());
            }
            if (query.getSize() != null && query.getSize() < list.size()) {
                subList = list.stream().skip((query.getCurrent() - 1) * query.getSize()).limit(query.getSize()).
                        collect(Collectors.toList());
            }
            List<TableCategoryDTO> tableCategorys = new ArrayList<>();
            List<TableCategoryDTO> tableCategoryRelations = tableCategoryRelationService.getInfoByDatasourceId(datasourceId);
            for (TableViewDto tableViewDto : subList) {
                TableCategoryDTO tableCategoryDTO = BeanUtil.toBean(tableViewDto, TableCategoryDTO.class);
                boolean bool = true;
                for (TableCategoryDTO tableCategoryRelation : tableCategoryRelations) {
                    if (tableCategoryRelation.getTableViewName().equals(tableViewDto.getTableViewName())) {
                        tableCategorys.add(tableCategoryRelation);
                        bool = false;
                        break;
                    }
                }
                if (bool) {
                    tableCategorys.add(tableCategoryDTO);
                }
            }
            page.setRecords(tableCategorys);
            page.setTotal(list.size());
            page.setPages(subList.size() % page.getSize());
        }
        // 如果分类id不为空且表名字为空
        if (classifyId != null) {
            QueryWrapper queryWrapper = new QueryWrapper();
            queryWrapper.eq("category_id", classifyId);
            if (StrUtil.isNotEmpty(tableName)) {
                queryWrapper.like("table_view_name", tableName);
            }
            queryWrapper.orderByDesc("order_index");
            Page<TableCategoryRelation> tableCategoryRelations = tableCategoryRelationService.page(new Page<>(page.getCurrent(), page.getSize()), queryWrapper);

            //过滤需要权限访问的表
            TableCategory tableCategory = tableCategoryService.getById(classifyId);

            List<TableCategoryDTO> data = new ArrayList<>();
            if (null != tableCategoryRelations && CollectionUtil.isNotEmpty(tableCategoryRelations.getRecords())) {
                tableCategoryRelations.getRecords().forEach(tableCategoryRelation -> {
                    TableCategoryDTO dto = new TableCategoryDTO();
                    if (null != tableCategory) {
                        dto.setCategoryName(tableCategory.getCategoryName());
                        dto.setDatasourceId(tableCategory.getDatasourceId());
                    }
                    dto.setCategoryId(tableCategoryRelation.getCategoryId());
                    dto.setOrderIndex(tableCategoryRelation.getOrderIndex());
                    dto.setTableViewName(tableCategoryRelation.getTableViewName());
                    dto.setTableComments(tableCategoryRelation.getTableComments());

                    data.add(dto);
                });
            }
            page.setTotal(tableCategoryRelations.getTotal());
            page.setRecords(data);
            page.setPages(tableCategoryRelations.getPages());
        }
        return page;
    }
}
