/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.report.entity;

import java.io.Serializable;
import java.math.BigDecimal;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

/**
 * 产品订购月报视图实体类
 *
 * @author BladeX
 * @since 2021-03-10
 */
@Data
@TableName("V_FIN_ORDER_MONTH")
@ApiModel(value = "OrderMonth对象", description = "产品订购月报视图")
public class OrderMonth implements Serializable {

	private static final long serialVersionUID = 1L;

	@TableField("THEMONTH")
	private BigDecimal themonth;
	/**
	 * 产品ID
	 */
	@ApiModelProperty(value = "产品ID")
	@TableField("SPID")
	private String spid;
	/**
	 * 产品名称-原始
	 */
	@ApiModelProperty(value = "产品名称-原始")
	@TableField("SPNAME")
	private String spname;
	@TableField("SPSHORTNAME")
	private String spshortname;
	/**
	 * 供应商ID
	 */
	@ApiModelProperty(value = "供应商ID")
	@TableField("CPID")
	private String cpid;
	/**
	 * 渠道ID
	 */
	@ApiModelProperty(value = "渠道ID")
	@TableField("CHANNEL")
	private String channel;
	/**
	 * 渠道名称
	 */
	@ApiModelProperty(value = "渠道名称")
	@TableField("CHANNELNAME")
	private String channelname;
	/**
	 * 渠道类别
	 */
	@ApiModelProperty(value = "渠道类别")
	@TableField("BELONGTO")
	private String belongto;
	/**
	 * 省份
	 */
	@ApiModelProperty(value = "省份")
	@TableField("PROVINCE")
	private String province;
	/**
	 * 城市
	 */
	@ApiModelProperty(value = "城市")
	@TableField("AREA")
	private String area;
	/**
	 * 访问次数
	 */
	@ApiModelProperty(value = "访问次数")
	@TableField("PV")
	private BigDecimal pv;
	/**
	 * 访问用户数
	 */
	@ApiModelProperty(value = "访问用户数")
	@TableField("UV")
	private BigDecimal uv;
	/**
	 * 有效用户
	 */
	@ApiModelProperty(value = "有效用户")
	@TableField("USERSUM")
	private BigDecimal usersum;
	/**
	 * 新增用户数
	 */
	@ApiModelProperty(value = "新增用户数")
	@TableField("ORDERNUM")
	private BigDecimal ordernum;
	@TableField("CANCELNUM")
	private BigDecimal cancelnum;
	/**
	 * 当月新增退订数
	 */
	@ApiModelProperty(value = "当月新增退订数")
	@TableField("MONTHCALCELNUM")
	private BigDecimal monthcalcelnum;
	@TableField("VALIDNUM")
	private BigDecimal validnum;
	/**
	 * 活跃用户
	 */
	@ApiModelProperty(value = "活跃用户")
	@TableField("ACTIVENUM")
	private BigDecimal activenum;
	/**
	 * 新增用户活跃数
	 */
	@ApiModelProperty(value = "新增用户活跃数")
	@TableField("ORDERACTIVENUM")
	private BigDecimal orderactivenum;
	/**
	 * 流量（KB）
	 */
	@ApiModelProperty(value = "流量（KB）")
	@TableField("DOWNBYTE")
	private BigDecimal downbyte;
	/**
	 * 新增用户流量（KB）
	 */
	@ApiModelProperty(value = "新增用户流量（KB）")
	@TableField("ORDERDOWNBYTE")
	private BigDecimal orderdownbyte;
	/**
	 * 新增用户中新用户数
	 */
	@ApiModelProperty(value = "新增用户中新用户数")
	@TableField("ORDERNUM_NEW")
	private BigDecimal ordernumNew;
	/**
	 * 产品大类
	 */
	@ApiModelProperty(value = "产品大类")
	@TableField("CATEGORYNAME")
	private String categoryname;
	@TableField("CPID2")
	private String cpid2;
	/**
	 * 供应商名称
	 */
	@ApiModelProperty(value = "供应商名称")
	@TableField("CPNAME")
	private String cpname;
	/**
	 * 产品名称
	 */
	@ApiModelProperty(value = "产品名称")
	@TableField("NEWSPNAME")
	private String newspname;


}
