/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.report.service;

import org.springblade.report.dto.oracle.ReportColumnDTO;
import org.springblade.report.dto.oracle.ReportViewDTO;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * 产品订购月报视图 服务类
 *
 * @author BladeX
 * @since 2021-03-10
 */
public interface IReportBaseService {

	/**
	 * 查询报表视图
	 *
	 * @return
	 */
	List<ReportViewDTO> selectReportViewList();

	/**
	 * 查询视图字段列表
	 *
	 * @param viewName 视图名称
	 * @return
	 */
	List<ReportColumnDTO> selectReportColumnList(String viewName);

	List<Object> selectRowConvertCol();

	List<LinkedHashMap<String, String>> selectCpNameList();

	List<LinkedHashMap<String, Object>> getReportResult(String sql);

	/**
	 * 查询有哪些行转列维度
	 *
	 * @param fieldSql 字段列表
	 * @param whereSql 查询条件
	 * @return
	 */
	List<LinkedHashMap<String, Object>> selectRowToColList(String tableViewName, String fieldSql, String whereSql);


	/**
	 * 动态查询报表数据
	 *
	 * @param rowFieldsSql 行维度字段列表
	 * @param sql          行转列语句
	 * @return
	 */
	List<LinkedHashMap<String, Object>> getReportData(String tableViewName, String rowFieldsSql, String sql, String whereSql, String orderBySql);

	List<LinkedHashMap<String, Object>> getReportDataByCondition(String poolName, String tableViewName, String selectSql, String whereSql, String orderBySql,
																 String groupBySql, String limitSql);
}
