/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.report.service.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import org.springblade.report.dto.oracle.ReportColumnDTO;
import org.springblade.report.dto.oracle.ReportViewDTO;
import org.springblade.report.mapper.ReportDaoMapper;
import org.springblade.report.service.IReportBaseService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.LinkedHashMap;
import java.util.List;

/**
 * 产品订购月报视图 服务实现类
 *
 * @author BladeX
 * @since 2021-03-10
 */
@Service
@DS("oracle143")
public class ReportBaseServiceImpl implements IReportBaseService {

	@Resource
	private ReportDaoMapper reportDaoMapper;

	@Override
	public List<ReportViewDTO> selectReportViewList() {
		return reportDaoMapper.selectReportViewList();
	}

	@Override
	public List<ReportColumnDTO> selectReportColumnList(String viewName) {
		return reportDaoMapper.selectReportColumnList(viewName);
	}

	@Override
	public List<Object> selectRowConvertCol() {
		return reportDaoMapper.selectRowConvertCol();
	}

	@Override
	public List<LinkedHashMap<String, String>> selectCpNameList() {
		return reportDaoMapper.selectCategoryAndCpNameList();
	}

	@Override
	public List<LinkedHashMap<String, Object>> getReportResult(String sql) {
		return reportDaoMapper.getReportResult(sql);
	}

	@Override
	public List<LinkedHashMap<String, Object>> getReportData(String tableViewName, String rowFieldsSql, String sql, String whereSql, String orderBySql) {
		return reportDaoMapper.getReportData(tableViewName, rowFieldsSql, sql, whereSql, orderBySql);
	}

	@DS("#poolName")
	@Override
	public List<LinkedHashMap<String, Object>> getReportDataByCondition(String poolName, String tableViewName, String selectSql, String whereSql, String orderBySql,
																		String groupBySql, String limitSql) {
		return reportDaoMapper.getReportDataByCondition(tableViewName, selectSql, whereSql, orderBySql, groupBySql, limitSql);
	}

	@Override
	public List<LinkedHashMap<String, Object>> selectRowToColList(String tableViewName, String fieldSql, String whereSql) {
		return reportDaoMapper.selectRowToColList(tableViewName, fieldSql, whereSql);
	}
}
