/**
 *
 */
package org.springblade.resouces;

import org.springblade.bi.client.CubeDataClient;
import org.springblade.bi.client.form.AggCubeCreateForm;
import org.springblade.bi.client.form.ChangeCubeDataStatusParams;
import org.springblade.bi.client.form.CubeBrowserSqlForm;
import org.springblade.bi.client.model.CubeBrowserSqlModel;
import org.springblade.bi.client.model.CubeDataModel;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.exception.RpcException;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.cube.dto.BrowserParam;
import org.springblade.cube.dto.BrowserRequest;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.service.CubeDataBrowserService;
import org.springblade.cube.service.CubeDataService;
import org.springblade.cube.service.MixCubeDataService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.hutool.core.bean.BeanUtil;

/**
 * @author wookvn
 */
@RestController
public class CubeDataResouce implements CubeDataClient {

    @Autowired
    private CubeDataService cubeDataService;

    @Autowired
    private CubeDataBrowserService cubeDataBrowserService;

    @Autowired
    private MixCubeDataService mixCubeDataService;

    @Override
    public CubeDataModel getById(Integer id) {
        CubeData cubeData = cubeDataService.getById(id);
        return BeanUtil.copyProperties(cubeData, CubeDataModel.class);
    }

    @Override
    public CubeBrowserSqlModel buildBrowserSql(CubeBrowserSqlForm browserSqlForm) {
        try {
			BladeUser bladeUser = new BladeUser();
			bladeUser.setUserId(browserSqlForm.getUserId());
			bladeUser.setTenantId(browserSqlForm.getTenantId());

			Query pageQuery = new Query();
			pageQuery.setAscs(browserSqlForm.getAscs());
			pageQuery.setDescs(browserSqlForm.getDescs());

			BrowserRequest browserRequest = new BrowserRequest();
			browserRequest.setPageQuery(pageQuery);
			browserRequest.setParamList(BeanUtil.copyToList(browserSqlForm.getParamList(), BrowserParam.class));

			return browserSqlForm.getTableLinkJoinParam() != null 
					? cubeDataBrowserService.buildSql(browserSqlForm.getCubeId(), browserRequest, browserSqlForm.getTableLinkJoinParam(),browserSqlForm.getSubTableQueryParams(), bladeUser)
					: cubeDataBrowserService.buildSql(browserSqlForm.getCubeId(), browserRequest, bladeUser);
		} catch (Exception e) {
			throw new RpcException(e.getMessage(), e);
		}
    }

    @Override
    public boolean changeCubeDataStatus(ChangeCubeDataStatusParams params) {
        if (CollectionUtil.isNotEmpty(params.getCubeDataIds())) {
            for (Integer cubeDataId : params.getCubeDataIds()) {
                CubeData cubeData = new CubeData();
                cubeData.setDisplayStatus(params.isDisplay() ? 1 : 0);
                cubeData.setId(cubeDataId);
                cubeDataService.updateById(cubeData);
            }
        }
        return true;
    }

    @Override
    public Integer createAggCubeData(AggCubeCreateForm aggCubeCreateForm) {
        try {
            return mixCubeDataService.doSaveOrUpdate(aggCubeCreateForm);
        } catch (Exception e) {
            throw new RpcException(e.getMessage(), e);
        }
    }

}
