package org.springblade.resouces;

import cn.hutool.core.collection.CollectionUtil;
import lombok.extern.slf4j.Slf4j;
import org.springblade.bi.client.DmlDataSourceClient;
import org.springblade.bi.client.form.DmlSaveBatchForm;
import org.springblade.bi.client.form.DmlSaveForm;
import org.springblade.bi.client.form.DmlUptDelForm;
import org.springblade.core.tool.exception.RpcException;
import org.springblade.cube.service.DynamicTableService;
import org.springblade.report.entity.ReportDatasource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

@Slf4j
@RestController
public class DmlDataSourceResource implements DmlDataSourceClient {

    @Autowired
    private DynamicTableService dynamicTableService;

    @Override
    public Map<String, Object> save(DmlSaveForm dmlSaveForm) {
        try{
            ReportDatasource reportDatasource = new ReportDatasource();
            reportDatasource.setPoolName(dmlSaveForm.getPoolName());
            return dynamicTableService.save(reportDatasource, dmlSaveForm.getTableName(), dmlSaveForm.getIdColumn(), dmlSaveForm.getParamObj());
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RpcException(e.getMessage(), e);
        }
    }

    @Override
    public List<Map<String, Object>> saveBatch(DmlSaveBatchForm dmlSaveBatchForm) {
        try{
            ReportDatasource reportDatasource = new ReportDatasource();
            reportDatasource.setPoolName(dmlSaveBatchForm.getPoolName());
            return dynamicTableService.saveBatch(reportDatasource, dmlSaveBatchForm.getTableName(), dmlSaveBatchForm.getIdColumn(), dmlSaveBatchForm.getParamList());
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RpcException(e.getMessage(), e);
        }
    }

    @Override
    public void updateOrDelete(DmlUptDelForm dmlUptDelForm) {
        try{
            ReportDatasource reportDatasource = new ReportDatasource();
            reportDatasource.setPoolName(dmlUptDelForm.getPoolName());
            Map<String, Object> paramObj = dmlUptDelForm.getParamObj();
            String sql = dmlUptDelForm.getSql();
            if(CollectionUtil.isNotEmpty(paramObj)) {
                dynamicTableService.namedPreparedExecute(reportDatasource, sql, CollectionUtil.toList(paramObj));
            }else {
                dynamicTableService.executeSql(reportDatasource, sql);
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RpcException(e.getMessage(), e);
        }
    }
}
