/**
 *
 */
package org.springblade.resouces;

import java.util.List;

import org.springblade.bi.client.DynamicDataSourceClient;
import org.springblade.bi.client.form.BaseDsForm;
import org.springblade.bi.client.form.PageDsForm;
import org.springblade.bi.client.model.PageModel;
import org.springblade.core.tool.exception.RpcException;
import org.springblade.cube.service.DynamicTableService;
import org.springblade.report.entity.ReportDatasource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import cn.hutool.core.collection.CollectionUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 */
@RestController
@Slf4j
public class DynamicDataSourceResource implements DynamicDataSourceClient {

	@Autowired
	private DynamicTableService dynamicTableService;

	@Override
	public List<JSONObject> list(BaseDsForm dsForm) {
		ReportDatasource reportDatasource = new ReportDatasource();
		reportDatasource.setPoolName(dsForm.getPoolName());
		return dynamicTableService.selectListForJSONObject(reportDatasource, dsForm.getSql());
	}

	@Override
	public PageModel<JSONObject> page(PageDsForm dsForm) {
		ReportDatasource reportDatasource = new ReportDatasource();
		reportDatasource.setPoolName(dsForm.getPoolName());
		Page<?> pageParam = new Page<>(dsForm.getCurrent(), dsForm.getSize());
		IPage<JSONObject> page = dynamicTableService.selectPageForJSONObject(pageParam, reportDatasource, dsForm.getSql());
		PageModel<JSONObject> pageResult = new PageModel<>();
		if (page != null && CollectionUtil.isNotEmpty(page.getRecords())) {
			pageResult.setRecords(page.getRecords());
			pageResult.setCurrent(page.getCurrent());
			pageResult.setPages(page.getPages());
			pageResult.setTotal(page.getTotal());
			pageResult.setSize(page.getSize());
		}
		return pageResult;
	}

	@Override
	public JSONObject getOne(BaseDsForm dsForm) {
		ReportDatasource reportDatasource = new ReportDatasource();
		reportDatasource.setPoolName(dsForm.getPoolName());
		return dynamicTableService.selectOneForJSONObject(reportDatasource, dsForm.getSql());
	}

	@Override
	public void execute(BaseDsForm dsForm) {
		try {
			ReportDatasource reportDatasource = new ReportDatasource();
			reportDatasource.setPoolName(dsForm.getPoolName());
			dynamicTableService.executeSql(reportDatasource, dsForm.getSql());
		} catch (Exception e) {
			throw new RpcException(e.getMessage(), e);
		}
	}

	@Override
	public JSONObject executeNew(BaseDsForm dsForm) {
		JSONObject jsonObject = new JSONObject();
		try {
			ReportDatasource reportDatasource = new ReportDatasource();
			reportDatasource.setPoolName(dsForm.getPoolName());
			dynamicTableService.executeSql(reportDatasource, dsForm.getSql());
			jsonObject.put("code", "200");
			return jsonObject;
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			jsonObject.put("code", "500");
			return jsonObject;
		}
	}

	@Override
	public Long insertReturnAutoIncrementIdValue(BaseDsForm dsForm) {
		try {
			ReportDatasource reportDatasource = new ReportDatasource();
			reportDatasource.setPoolName(dsForm.getPoolName());
			return dynamicTableService.insertReturnIdValue(reportDatasource, dsForm.getSql(), dsForm.getIdColumn());
		} catch (Exception e) {
			throw new RpcException(e.getMessage(), e);
		}
	}

	@Override
	public void preparedExecute(BaseDsForm dsForm) {
		try {
			ReportDatasource reportDatasource = new ReportDatasource();
			reportDatasource.setPoolName(dsForm.getPoolName());
			dynamicTableService.preparedExecute(reportDatasource, dsForm.getSql(), dsForm.getBatchArgs());
		} catch (Exception e) {
			throw new RpcException(e.getMessage(), e);
		}
	}

}