/**
 *
 */
package org.springblade.subscribe.controller;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.api.ResultCode;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.core.tool.utils.Func;
import org.springblade.message.app.DingtalkMsgTemplate;
import org.springblade.message.app.FeishuMsgTemplate;
import org.springblade.message.app.QyWechatMsgTemplate;
import org.springblade.subscribe.dto.VisualSubscribeAggregation;
import org.springblade.subscribe.entity.VisualSubscribe;
import org.springblade.subscribe.entity.VisualSubscribeNotifyConf;
import org.springblade.subscribe.entity.VisualSubscribeOperateLog;
import org.springblade.subscribe.enums.DataType;
import org.springblade.subscribe.enums.NotifyType;
import org.springblade.subscribe.enums.OperateType;
import org.springblade.subscribe.enums.TriggerPeriod;
import org.springblade.subscribe.service.VisualSubscribeNotifyConfService;
import org.springblade.subscribe.service.VisualSubscribeOperateLogService;
import org.springblade.subscribe.service.VisualSubscribeService;
import org.springblade.subscribe.vo.VisualSubscribeBatchVO;
import org.springblade.subscribe.vo.VisualSubscribeVO;
import org.springblade.system.client.UserClient;
import org.springblade.system.client.model.UserModel;
import org.springblade.visual.utils.CheckEmailUtil;
import org.springblade.visual.utils.MobileUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.taobao.api.ApiException;
import com.xxl.job.core.biz.AdminBiz;
import com.xxl.job.core.biz.model.JobInfo;
import com.xxl.job.core.biz.model.ReturnT;

import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Slf4j
@RestController
@RequestMapping("/subscribe")
@Api(value = "订阅接口", tags = "订阅服务接口")
public class VisualSubscribeController extends BladeController {

	@Autowired
	private AdminBiz adminBiz;

	@Autowired
	private VisualSubscribeService visualSubscribeService;

	@Autowired
	private UserClient userClient;

	@Autowired
	private VisualSubscribeNotifyConfService notifyConfService;

	@Autowired
	private VisualSubscribeOperateLogService operateLogService;

	@Autowired
	private DingtalkMsgTemplate dingtalkMsgTemplate;

	@Autowired
	private FeishuMsgTemplate feishuMsgTemplate;

	@Autowired
	private QyWechatMsgTemplate qyWechatMsgTemplate;

	@Value("${xxl.job.executor.groupid}")
	private int groupId;

	/**
	 *
	 * @param visualSubscribe
	 * @param query
	 * @return
	 */
	@GetMapping("/list")
	@ApiOperation(value = "订阅服务列表")
	public R<IPage<VisualSubscribe>> list(VisualSubscribe visualSubscribe, Query query){
		QueryWrapper<VisualSubscribe> queryWrapper = new QueryWrapper<VisualSubscribe>(visualSubscribe);
		queryWrapper.lambda().eq(VisualSubscribe::getUserId, getUser().getUserId()).orderByDesc(VisualSubscribe::getId);
		return R.data(visualSubscribeService.page(Condition.getPage(query), queryWrapper));
	}

	/**
	 *
	 * @param query
	 * @return
	 */
	@GetMapping("/admList")
	@ApiOperation(value = "订阅服务管理列表")
	public R<IPage<VisualSubscribe>> admlist(DataType dataType, Integer dataId, Query query){
		LambdaQueryWrapper<VisualSubscribe> queryWrapper = Wrappers.<VisualSubscribe>lambdaQuery();
		queryWrapper.eq(dataType != null, VisualSubscribe::getDataType, dataType)
				.eq(dataId != null, VisualSubscribe::getDataId, dataId).orderByDesc(VisualSubscribe::getId);
		return R.data(visualSubscribeService.page(Condition.getPage(query), queryWrapper));
	}

	/**
	 *
	 * @return
	 */
	@GetMapping("/aggList")
	@ApiOperation(value = "订阅服务汇总列表")
	public R<IPage<VisualSubscribeAggregation>> aggList(Long userId, String subTitle, Query query){
		QueryWrapper<VisualSubscribe> queryWrapper = Wrappers.query();
		queryWrapper.lambda().eq(userId != null, VisualSubscribe::getUserId, userId)
				.like(StrUtil.isNotBlank(subTitle), VisualSubscribe::getSubTitle, subTitle);
		return R.data(visualSubscribeService.aggPage(Condition.getPage(query), queryWrapper));
	}

	/**
	 *
	 * @param id
	 * @return
	 */
	@GetMapping("/info")
	@ApiOperation(value = "根据ID查询订阅服务")
	public R<VisualSubscribe> info(@RequestParam Integer id) {
		return R.data(visualSubscribeService.getById(id));
	}

	/**
	 *
	 * @param dataType
	 * @param dataId
	 * @return
	 */
	@GetMapping("/status")
	@ApiOperation(value = "根据大屏或报告ID查询是否订阅")
	public R<Integer> status(@RequestParam DataType dataType, @RequestParam Integer dataId){
		LambdaQueryWrapper<VisualSubscribe> queryWrapper = Wrappers.<VisualSubscribe>lambdaQuery()
				.eq(VisualSubscribe::getDataId, dataId)
				.eq(VisualSubscribe::getDataType, dataType)
				.eq(VisualSubscribe::getUserId, getUser().getUserId());
		VisualSubscribe visualSubscribe = visualSubscribeService.getOne(queryWrapper.last("limit 1"));
		return R.data(visualSubscribe != null ? visualSubscribe.getId() : 0);
	}

	/**
	 *
	 * @param visualSubscribe
	 * @return
	 */
	@PostMapping("/submit")
	@ApiOperation(value = "新增或修改订阅服务")
	public R<String> doSubmit(@RequestBody VisualSubscribe visualSubscribe){
		int jobId = 0; boolean uptJob = false;
		VisualSubscribeOperateLog operateLog = new VisualSubscribeOperateLog();
		BeanUtils.copyProperties(visualSubscribe,operateLog);
		if(visualSubscribe.getId() == null) {
			visualSubscribe.setCreateTime(new Date());
			visualSubscribe.setUserId(getUser().getUserId());
			visualSubscribe.setUserName(getUser().getNickName());
			if(visualSubscribe.getTriggerMinute() == null) {
				visualSubscribe.setTriggerMinute(RandomUtil.randomInt(0, 10));
			}
			uptJob = true;

			operateLog.setCreateTime(new Date());
			operateLog.setUserId(getUser().getUserId());
			operateLog.setUserName(getUser().getNickName());
			operateLog.setOperatorId(getUser().getUserId());
			operateLog.setOperatorName(getUser().getNickName());
			operateLog.setOperateType(OperateType.add);

		}else {
			operateLog.setId(null);
			operateLog.setCreateTime(new Date());
			operateLog.setUserId(getUser().getUserId());
			operateLog.setUserName(getUser().getNickName());
			operateLog.setOperatorId(getUser().getUserId());
			operateLog.setOperatorName(getUser().getNickName());
			operateLog.setOperateType(OperateType.edit);

			VisualSubscribe dbSubscribe = visualSubscribeService.getById(visualSubscribe.getId());
			jobId = dbSubscribe.getJobId();
			//判断是否需要修改xxljob任务

			if (visualSubscribe.getTriggerPeriod()== TriggerPeriod.hour){
				uptJob=dbSubscribe.getTriggerPeriod() != visualSubscribe.getTriggerPeriod()||
						dbSubscribe.getBeginTriggerHour()!=visualSubscribe.getBeginTriggerHour()||
						dbSubscribe.getEndTriggerHour()!=visualSubscribe.getEndTriggerHour()||
						dbSubscribe.getPeriod()!=visualSubscribe.getPeriod();

			}else {
				uptJob = dbSubscribe.getTriggerDayValue() != visualSubscribe.getTriggerDayValue() ||
						dbSubscribe.getTriggerPeriod() != visualSubscribe.getTriggerPeriod() ||
						dbSubscribe.getTriggerHour() != visualSubscribe.getTriggerHour() ||
						dbSubscribe.getTriggerWeekValue() != visualSubscribe.getTriggerWeekValue() ||
						dbSubscribe.getTriggerMinute() != visualSubscribe.getTriggerMinute();
			}
		}
		//先添加获取自增ID
		boolean _ret = visualSubscribeService.saveOrUpdate(visualSubscribe);
		operateLogService.save(operateLog);
		if(uptJob && _ret) {
			ReturnT<String> ret = _handleJob(visualSubscribe, jobId);
			log.info("新增或修改xxljob任务|uptStatus={},result={}", visualSubscribe.getId() == null, ret);
			if (jobId == 0 && ReturnT.SUCCESS_CODE == ret.getCode()){
				//首次添加任务，需要把任务ID更新进去
				VisualSubscribe entity = new VisualSubscribe();
				entity.setId(visualSubscribe.getId());
				entity.setJobId(Integer.valueOf(ret.getContent()));
				visualSubscribeService.updateById(entity);

			}
		}
		return R.status(_ret);
	}


	/**
	 *批量新增订阅服务
	 * @return
	 */
	@PostMapping("/submitBatch")
	@ApiOperation(value = "批量新增订阅服务")
	public R<String> submitBatch(@RequestBody VisualSubscribeBatchVO visualSubscribeVO) {
		List<String> userIds = visualSubscribeVO.getUserIds();
		//订阅内容集合
		List<VisualSubscribeVO> dataList = visualSubscribeVO.getDataList();
		try {
			if (userIds.size() > 0 && dataList.size()>0) {
				for (String userId : userIds) {
					for (VisualSubscribeVO data : dataList) {
						VisualSubscribe visualSubscribe = new VisualSubscribe();
						BeanUtils.copyProperties(visualSubscribeVO, visualSubscribe);
						UserModel userModel = userClient.getById(Long.parseLong(userId));
						visualSubscribe.setCreateTime(new Date());
						visualSubscribe.setUserId(Long.parseLong(userId));
						visualSubscribe.setUserName(userModel.getRealName());
						visualSubscribe.setDataId(data.getDataId());
						visualSubscribe.setSubTitle(data.getSubTitle());
						visualSubscribe.setDataType(data.getDataType());
						visualSubscribe.setNotifyTitle(data.getNotifyTitle());
						if (visualSubscribe.getTriggerMinute() == null) {
							visualSubscribe.setTriggerMinute(RandomUtil.randomInt(0, 10));
						}
						boolean _ret = visualSubscribeService.saveOrUpdate(visualSubscribe);
						if (_ret) {
							VisualSubscribeOperateLog operateLog = new VisualSubscribeOperateLog();
							BeanUtils.copyProperties(visualSubscribe,operateLog);
							operateLog.setId(null);
							operateLog.setCreateTime(new Date());
							operateLog.setOperatorId(getUser().getUserId());
							operateLog.setOperatorName(getUser().getNickName());
							operateLog.setOperateType(OperateType.add);
							operateLogService.save(operateLog);
							ReturnT<String> ret = _handleJob(visualSubscribe, 0);
							log.info("新增或修改xxljob任务|uptStatus={},result={}", visualSubscribe.getId() == null, ret);
							if (ReturnT.SUCCESS_CODE == ret.getCode()) {
								//首次添加任务，需要把任务ID更新进去
								VisualSubscribe entity = new VisualSubscribe();
								entity.setId(visualSubscribe.getId());
								entity.setJobId(Integer.valueOf(ret.getContent()));
								visualSubscribeService.updateById(entity);
							}
						}
					}
				}
			}
		}catch (Exception e) {
			log.error(e.getMessage(), e);
			return R.fail("批量新增订阅服务失败");
		}
		return R.success("批量新增订阅服务成功");
	}
	/**
	 * 取消订阅服务
	 * @param ids
	 * @return
	 */
	@PostMapping("/remove")
	@ApiOperation(value = "根据ID数组删除订阅服务(多个逗号分割)", httpMethod = "POST")
	public R<String> doRemove(@RequestParam String ids) {
		List<Integer> visualSubscribeIds = Func.toIntList(ids);
		for (Integer id : visualSubscribeIds){

			VisualSubscribe dbSubscribe = visualSubscribeService.getById(id);
			if (dbSubscribe != null && dbSubscribe.getJobId()!= null){
				int jobId = dbSubscribe.getJobId();
				//先移除订阅服务
				ReturnT<String> ret = adminBiz.jobRemove(jobId);
				VisualSubscribeOperateLog operateLog = new VisualSubscribeOperateLog();
				BeanUtils.copyProperties(dbSubscribe,operateLog);
				operateLog.setId(null);
				operateLog.setCreateTime(new Date());
				operateLog.setOperatorId(getUser().getUserId());
				operateLog.setOperatorName(getUser().getNickName());
				operateLog.setOperateType(OperateType.remove);
				operateLogService.save(operateLog);
				log.info("移除xxljob任务|result="+ret);
			}
		}
		return R.status(visualSubscribeService.removeByIds(visualSubscribeIds));
	}

	/**
	 * 处理xxljob任务
	 * @param visualSubscribe
	 * @return
	 */
	private ReturnT<String> _handleJob(VisualSubscribe visualSubscribe, int jobId) {
		String cron ="";
		if (visualSubscribe.getTriggerPeriod()== TriggerPeriod.hour){
			//0 0/30 9-17 * * ?
			cron=String.format("0 0/%s %s-%s * * ?", visualSubscribe.getPeriod(), visualSubscribe.getBeginTriggerHour(),visualSubscribe.getEndTriggerHour());
		}else {
			cron = visualSubscribe.getTriggerPeriod().buildCron(
					visualSubscribe.getTriggerDayValue(), visualSubscribe.getTriggerWeekValue(), visualSubscribe.getTriggerHour(), visualSubscribe.getTriggerMinute());
		}
		JobInfo xxlJobInfo = new JobInfo();
		xxlJobInfo.setJobGroup(groupId);
		xxlJobInfo.setJobCron(cron);
		xxlJobInfo.setJobDesc("订阅服务:"+visualSubscribe.getSubTitle());
		xxlJobInfo.setAuthor(visualSubscribe.getUserId().toString());
		xxlJobInfo.setScheduleType("CRON");
		xxlJobInfo.setScheduleConf(cron);
		xxlJobInfo.setMisfireStrategy("DO_NOTHING");
		xxlJobInfo.setExecutorRouteStrategy("ROUND");
		xxlJobInfo.setExecutorHandler("visualSubscribeJobHandler");
		xxlJobInfo.setExecutorBlockStrategy("SERIAL_EXECUTION");
		//订阅服务ID作为参数
		xxlJobInfo.setExecutorParam("" + visualSubscribe.getId());
		xxlJobInfo.setExecutorTimeout(0);
		xxlJobInfo.setExecutorFailRetryCount(0);
		xxlJobInfo.setGlueUpdatetime(new Date());
		xxlJobInfo.setGlueType("BEAN");
		xxlJobInfo.setTriggerLastTime(0);
		xxlJobInfo.setTriggerNextTime(0);
		if(jobId == 0) { //新增
			xxlJobInfo.setAddTime(new Date());
			xxlJobInfo.setTriggerStatus(1);//运行
			return adminBiz.jobAdd(xxlJobInfo);
		}else {
			xxlJobInfo.setId(jobId);
			return adminBiz.jobUpdate(xxlJobInfo);
		}
	}

	/**
	 * 	判断账户是否配置
	 * @return
	 */
	@PostMapping("/check")
	@ApiOperation(value = "判断账户是否配置", httpMethod = "POST")
	public R check(@RequestBody VisualSubscribe visualSubscribe) throws ApiException {
		//查询用户信息
		UserModel userModel = userClient.getById(getUser().getUserId());
		if (userModel != null){
			if (visualSubscribe.getNotifyType() == NotifyType.email){
				if (StrUtil.isNotBlank(userModel.getEmail())){
					boolean b = CheckEmailUtil.checkEmail(userModel.getEmail());
					if (b){
						return  R.data(ResultCode.SUCCESS.getCode(),0,"邮箱验证通过!");
					}else {
						return  R.data(ResultCode.SUCCESS.getCode(),1,"该用户邮箱格式不正确!");
					}
				}else {
					return  R.data(ResultCode.SUCCESS.getCode(),1,"该用户没有邮箱地址!");
				}

			}else if (visualSubscribe.getNotifyType() == NotifyType.qywechat){
				if (StrUtil.isNotBlank(userModel.getPhone())){
					boolean b = MobileUtil.isLegalMobileNumber(userModel.getPhone());
					if (b){
						QueryWrapper<VisualSubscribeNotifyConf> queryWrapper =new QueryWrapper<>();
						queryWrapper.lambda().eq(VisualSubscribeNotifyConf::getNotifyType, visualSubscribe.getNotifyType());
						List<VisualSubscribeNotifyConf> list = notifyConfService.list(queryWrapper);
						Map<String, String> map = list.stream().collect(Collectors.toMap(VisualSubscribeNotifyConf::getAttriKey, VisualSubscribeNotifyConf::getAttriValue));
						String corp_id = map.get("corp_id");
						String corp_secret = map.get("corp_secret");

						String userId = null;
						try {
							String accessToken = qyWechatMsgTemplate.getAccessToken(corp_id, corp_secret);
							List<String> userIds = qyWechatMsgTemplate.getUserIds(accessToken, Arrays.asList(userModel.getPhone()));
							if(CollectionUtil.isNotEmpty(userIds)) {
								userId = userIds.get(0);
							}
						} catch (Exception e) {
							log.error(e.getMessage(), e);
						}
						if (StrUtil.isNotBlank(userId)){
							return  R.data(ResultCode.SUCCESS.getCode(),0,"企业微信验证通过");
						}else {
							return  R.data(ResultCode.SUCCESS.getCode(),1,"该用户手机号码未绑定企业微信!");
						}
					}else {
						return  R.data(ResultCode.SUCCESS.getCode(),1,"该用户手机号码不正确!");
					}
				}else {
					return  R.data(ResultCode.SUCCESS.getCode(),1,"该用户没有手机号码!");
				}

			}else if (visualSubscribe.getNotifyType() == NotifyType.dingtalk){
				if (StrUtil.isNotBlank(userModel.getPhone())){
					boolean b = MobileUtil.isLegalMobileNumber(userModel.getPhone());
					if (b){
						QueryWrapper<VisualSubscribeNotifyConf> queryWrapper =new QueryWrapper<>();
						queryWrapper.lambda().eq(VisualSubscribeNotifyConf::getNotifyType, visualSubscribe.getNotifyType());
						List<VisualSubscribeNotifyConf> list = notifyConfService.list(queryWrapper);
						Map<String, String> map = list.stream().collect(Collectors.toMap(VisualSubscribeNotifyConf::getAttriKey, VisualSubscribeNotifyConf::getAttriValue));
						String corp_id = map.get("app_key");
						String corp_secret = map.get("app_secret");
						String userId = null;
						try {
							String accessToken = dingtalkMsgTemplate.getAccessToken(corp_id, corp_secret);
							List<String> userIds = dingtalkMsgTemplate.getUserIds(accessToken, Arrays.asList(userModel.getPhone()));
							if(CollectionUtil.isNotEmpty(userIds)) {
								userId = userIds.get(0);
							}
						} catch (Exception e) {
							log.error(e.getMessage(), e);
						}
						if (StrUtil.isNotBlank(userId)){
							return  R.data(ResultCode.SUCCESS.getCode(),0,"钉钉验证通过");
						}else {
							return  R.data(ResultCode.SUCCESS.getCode(),1,"该用户手机号码未绑定钉钉!");
						}
					}else {
						return  R.data(ResultCode.SUCCESS.getCode(),1,"该用户手机号码不正确!");
					}
				}else {
					return  R.data(ResultCode.SUCCESS.getCode(),1,"该用户没有手机号码!");
				}
			}else if (visualSubscribe.getNotifyType() == NotifyType.feishu){
				if (StrUtil.isNotBlank(userModel.getPhone())){
					boolean b = MobileUtil.isLegalMobileNumber(userModel.getPhone());
					if (b){
						QueryWrapper<VisualSubscribeNotifyConf> queryWrapper =new QueryWrapper<>();
						queryWrapper.lambda().eq(VisualSubscribeNotifyConf::getNotifyType, visualSubscribe.getNotifyType());
						List<VisualSubscribeNotifyConf> list = notifyConfService.list(queryWrapper);
						Map<String, String> map = list.stream().collect(Collectors.toMap(VisualSubscribeNotifyConf::getAttriKey, VisualSubscribeNotifyConf::getAttriValue));
						String corp_id = map.get("app_id");
						String corp_secret = map.get("app_secret");
						String userId = null;
						try {
							String accessToken = feishuMsgTemplate.getAccessToken(corp_id, corp_secret);
							List<String> userIds = feishuMsgTemplate.getUserIds(accessToken, Arrays.asList(userModel.getPhone()));
							if(CollectionUtil.isNotEmpty(userIds)) {
								userId = userIds.get(0);
							}
						} catch (Exception e) {
							log.error(e.getMessage(), e);
						}
						if (StrUtil.isNotBlank(userId)){
							return  R.data(ResultCode.SUCCESS.getCode(),0,"飞书验证通过");
						}else {
							return  R.data(ResultCode.SUCCESS.getCode(),1,"该用户手机号码未绑定飞书!");
						}
					}else {
						return  R.data(ResultCode.SUCCESS.getCode(),1,"该用户手机号码不正确!");
					}
				}else {
					return  R.data(ResultCode.SUCCESS.getCode(),1,"该用户没有手机号码!");
				}
			}
		}else {
			return  R.data(ResultCode.SUCCESS.getCode(),1,"该用户不存在!");
		}
		return null;
	}
}