/**
 * 
 */
package org.springblade.subscribe.task.handler;

import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.net.URL;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import javax.imageio.ImageIO;

import org.apache.commons.io.FileUtils;
import org.springblade.core.tool.support.Kv;
import org.springblade.subscribe.entity.VisualSubscribeNotifyConf;
import org.springblade.subscribe.enums.DataType;
import org.springblade.subscribe.enums.NotifyType;
import org.springblade.subscribe.service.VisualSubscribeNotifyConfService;
import org.springblade.system.client.UserClient;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.service.VisualDataService;
import org.springblade.visual.utils.PdfConverterUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Slf4j
@Component
public class VisualDataPdfHandler {
	
	@Autowired
	private VisualDataService visualDataService;

	@Autowired
	private UserClient userClient;

	@Autowired
	private VisualSubscribeNotifyConfService notifyConfService;

	@Value("${blade.domain}")
	private String domain;

	@Value("${blade.pdfUrl}")
	private String pdfUrl;
	
	//文件目录
	@Value("${blade.file.exportPdf}")
	private String exportPdfDir;

	/**
	 * 获取大屏快照截图的图片地址
	 * @param userId
	 * @param account
	 * @param dataIds
	 * @param dataType
	 * @return
	 */
	public List<String> getVisualScreenShotImageUrl(Long userId, String account, List<Integer> dataIds, DataType dataType){
		List<VisualData> visualList = new ArrayList<>();
		if (dataType == DataType.visual  || dataType == DataType.report) { //大屏或者自主报表
			visualList = visualDataService.listByIds(dataIds);
		} else if (dataType == DataType.ppt  ){//幻灯片
			visualList = visualDataService.list(Wrappers.<VisualData>lambdaQuery().in(VisualData::getPowerPointId, dataIds).orderByAsc(VisualData::getOrderIndex).orderByAsc(VisualData::getId));
		}
		// 大屏不为空
		if (CollectionUtil.isEmpty(visualList)) {
			return null;
		}
		try {
			// 查询用户accessToken
			String accessToken = userClient.genAccessToken(userId);
			// 调用接口导出pdf {code: 200, data:[], msg: ""}
			String body = JSON.toJSONString(_buildParam(visualList, accessToken, dataType, account));
			log.info("调用pdf接口前置|body={}", body);
			// 设置超时5分钟
			String resBody = HttpUtil.post(pdfUrl, body, 300000);
			log.info("调用pdf接口完成resBody:" + resBody);
			JSONArray jsonArray = JSON.parseObject(resBody).getJSONArray("data");
			if (jsonArray != null) {
				return jsonArray.toList(String.class);
			}
		} catch (Exception e) {
			log.error(e.getMessage(), e);
		}
		return null;
	}
	
	/**
	 * 根据图片地址构建PDF文件
	 * @param imageUrlList
	 * @param fileName
	 * @return
	 * @throws Exception
	 */
	public File buildImagePdf(List<String> imageUrlList, String fileName) throws Exception{
		List<BufferedImage> imageStreamList = new ArrayList<>();
		for (String imageUrl : imageUrlList) {
			BufferedImage imageStream = ImageIO.read(new URL(imageUrl));
			imageStreamList.add(imageStream);
		}
		ByteArrayInputStream pdfFromImage = PdfConverterUtil.buildSinglePagePdfFromImage(imageStreamList);
		LocalDate localDate = LocalDate.now();
		//按照年月分目录
		File dir = FileUtil.file(exportPdfDir, String.valueOf(localDate.getYear()), localDate.getMonth().name().toLowerCase());
		if(!dir.exists()) {
			dir.mkdirs();
		}
		File file = FileUtil.file(dir, fileName + ".pdf");
		FileUtils.copyInputStreamToFile(pdfFromImage, file);
		return file;
	}
	
	/**
	 * 构造参数
	 * 
	 * @param visualList
	 * @param token
	 * @return
	 */
	private Kv _buildParam(List<VisualData> visualList, String token,DataType dataType, String account) {
		List<Kv> siteList = new ArrayList<>();
		List<VisualSubscribeNotifyConf> watermarkList = notifyConfService.notifyConfList("0",NotifyType.watermark);
		Map<String, String> watermarkMap = watermarkList.stream().collect(
				Collectors.toMap(VisualSubscribeNotifyConf::getAttriKey, VisualSubscribeNotifyConf::getAttriValue));
		String water_mark = watermarkMap.get("water_mark");
		Kv detail = Kv.create().set("homeUrl", domain).set("token", token).set("site", siteList);
		if (StrUtil.isNotBlank(water_mark) && "1".equals(water_mark)) {// 开启水印
			detail.set("watermark", 1);
			String water_mark_type = watermarkMap.get("water_mark_type");
			String water_mark_content = watermarkMap.get("water_mark_content");
			if (StrUtil.isNotBlank(water_mark_type) && "0".equals(water_mark_type)) {// 账号名称
				detail.set("text", account);
			}
			if (StrUtil.isNotBlank(water_mark_type) && "1".equals(water_mark_type)) {// 自定义
				detail.set("text", water_mark_content);
			}
		}
		for (VisualData visualData : visualList) {
//			Integer ctId = visualData.getCategory() == null ? visualData.getHistoryReportId()
//					: visualData.getCategory();
//			String urlReport = isReport ? "report/" : "",
			String	url="";
			if (dataType == DataType.visual  || dataType == DataType.report) { //大屏或者自主报表
				url = String.format("newbi/fullscreen/%s", visualData.getId());
			}else if (dataType == DataType.ppt  ){//幻灯片
				url = String.format("fullpowerpoint/%s/%s", visualData.getPowerPointId(),visualData.getId());
			}
			siteList.add(Kv.create().set("id", visualData.getId()).set("url", url).set("content",
					JSON.parseObject(visualData.getContent())));
		}
		return detail;
	}
}
