package org.springblade.usergroup.controller;


import java.util.Date;
import java.util.List;

import javax.validation.Valid;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.usergroup.entity.DefineGroup;
import org.springblade.usergroup.entity.GroupClassify;
import org.springblade.usergroup.service.DefineGroupService;
import org.springblade.usergroup.service.GroupClassifyService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@RestController
@RequestMapping("/group/groupClassify")
@Api(value = "人群分类管理", tags = "人群分类管理")
public class GroupClassifyController extends BladeController {

	@Autowired
	private GroupClassifyService groupClassifyService;

	@Autowired
	private DefineGroupService defineGroupService;


	/**
	 *
	 * @param groupClassify
	 * @param query
	 * @return
	 */
	@GetMapping("/list")
	@ApiOperation(value = "列表")
	public R<IPage<GroupClassify>> list(GroupClassify groupClassify, Query query){
		QueryWrapper<GroupClassify> queryWrapper = new QueryWrapper<GroupClassify>(groupClassify);
		return R.data(groupClassifyService.page(Condition.getPage(query), queryWrapper.lambda().orderByDesc(GroupClassify::getId)));
	}

	/**
	 *
	 * @return
	 */
	@GetMapping("/test/{name}")
	@ApiOperation(value = "测试")
	public void test(@PathVariable("name") String name,String processId){
		GroupClassify groupClassify=new GroupClassify();
		groupClassify.setCreateTime(new Date());
		groupClassify.setClassifyName(name+"processId:"+processId);
		groupClassify.setCreateBy("ceshi");
		groupClassify.setCreateUserId(1L);
		groupClassifyService.save(groupClassify);
	}


	/**
	 * 新增
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "新增 save", notes = "传入groupClassify")
	public R<Integer> save(@Valid @RequestBody GroupClassify groupClassify) {
		GroupClassify one = groupClassifyService.getOne(new LambdaQueryWrapper<GroupClassify>()
			.eq(GroupClassify::getClassifyName, groupClassify.getClassifyName())
		);
		if (groupClassify.getId() == null){
			if(one != null){
				return R.fail("人群分类名称重复");
			}
			groupClassify.setCreateBy(SecureUtil.getNickName());
			groupClassify.setCreateTime(new Date());
			groupClassify.setCreateUserId(SecureUtil.getUserId());
			groupClassifyService.save(groupClassify);
		}else {
				if (one != null  && (!one.getId().equals(groupClassify.getId()))){
					return R.fail("人群分类名称重复");
				}else {
					groupClassify.setUpdateTime(new Date());
					groupClassifyService.updateById(groupClassify);
				}
		}
		return R.data(groupClassify.getId());
	}



	/**
	 * 	删除
	 * @param ids
	 * @return
	 */
	@PostMapping("/remove")
	@ApiOperation(value = "根据ID数组删除自定义人群(多个逗号分割)", httpMethod = "POST")
	@ApiOperationSupport(order = 3)
	public R<String> doRemove(@ApiParam(value = "主键", required = true) @RequestParam String ids) {
		List<Integer> classifyIds = Func.toIntList(ids);
		for (Integer id : classifyIds){
			long count = defineGroupService.count(new LambdaQueryWrapper<DefineGroup>()
					.eq(DefineGroup::getClassifyId,id)
			);
			if (count > 0){
				return R.fail("该分类下有定义的人群，请勿删除");
			}

		}

		return R.status(groupClassifyService.removeByIds(Func.toIntList(ids)));
	}

}
