/**
 * 
 */
package org.springblade.visual.controller;

import java.io.File;
import java.net.URLEncoder;
import java.util.Base64;
import java.util.Date;
import java.util.concurrent.CountDownLatch;

import javax.servlet.http.HttpServletResponse;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.visual.dto.ComponentDto;
import org.springblade.visual.dto.QuerySqlParam;
import org.springblade.visual.entity.VisualComponent;
import org.springblade.visual.entity.VisualComponentExport;
import org.springblade.visual.enums.ComponentTypeEnum;
import org.springblade.visual.service.QuerySqlService;
import org.springblade.visual.service.VisualComponentExportService;
import org.springblade.visual.service.VisualDataPermissionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.microsoft.sqlserver.jdbc.StringUtils;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.SneakyThrows;

/**
 * @author wookvn
 *
 */
@RestController
@RequestMapping("visual/componentDetail")
@Api(value = "模版导出组件", tags = "清单组件导出任务接口")
public class VisualComponentExportCotroller extends BladeController{

	@Autowired
	private VisualComponentExportService visualComponentExportService;
	
	@Autowired
	private QuerySqlService querySqlService;
	
	@Autowired
	private VisualDataPermissionService visualDataPermissionService;
	
	/**
	 * 列表
	 */
	@GetMapping("/exportList")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "分页查询我的导出任务", notes = "传入visualComponent")
	public R<IPage<VisualComponentExport>> page(VisualComponentExport visualComponentExport, Query query) {
		LambdaQueryWrapper<VisualComponentExport> queryWrapper = new LambdaQueryWrapper<>();
		if (visualComponentExport != null && StrUtil.isNotEmpty(visualComponentExport.getComponentName())){
			queryWrapper.like(VisualComponentExport::getComponentName,visualComponentExport.getComponentName());
		}
		if (visualComponentExport != null && StrUtil.isNotEmpty(visualComponentExport.getVisualName())){
			queryWrapper.like(VisualComponentExport::getVisualName,visualComponentExport.getVisualName());
		}
		if (visualComponentExport != null && StrUtil.isNotEmpty(visualComponentExport.getFileName())){
			queryWrapper.like(VisualComponentExport::getFileName,visualComponentExport.getFileName());
		}
		//超级管理员，可以查看所有人
		queryWrapper.eq(!AuthUtil.isAdmin(), VisualComponentExport::getOperatorId, getUser().getUserId()).orderByDesc(VisualComponentExport::getId);
		return R.data(visualComponentExportService.page(Condition.getPage(query), queryWrapper));
	}
	
	
	/**
	 * 
	 * @param value
	 * @return
	 * @throws Exception
	 */
	@PostMapping("/export")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "清单数据导出", notes = "传入参数与清单预览接口一致")
	public R<String> export(@RequestBody String value, HttpServletResponse response) throws Exception{
		String decoderStr = new String(Base64.getDecoder().decode(value), "utf-8");
		ComponentDto componentDto = JSON.parseObject(decoderStr, ComponentDto.class);
		if(StringUtils.isEmpty(componentDto.getColumnList())) {
			return R.fail("字段列表不能为空");
		}
		if(componentDto.getVisualDataId() != null) {
			//校验大屏是否有访问权限
			if(!visualDataPermissionService.checkVisualDataForUser(componentDto.getVisualDataId(), componentDto.getVisualDataCreateUserId(), getUser())) {
				return R.fail("该大屏没有访问权限");
			}
		}
		VisualComponent component = componentDto.getComponent();
		R<QuerySqlParam> r = querySqlService.parseQuerySql(component.getCubeDataId(), componentDto, getUser(), ComponentTypeEnum.DETAIL);
		if(!r.isSuccess()) {
			return R.fail(r.getMsg());
		}
		//生成一条导出任务记录
		VisualComponentExport visualComponentExport = new VisualComponentExport();
		visualComponentExport.setVisualId(componentDto.getVisualDataId());
		visualComponentExport.setComponentId(componentDto.getComponent().getId());
		visualComponentExport.setCreateTime(new Date());
		visualComponentExport.setExportStatus(0);
		visualComponentExport.setVisualCategoryId(componentDto.getVisualDataCid());
		visualComponentExport.setVisualName(componentDto.getVisualDataTitle());
		visualComponentExport.setComponentName(componentDto.getComponent().getName());
		visualComponentExport.setOperator(getUser().getNickName());
		visualComponentExport.setOperatorId(getUser().getUserId());
		if(StrUtil.isNotBlank(componentDto.getFileName())) {
			visualComponentExport.setFileName(componentDto.getFileName() + ".xlsx");
		}else {
			visualComponentExport.setFileName(String.format("%s_%s.xlsx", 
					componentDto.getComponent().getName(), 
					DateUtil.date().toString(DatePattern.PURE_DATETIME_FORMAT)));
		}
		boolean save = visualComponentExportService.save(visualComponentExport);

		if (componentDto.getIsSync() != null && componentDto.getIsSync()) {
			visualComponentExportService.export(visualComponentExport, r.getData());
			return R.data(visualComponentExport.getId().toString());
		} else {
			//执行异步导出任务
			visualComponentExportService.doExport(visualComponentExport, r.getData());
		}
		return R.status(save);
	}
	
	/**
	 * 
	 * @param visualId
	 * @param exportId
	 */
	@SneakyThrows
	@GetMapping("/download/{exportId}")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "下载导出的清单excel", notes = "返回一个二进制数据流【需要鉴权】")
	public void download(@PathVariable Integer exportId, HttpServletResponse response) {
		VisualComponentExport visualComponentExport = visualComponentExportService.getById(exportId);
		if(visualComponentExport != null) {
			File file = visualComponentExportService.buildFile(visualComponentExport.getFileName(), visualComponentExport.getVisualId());
			// 设置响应头
			response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml,sheet;charset=utf-8");
			response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(visualComponentExport.getFileName(), "UTF-8"));
			FileUtil.writeToStream(file, response.getOutputStream());
		}
	}
}
