package org.springblade.visual.controller;


import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.codehaus.groovy.syntax.TokenUtil;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.StringUtil;
import org.springblade.system.client.UserClient;
import org.springblade.system.client.model.UserModel;
import org.springblade.visual.entity.GetBiTokenVO;
import org.springblade.visual.entity.VisualLoginFreeLink;
import org.springblade.visual.enums.WebEmbedConstant;
import org.springblade.visual.service.VisualLoginFreeLinkService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.crypto.SecretKey;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author cyc
 *
 */
@RestController
@RequestMapping("/visual/shortlink")
@Api(value = "短链接应用", tags = "短链接应用接口")
public class VisualCreateUrlController {

    @Autowired
    private UserClient userClient;

    @Autowired
    private VisualLoginFreeLinkService loginFreeLinkService;

    @Value("${blade.loginFreeDomain}")
    private String domain;


    /**
     * 免登录链接功能
     */
    @GetMapping("/getUrlByShortLink")
    @ApiOperation(value = "免登录链接功能")
    @ApiOperationSupport(order = 1)
    public R<String> getUrlByShortLink(String shortLink) throws ParseException {
        List<VisualLoginFreeLink> list=new ArrayList<>();
        VisualLoginFreeLink loginFreeLink=null;
        String[] shortArray =shortLink.split("#");
        if (shortArray.length>1){
            list =loginFreeLinkService.list(new LambdaQueryWrapper<VisualLoginFreeLink>()
                    .like(VisualLoginFreeLink::getShortLink,shortArray[1])
                    .eq(VisualLoginFreeLink::getStatus,1)
            );
            for (VisualLoginFreeLink visualLoginFreeLink: list){
                if (visualLoginFreeLink.getShortLink().indexOf(shortArray[1])!=1){
                    loginFreeLink=visualLoginFreeLink;
                }
            }
        }
        if (loginFreeLink != null){
            //判断是否过期
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            Date validDate = format.parse( loginFreeLink.getValidDate());
            long  dateTime= validDate.getTime();
            long nowTime = System.currentTimeMillis();
            if (nowTime >= dateTime){
                return R.fail("该链接授权已过期!");
            }
            String accessToken = userClient.genToken(loginFreeLink.getUserId(),String.valueOf(dateTime+24*3600*1000-nowTime));
            return R.data(String.format("%s?token=%s",loginFreeLink.getRealUrl(),accessToken));
        }else {
            return R.fail("该链接未被授权或者未上线!");
        }
    }

    /**
     * 获取bitoken
     */
    @PostMapping("/getBiToken")
    @ApiOperation(value = "获取bitoken")
    @ApiOperationSupport(order = 2)
    public R getBiToken(@RequestBody GetBiTokenVO getBiTokenVO){
//        if (StringUtil.isBlank(userId)){
//            //超级管理员
//            userId="1123598821738675201";
//        }
//
        JSONObject result =new JSONObject();
        if (!getBiTokenVO.getApiKey().equals(WebEmbedConstant.ApiKey) || !getBiTokenVO.getSecretKey().equals(WebEmbedConstant.SecretKey)){
            return R.fail("ApiKey或SecretKey有误");
        }
        String domain="";
        String[] shortArray = getBiTokenVO.getShortLink().split("#");
        if (shortArray.length>1){
            domain = shortArray[0];
        }
        VisualLoginFreeLink loginFreeLink=null;
        List<VisualLoginFreeLink> list =loginFreeLinkService.list(new LambdaQueryWrapper<VisualLoginFreeLink>()
                .like(VisualLoginFreeLink::getShortLink,shortArray[1])
                .eq(VisualLoginFreeLink::getStatus,1)
        );
        for (VisualLoginFreeLink visualLoginFreeLink: list){
            if (visualLoginFreeLink.getShortLink().indexOf(shortArray[1])!=1){
                loginFreeLink=visualLoginFreeLink;
            }
        }

        if (loginFreeLink!= null){
            //生成本系统token,默认时间48
            String genToken = userClient.genToken(loginFreeLink.getUserId(), String.valueOf(48*3600*1000));
            loginFreeLink.setAccessToken(genToken);
            loginFreeLinkService.updateById(loginFreeLink);
            result.put("biToken",genToken);
            result.put("linkUrl",domain+"#"+loginFreeLink.getRealUrl());
            return R.data(result);
        }else {
            return R.fail("该短链接有误，请联系管理员");
        }

    }




    /**
     * 验证身份，通过token获取真实链接
     */
    @GetMapping("/getUrlValid")
    @ApiOperation(value = "创建链接")
    @ApiOperationSupport(order = 3)
    public R<String> getUrlValid(String userId,String token,String shortLink,long expiresTime) throws ParseException {
        UserModel userModel = userClient.getByThirdUserId(userId);
        VisualLoginFreeLink loginFreeLink =loginFreeLinkService.getOne(new LambdaQueryWrapper<VisualLoginFreeLink>()
                .eq(VisualLoginFreeLink::getShortLink,shortLink)
                .eq(VisualLoginFreeLink::getStatus,1)
        );
        if (userModel!=null && loginFreeLink!= null){
            //判断是否过期
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            Date validDate = format.parse( loginFreeLink.getValidDate());
            long dateTime= validDate.getTime();
            long nowTime = System.currentTimeMillis();
            if (nowTime >= dateTime){
                return R.fail("该链接授权已过期!");
            }
            //保存三方token
            loginFreeLink.setUserToken(token);
            //生成本系统token
            String genToken = userClient.genToken(userModel.getId(), String.valueOf(expiresTime*1000));
            loginFreeLink.setAccessToken(genToken);
            loginFreeLinkService.updateById(loginFreeLink);
            return R.data(String.format("%s?token=%s",shortLink,genToken));
        }else {
            return R.fail("该用户没有权限");
        }

    }
    /**
     * 不需要验证身份
     */
    @GetMapping("/getUrlNoValid")
    @ApiOperation(value = "创建链接")
    @ApiOperationSupport(order = 4)
    public R<String> getUrlNoValid(String userId,String shortLink,long expiresTime) throws ParseException {
        UserModel userModel = userClient.getByThirdUserId(userId);
        VisualLoginFreeLink loginFreeLink=null;
        List<VisualLoginFreeLink> list =loginFreeLinkService.list(new LambdaQueryWrapper<VisualLoginFreeLink>()
                .eq(VisualLoginFreeLink::getShortLink,shortLink)
                .eq(VisualLoginFreeLink::getStatus,1)
        );
        for (VisualLoginFreeLink visualLoginFreeLink: list){
            if (visualLoginFreeLink.getShortLink().indexOf(shortLink)!=1){
                loginFreeLink=visualLoginFreeLink;
            }
        }
        if (userModel!=null && loginFreeLink!= null){
            //判断是否过期
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            Date validDate = format.parse( loginFreeLink.getValidDate());
            long dateTime= validDate.getTime();
            long nowTime = System.currentTimeMillis();
            if (nowTime >= dateTime){
                return R.fail("该链接授权已过期!");
            }
            //生成本系统token
            String genToken = userClient.genToken(userModel.getId(), String.valueOf(expiresTime*1000));
            loginFreeLink.setAccessToken(genToken);
            loginFreeLinkService.updateById(loginFreeLink);
            return R.data(String.format("%s?token=%s",loginFreeLink.getRealUrl(),genToken));
        }else {
            return R.fail("该用户没有权限");
        }

    }

    /**
     * 判断token有效性
     */
    @GetMapping("/checkToken")
    @ApiOperation(value = "判断token有效性")
    @ApiOperationSupport(order = 5)
    public R<String> checkToken(String token,String shortLink) throws ParseException {
        VisualLoginFreeLink loginFreeLink=null;
        List<VisualLoginFreeLink> list =loginFreeLinkService.list(new LambdaQueryWrapper<VisualLoginFreeLink>()
                .eq(VisualLoginFreeLink::getUserToken,token)
                .eq(VisualLoginFreeLink::getShortLink,shortLink)
        );
        for (VisualLoginFreeLink visualLoginFreeLink: list){
            if (visualLoginFreeLink.getShortLink().indexOf(shortLink)!=1){
                loginFreeLink=visualLoginFreeLink;
            }
        }
        if (loginFreeLink!= null){
            return R.data(loginFreeLink.getRealUrl());
        }else {
            return R.fail("该用户没有权限");
        }
    }
}
