/**
 * 
 */
package org.springblade.visual.controller;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Base64;
import java.util.List;
import java.util.concurrent.TimeUnit;

import cn.hutool.core.lang.UUID;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.log.annotation.ApiLog;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.system.client.DataPermClient;
import org.springblade.visual.dto.VisualComponentCubeDataDto;
import org.springblade.visual.dto.VisualDto;
import org.springblade.visual.dto.VisualLockDto;
import org.springblade.visual.dto.VisualUnlockParams;
import org.springblade.visual.entity.*;
import org.springblade.visual.enums.VisualTypeEnum;
import org.springblade.visual.service.*;
import org.springblade.visual.vo.CopyVisualDataCubeDataVo;
import org.springblade.visual.vo.VisualClassifyVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.annotations.ApiResponse;

/**
 * @author wookvn
 *
 */
@RestController
@RequestMapping("visual/data")
@Api(value = "大屏模版接口", tags = "大屏模版接口")
public class VisualDataController extends BladeController {

	@Autowired
	private VisualDataService visualDataService;
	
	@Autowired
	private VisualComponentService visualComponentService;
	
	@Autowired
	private VisualClassifyService visualCategoryService;

	@Autowired
	private VisualTemplateService visualTemplateService;

	@Autowired
	private DataPermClient dataPermClient;

	@Autowired
	private VisualSelfReportCategoryService visualSelfReportCategoryService;

	@Autowired
	private VisualClassifyFavouriteService visualClassifyFavouriteService;

	@Autowired
	private VisualReportCategoryFavouriteService visualReportCategoryFavouriteService;

	@Autowired
	private VisualTemplateClassifyService visualTemplateClassifyService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入visual")
	public R<VisualData> detail(VisualData visualData) {
		VisualData detail = visualDataService.getById(visualData.getId());
		if(detail == null) {
			return R.fail("大屏模版不存在");
		}
		//查询大屏使用的所有组件
		LambdaQueryWrapper<VisualComponent> queryWrapper = Wrappers.<VisualComponent>lambdaQuery()
				.select(VisualComponent::getId)
				.eq(VisualComponent::getVisualId, detail.getId())
				.eq(VisualComponent::getIsMobile, 0);
		List<Integer> componentIdList = visualComponentService.listObjs(queryWrapper, a -> Integer.valueOf(a.toString()));
		detail.setComponentIds(componentIdList);
		// 查询大屏组件信息
		return R.data(detail);
	}
	
	/**
	 * 
	 * @param visualData
	 * @return
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "列表")
	public R<IPage<VisualData>> list(VisualData visualData, Query query){
		Long userId = getUser().getUserId();
		visualData.setVisualDataType(0);
		LambdaQueryWrapper<VisualData> queryWrapper = Condition.getQueryWrapper(visualData).lambda().orderByAsc(VisualData::getOrderIndex);
		Boolean checkSuperPermRet = dataPermClient.checkSuperPermission(getUser().getUserId());
		if (visualData.getVisualType() == null) {
			visualData.setVisualType(VisualTypeEnum.GENERAL);
		}
		if(checkSuperPermRet == null || !checkSuperPermRet) {
			queryWrapper.and(t ->{
				t.eq(VisualData::getCreateUser, userId)
				.or().inSql(VisualData::getId, "select a.visual_id from visual_data_permission a inner join data_permission_user b on a.data_perm_id = b.data_perm_id where b.user_id = " + userId)
				.or().eq(VisualData::getPublicStatus, 1);
			}).eq(VisualData::getVisualDataType,0);// 查询公共大屏
		}
		return R.data(visualDataService.page(Condition.getPage(query), queryWrapper));
	}
	
	/**
	 * 
	 * @param visualData
	 * @return
	 */
	@GetMapping("/myList")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "我的收藏列表")
	public R<IPage<VisualData>> mylist(VisualData visualData, Query query){
		visualData.setCreateUser(getUser().getUserId());
		visualData.setVisualType(VisualTypeEnum.FAVOURITE);
		return R.data(visualDataService.page(Condition.getPage(query), Condition.getQueryWrapper(visualData).lambda().orderByAsc(VisualData::getOrderIndex)));
	}
	
	/**
	 * 
	 * @param visualData
	 * @return
	 */
	@ApiLog("报表组件")
	@PostMapping("/update")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "更新大屏（仅更新大屏的数据，不传递参数不更新）")
	public R<String> update(@RequestBody VisualData visualData) {
		if(visualData.getCategory() != null) {
			long count = visualCategoryService.count(Wrappers.<VisualClassify>lambdaQuery().eq(VisualClassify::getParentId, visualData.getCategory()));
			if(count > 0) {
				return R.fail("该分类下有子分类，不能放置大屏。");
			}
		}
		return R.status(visualDataService.updateById(visualData));
	}
	
	/**
	 * 
	 * @return
	 * @throws IOException 
	 */
	@ApiLog("报表组件")
	@PostMapping("/submit")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "新增或更新大屏", notes = "返回值：大屏ID")
	@ApiResponse(message = "大屏ID", code = 200)
	public R<Integer> submit(@RequestBody String value) throws IOException {
		String decoderStr = new String(Base64.getDecoder().decode(value), "utf-8");
		VisualDto visualDto = JSON.parseObject(decoderStr, VisualDto.class);
		VisualData visualData = visualDto.getVisualData();
		visualData.setVisualDataType(0);
		visualDto.setVisualData(visualData);
		return R.data(visualDataService.doSaveOrUpdate(visualDto, getUser()));
	}

	/**
	 * 
	 * @param ids
	 * @return
	 */
	@ApiLog("报表组件")
	@PostMapping("/delete")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "删除大屏")
	public R<String> delete(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		visualDataService.doRemove(Func.toIntList(ids));
		return R.success("删除成功");
	}
	
	/**
	 * 拷贝 可视化表
	 */
	@ApiLog("报表组件")
	@PostMapping("/copy")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "拷贝大屏模板")
	public R<Integer> copy(
			@ApiParam(value = "原大屏id", required = true) @RequestParam Integer id,
			@ApiParam(value = "复制大屏名称", required = false) @RequestParam(required = false) String visualName,
			@ApiParam(value = "新分类id", required = false) @RequestParam(required = false) Integer newCategoryId,
			@ApiParam(value = "租户id") @RequestParam(required = false) String tenantId,
			@ApiParam(value = "大屏类型(默认不传递：GENERAL)", required = false) @RequestParam(required = false, value = "visualType", defaultValue = "GENERAL") VisualTypeEnum visualType) {
		String name = "[副本]";
		if (StrUtil.isNotEmpty(visualName)) {
			name = visualName;
		}
		List<Integer> visualIdList = visualDataService.doCopy(CollectionUtil.newHashSet(id), newCategoryId, null, null, visualType, getUser(), name, tenantId);
		return R.data(CollectionUtil.isNotEmpty(visualIdList) ? visualIdList.get(0) : null);
	}

	/**
	 *  根据大屏id获取组件和数据集
	 */
	@GetMapping("/component-cube")
	@ApiOperationSupport(order = 10)
	@ApiOperation(value = "根据大屏id获取组件和数据集")
	public R<List<VisualComponentCubeDataDto>> getComponentCubeDataByVisualId(Integer visualId){
		return R.data(visualDataService.ComponentCubeData(visualId));
	}

	/**
	 * 拷贝 可视化表
	 */
	@ApiLog("报表组件")
	@PostMapping("/copy-cube")
	@ApiOperationSupport(order = 9)
	@ApiOperation(value = "拷贝大屏模板和组件数据集")
	public R<Integer> copyCube(@RequestBody CopyVisualDataCubeDataVo cubeData) {
		return visualDataService.doCopyCube(cubeData,getUser());
	}
	
	/**
	 * 调整顺序
	 * @return
	 */
	@ApiLog("报表组件")
	@PostMapping("/adjustOrder")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "调整顺序", notes = "传入大屏ID列表【切记：保证大屏顺序】")
	public R<String> uptOrder(@ApiParam(value = "主键集合  1,2,3", required = true) @RequestParam String ids) {
		List<VisualData> entityList = new ArrayList<VisualData>();
		List<Integer> visualIdList = Func.toIntList(ids);
		for(int i = 0; i < visualIdList.size(); i++) {
			VisualData data = new VisualData();
			data.setId(visualIdList.get(i));
			data.setOrderIndex(i);
			entityList.add(data);
		}
		return R.status(visualDataService.updateBatchById(entityList));
	}

	/**
	 * 根据大屏id获取分类名称
	 */
	@GetMapping("/get/classify")
	@ApiOperationSupport(order = 9)
	@ApiOperation(value = "根据大屏id查询大屏分类", notes = "传入大屏 id,type:(1:大屏类，2：模板)")
	public R<VisualClassifyVO> getClassifyByVisualId(@RequestParam(required = false) String visualId,
													 @RequestParam(required = false) String type) {


		VisualClassifyVO classify = new VisualClassifyVO();
		if (StrUtil.isEmpty(type) || (!type.equals("null") && type.equals("1"))) {
			// 先查询大屏信息
			VisualData visualData = visualDataService.getById(Integer.parseInt(visualId));
			if (visualData == null || visualData.getId() < 0) {
				return null;
			}
			// 自助报表
			if (visualData.getVisualDataType() == 1) {
				// 我的收藏类型
				if (visualData.getVisualType().getValue() == VisualTypeEnum.FAVOURITE.getValue()) {
					VisualReportCategoryFavourite favourite = visualReportCategoryFavouriteService.getById(visualData.getCategory());
					classify.setId(favourite.getId());
					classify.setName(favourite.getCategoryName());
				} else {
					VisualSelfReportCategory category = visualSelfReportCategoryService.getById(visualData.getCategory());
					classify.setId(category.getId());
					classify.setName(category.getCategoryValue());
				}
			} else { // 普通大屏
				// 我的收藏类型
				if (visualData.getVisualType().getValue() == VisualTypeEnum.FAVOURITE.getValue()) {
					VisualClassifyFavourite favourite = visualClassifyFavouriteService.getById(visualData.getCategory());
					classify.setId(favourite.getId());
					classify.setName(favourite.getClassifyName());
				} else {
					VisualClassify visualClassify = visualCategoryService.getById(visualData.getCategory());
					classify.setId(visualClassify.getId());
					classify.setName(visualClassify.getCategoryValue());
				}
			}
		} else {
			VisualTemplate template = visualTemplateService.getById(visualId);
			if (template == null || template.getId() < 0) {
				return null;
			}
			VisualTemplateClassify templateClassify = visualTemplateClassifyService.getById(template.getCategory());
			classify.setId(templateClassify.getId());
			classify.setName(templateClassify.getCategoryValue());
		}

		return R.data(classify);
	}

	@PostMapping("/lock")
	@ApiOperation(value = "大屏锁")
	public VisualLockDto lLock(Integer visualId) {
		return visualDataService.visualLock(visualId);
	}

	@ApiOperation(value = "解锁")
	@PostMapping("/unlock")
	public boolean visualUnlock(@RequestBody VisualUnlockParams params) {
		return visualDataService.visualUnlock(params);
	}


}
