/**
 * 
 */
package org.springblade.visual.controller;

import java.util.Date;

import javax.validation.Valid;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.visual.entity.VisualEnum;
import org.springblade.visual.service.VisualEnumService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

/**
 * @author wookvn
 *
 */
@RestController
@RequestMapping("visual/enum")
@Api(value = "自定义枚举接口", tags = "自定义枚举接口")
public class VisualEnumController extends BladeController {

	@Autowired
	private VisualEnumService visualEnumService;
	
	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入visualEnum")
	public R<VisualEnum> detail(VisualEnum visualEnum) {
		VisualEnum detail = visualEnumService.getOne(Condition.getQueryWrapper(visualEnum).last("limit 1"));
		return R.data(detail);
	}

	/**
	 * 分页 可视化分类表
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入visualEnum")
	public R<IPage<VisualEnum>> list(VisualEnum visualEnum, Query query) {
		IPage<VisualEnum> pages = visualEnumService.doPage(visualEnum,query,getUser());
		return R.data(pages);
	}

	/**
	 * 新增或修改
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入visualEnum")
	public R<String> submit(@Valid @RequestBody VisualEnum visualEnum) {
		if(visualEnum.getId() == null) {
			visualEnum.setCreateTime(new Date());
			visualEnum.setUserId(getUser().getUserId());
			visualEnum.setUserName(getUser().getNickName());
		}else {
			visualEnum.setUpdateTime(new Date());
		}
		return R.status(visualEnumService.saveOrUpdate(visualEnum));
	}
	
	/**
	 * 删除 
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "根据ID删除", notes = "传入id")
	public R<String> remove(@ApiParam(value = "主键", required = true) @RequestParam Integer id) {
		return R.status(visualEnumService.removeById(id));
	}
}
