package org.springblade.visual.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.api.R;
import org.springblade.system.client.DataPermClient;
import org.springblade.visual.entity.VisualExternalLinkInfo;
import org.springblade.visual.service.VisualExternalLinkInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

@RestController
@RequestMapping("/external-link")
@Api(value = "自定义iframe上传接口", tags = "自定义iframe上传接口")
public class VisualExternalLinkInfoController extends BladeController {

    @Autowired
    private VisualExternalLinkInfoService visualExternalLinkInfoService;

    @Autowired
    private DataPermClient dataPermClient;

    @GetMapping("/page")
    @ApiOperation("分页查询当前登录人员上传的iframe")
    public R<Page<VisualExternalLinkInfo>> getPageList(VisualExternalLinkInfo linkInfo, Query query){

        Boolean checkSuperPermRet = dataPermClient.checkSuperPermission(getUser().getUserId());
        return R.data(visualExternalLinkInfoService.doPage(linkInfo,query,getUser(),checkSuperPermRet));
    }

    @PostMapping("/upload")
    @ApiOperation("上传文件,部署项目")
    public R<String> saveLink(@RequestParam("file") @ApiParam(value = "二进制文件流") MultipartFile file,
                                @ApiParam("租户id")String trendId,@ApiParam("文件路径")String path){
        BladeUser user = getUser();
        return R.data(visualExternalLinkInfoService.doUpload(file,user,trendId,path));
    }

    @PostMapping("/lose/link")
    @ApiOperation("禁用链接")
    public R<Boolean> loseExternalLink(Integer id){

        VisualExternalLinkInfo linkInfo = new VisualExternalLinkInfo();
        linkInfo.setId(id);
        linkInfo.setStatus(0);
        return R.data(visualExternalLinkInfoService.updateById(linkInfo));
    }

    @PostMapping("/lose/linkList")
    @ApiOperation("批量禁用链接")
    public R<Boolean> loseExternalLinkList(List<Integer> ids){

        boolean update = visualExternalLinkInfoService.update(Wrappers.<VisualExternalLinkInfo>update()
                .lambda().set(VisualExternalLinkInfo::getStatus, 0)
                .in(VisualExternalLinkInfo::getId, ids));
        return R.data(update);
    }
}
