package org.springblade.visual.controller;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.api.R;
import org.springblade.visual.entity.VisualComponent;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.service.VisualComponentService;
import org.springblade.visual.service.VisualDataService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 移动应用大屏
 */
@RestController
@RequestMapping("/mobile")
@Api(value = "移动应用大屏接口",tags = "移动应用大屏接口")
public class VisualMobileDataController extends BladeController {

    @Autowired
    private VisualDataService visualDataService;

    @Autowired
    private VisualComponentService visualComponentService;

    @GetMapping("/exchange")
    @ApiOperation("切换移动应用大屏")
    public R<VisualData> exchangeMobile(Integer id){
        // 根据id查询大屏信息
        VisualData visualData = visualDataService.getById(id);
       // 查询大屏组件信息
        LambdaQueryWrapper<VisualComponent> queryWrapper = Wrappers.<VisualComponent>lambdaQuery()
                .eq(VisualComponent::getVisualId, visualData.getId());
        List<VisualComponent> components = visualComponentService.list(queryWrapper);
        Map<Integer, List<VisualComponent>> listMap = components.stream().collect(Collectors.groupingBy(VisualComponent::getIsMobile));
        if (CollectionUtil.isNotEmpty(listMap)){
            // 普通大屏组件id
            if (CollectionUtil.isNotEmpty(listMap.get(0))) {
                List<VisualComponent> visualComponents = listMap.get(0);
                List<Integer> componentIdList = visualComponents.stream().map(VisualComponent::getId).collect(Collectors.toList());
                visualData.setComponentIds(componentIdList);
            }
            // 移动应用大屏id
            if (CollectionUtil.isNotEmpty(listMap.get(1))) {
                List<VisualComponent> visualComponents = listMap.get(1);
                List<Integer> componentIdList = visualComponents.stream().map(VisualComponent::getId).collect(Collectors.toList());
                visualData.setMobileComponentIds(componentIdList);
            }
        }

        return R.data(visualData);
    }
}
