/**
 * 
 */
package org.springblade.visual.controller;

import java.util.Arrays;
import java.util.List;

import com.alibaba.fastjson2.JSONObject;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.visual.dto.VisualDataPermissionDoo;
import org.springblade.visual.dto.VisualDataPermissionDto;
import org.springblade.visual.dto.VisualPowerPermissionDoo;
import org.springblade.visual.dto.VisualPowerPermissionDto;
import org.springblade.visual.entity.VisualPowerPointPermission;
import org.springblade.visual.service.VisualPowerPointPermissionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;

/**
 *	幻灯片权限组 操作处理
 *
 * @author wookvn
 * @date   2023-02-04
 */
@RestController
@RequestMapping("visual/powerPointPermission")
@Api(value = "幻灯片权限组相关接口", tags = "幻灯片权限组接口")
public class VisualPowerPointPermissionController extends BladeController {

	@Autowired
	private VisualPowerPointPermissionService visualPowerPointPermissionService;
	
	/**
	 * 	列表数据
	 * @param visualPowerPointPermission
	 * @return
	 */
	@GetMapping("/page")
	@ApiOperation(value = "分页查询幻灯片权限组", httpMethod = "GET")
	public R<IPage<VisualPowerPointPermission>> pageList(VisualPowerPointPermission visualPowerPointPermission, Query query){
		QueryWrapper<VisualPowerPointPermission> queryWrapper = Condition.getQueryWrapper(visualPowerPointPermission);
		return R.data(visualPowerPointPermissionService.page(Condition.getPage(query), queryWrapper));
	}
	
	/**
	 * 单条数据查询接口
	 * @param id
	 * @return
	 */
	@GetMapping("/info")
	@ApiOperation(value = "根据ID获取幻灯片权限组", httpMethod = "GET")
	public R<VisualPowerPointPermission> info(@RequestParam Integer id) {
		return R.data(visualPowerPointPermissionService.getById(id));
	}
	
	/**
	 *	修改数据接口
	 * @param visualPowerPointPermission
	 * @return
	 */
	@PostMapping("/submit")
	@ApiOperation(value = "增加或修改幻灯片权限组", httpMethod = "POST")
	public R<String> doUpdate(@Validated @RequestBody VisualPowerPointPermission visualPowerPointPermission) {
		return R.status(visualPowerPointPermissionService.saveOrUpdate(visualPowerPointPermission));
	}
	
	/**
	 * 	删除数据接口
	 * @param id
	 * @return
	 */
	@PostMapping("/delete")
	@ApiOperation(value = "根据ID删除幻灯片权限组", notes = "多个逗号分割", httpMethod = "POST")
	public R<String> doDelete(@RequestParam Integer[] id) {
		return R.status(visualPowerPointPermissionService.removeByIds(Arrays.asList(id)));
	}

	/**
	 *
	 * @param dataPermId
	 * @param visualCategoryId
	 * @return
	 */
	@GetMapping("/pptIdList")
	@ApiOperation(value = "根据权限组ID和分类ID查询幻灯片ID列表")
	public R<List<Integer>> visualIdList(@RequestParam Integer dataPermId, @RequestParam Integer visualCategoryId){
		return R.data(visualPowerPointPermissionService.listVisualId(dataPermId, visualCategoryId));
	}

	/**
	 * 权限组管理-幻灯片设置的入口
	 * @param visualPowerPermDto
	 * @return
	 */
	@PostMapping("/add/perm")
	@ApiOperation(value = "添加或修该权限组与幻灯片的关系", notes = "权限组管理-幻灯片设置的入口")
	public R<String> doSubmit(@RequestBody VisualPowerPermissionDoo visualPowerPermDto) {
		return R.status(visualPowerPointPermissionService.doSaveOrUpdate(visualPowerPermDto, getUser()));
	}

	@PostMapping("/checkPermission/{powerId}/{userId}")
	@ApiOperation(value = "判断用户是否有幻灯片权限")
	public R<Boolean> checkPermission(@PathVariable Integer powerId, @PathVariable Long userId){
		BladeUser bladeUser = new BladeUser();
		bladeUser.setUserId(userId);
		return R.data(visualPowerPointPermissionService.checkVisualDataForUser(powerId, 0L, bladeUser));
	}

	@GetMapping("/dataPermList/{powerId}")
	@ApiOperation(value = "根据幻灯片ID查询权限组列表")
	public R<List<JSONObject>> dataPermList(@PathVariable Integer powerId){
		return R.data(visualPowerPointPermissionService.listDataPerm(powerId));
	}


	/**
	 * 幻灯片列表权限组管理入口
	 * @param visualPowerPerDto
	 * @return
	 */
	@PostMapping("/perform")
	@ApiOperation(value = "批量添加或修改权限组与幻灯片关系", notes = "幻灯片列表-权限组管理入口【超级管理员才能操作】")
	public R<String> doPerform(@RequestBody VisualPowerPermissionDto visualPowerPerDto) {
		if(!AuthUtil.isAdmin()) {
			return R.fail("只能超级管理员操作");
		}
		return R.status(visualPowerPointPermissionService.doPerform(visualPowerPerDto, getUser()));
	}
	
}