/**
 * 
 */
package org.springblade.visual.controller;

import java.util.*;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import cn.hutool.extra.servlet.ServletUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.cube.dto.PreviewResult;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.service.CubeDataPermissionService;
import org.springblade.cube.service.CubeDataService;
import org.springblade.cube.service.DynamicTableService;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.visual.dto.ComponentColumn;
import org.springblade.visual.dto.QuerySqlParam;
import org.springblade.visual.dto.WarningDto;
import org.springblade.visual.dto.WarningWayDto;
import org.springblade.visual.entity.VisualWarning;
import org.springblade.visual.service.QuerySqlService;
import org.springblade.visual.service.VisualWarningService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.xxl.job.core.biz.AdminBiz;
import com.xxl.job.core.biz.model.JobInfo;
import com.xxl.job.core.biz.model.ReturnT;

import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;

import javax.servlet.http.HttpServletRequest;

/**
 *	智能预警器 操作处理
 *
 * @author wookvn
 * @date   2022-03-09
 */
@Slf4j
@RestController
@RequestMapping("visual/warning")
@Api(value = "智能预警器相关接口", tags = {"智能预警器相关接口"})
public class VisualWarningController extends BladeController {
	
	@Autowired
	private DynamicTableService dynamicTableService;

	@Autowired
	private VisualWarningService visualWarningService;
	
	@Autowired
	private QuerySqlService querySqlService;
	
	@Autowired
	private CubeDataPermissionService cubeDataPermissionService;
	
	@Autowired
	private CubeDataService cubeDataService;
	
	@Autowired
	private AdminBiz adminBiz;
	
	@Value("${xxl.job.executor.groupid}")
    private int groupId;
	
	/**
	 * 	列表数据
	 * @param visualWarning
	 * @return
	 */
	@GetMapping("/list")
	@ApiOperation(value = "分页查询智能预警器信息", httpMethod = "GET")
	public R<IPage<VisualWarning>> list(VisualWarning visualWarning, Query query){
		IPage<VisualWarning> page = visualWarningService.page(Condition.getPage(query), Condition.getQueryWrapper(visualWarning).lambda().orderByDesc(VisualWarning::getId));
		return R.data(page);
	}
	
	/**
	 * 单条数据查询接口
	 * @param oprCenterId
	 * @return
	 */
	@GetMapping("/info")
	@ApiOperation(value = "根据ID获取智能预警器信息", httpMethod = "GET")
	public R<VisualWarning> info(@RequestParam String id) {
		Pattern pattern = Pattern.compile("-?\\d+(\\.\\d+)?");
		if (StrUtil.isNotEmpty(id) && pattern.matcher(id).matches()) {
			return R.data(visualWarningService.getById(id));
		} else {
			return R.data(null);
		}
	}
	
	/**
	 * 
	 * @param componentData
	 * @return
	 * @throws Exception 
	 */
	@PostMapping("/preview")
	@ApiOperation(value = "数据预览", notes = "传入WarningDto")
	public R<PreviewResult> preview(@RequestBody String value) throws Exception{
		String decoderStr = new String(Base64.getDecoder().decode(value), "utf-8");
		WarningDto warningDto = JSON.parseObject(decoderStr, WarningDto.class);
		if(StrUtil.isEmpty(warningDto.getColumnList())) {
			return R.fail("字段列表不能为空");
		}
		CubeData cubeData = cubeDataService.getById(warningDto.getCubeDataId());
		if(cubeData == null) {
			return R.fail("数据集不存在");
		}
		//检查数据集是否有权限
		boolean checkCubePermRet = cubeDataPermissionService.checkCubePermForUser(warningDto.getCubeDataId(), cubeData.getCreateUserId(), getUser());
		if(!checkCubePermRet) {
			return R.fail("数据集没有访问权限");
		}
		R<QuerySqlParam> r = querySqlService.parseQuerySql(warningDto.getCubeDataId(), warningDto, getUser());
		if(!r.isSuccess()) {
			return R.fail(r.getMsg());
		}
		QuerySqlParam querySqlParam = r.getData();
		ReportDatasource reportDatasource = querySqlParam.getReportDatasource();
		String querySql = querySqlParam.getQuerySql();
		Map<String, String> fieldDict = querySqlParam.getSelectColumnList().stream().collect(Collectors.toMap(ComponentColumn::getColumnAliasName, ComponentColumn::getColumnTitle));
		List<Map<String, Object>> resultList = dynamicTableService.selectList(reportDatasource, querySql);
		String cubeName = querySqlParam.getCubeData() != null ? querySqlParam.getCubeData().getCubeName() : null;
		return R.data(new PreviewResult(resultList, fieldDict, cubeName), querySql);
	}
	
	/**
	 *	新增修改数据接口添加审批流程
	 * @param value
	 * @return
	 * @throws Exception 
	 *//*
	@PostMapping("/submit")
	@ApiOperation(value = "新增修改数据接口添加审批流程", httpMethod = "POST")
	public R doSubmit(@RequestBody String value, HttpServletRequest request) throws Exception {
		String decoderStr = new String(Base64.getDecoder().decode(value), "utf-8");
		VisualWarning visualWarning = JSON.parseObject(decoderStr, VisualWarning.class);
		if(visualWarning.getId() != null) {
			VisualWarning dbVisualWarning = visualWarningService.getById(visualWarning.getId());
			dbVisualWarning.setApproveStatus(0);
			visualWarningService.updateById(dbVisualWarning);
		}

		//绑定流程，发送http请求
		Map<String, Object> paramMap =new HashMap<>();
		paramMap.put("busId","");
		paramMap.put("busJson",value);
		paramMap.put("dataJson","");
		paramMap.put("flowId",visualWarning.getFlowId());
		paramMap.put("creatorId",getUser().getUserId());
		paramMap.put("creatorName",getUser().getUserName());

		//获取header
		Map<String, String> headers = new HashMap<>();
		headers.put("Authorization","Basic c2FiZXI6c2FiZXJfc2VjcmV0");
		headers.put("Blade-Auth", ServletUtil.getHeader(request,"Blade-Auth","UTF-8"));

		String resBody = HttpUtil.createPost("https://windata.platomix.net/flow-api/flow/bus/save").addHeaders(headers).form(paramMap).execute().body();
//		String resBody = HttpUtil.createPost("localhost:6464/flow/bus/save").addHeaders(headers).form(paramMap).execute().body();
		log.info("resBody:"+resBody);
		JSONObject result = JSONObject.parseObject(resBody);
		if (result.getString("code").equals("200")){
			return R.success("正在等待审批，请稍等!");
		}else {
			return R.fail("流程创建失败!");
		}
	}
	*//**
	 *	审核之后回调接口
	 * @param busJson
	 * @return
	 * @throws Exception
	 *//*
	@PostMapping("/doSubmit")
	@ApiOperation(value = "增加或更新智能预警器", httpMethod = "POST")
	public R<Integer> doSubmit(String busJson) throws Exception {
		String decoderStr = new String(Base64.getDecoder().decode(busJson), "utf-8");
		VisualWarning visualWarning = JSON.parseObject(decoderStr, VisualWarning.class);
		int jobId = 0; boolean uptJob = true;
		visualWarning.setApproveStatus(1);//审批通过
		visualWarning.setUpdateTime(new Date());
		if(visualWarning.getId() == null) {
			visualWarning.setCreateUserId(getUser().getUserId());
			visualWarning.setCreateBy(getUser().getNickName());
			visualWarning.setCreateTime(new Date());
		}else {
			VisualWarning dbVisualWarning = visualWarningService.getById(visualWarning.getId());
			if(dbVisualWarning != null && dbVisualWarning.getJobId() != null) {
				jobId = dbVisualWarning.getJobId();
				uptJob = !visualWarning.getRequestFreq().equals(dbVisualWarning.getRequestFreq());
			}
		}
		try {
			WarningDto warningDto = new WarningDto();
			BeanUtils.copyProperties(visualWarning, warningDto);
			warningDto.setColumnList(visualWarning.getColumnJson());
			warningDto.setFilterList(visualWarning.getFilterJson());
			warningDto.setFilterRelation(visualWarning.getFilterRelationJson());
			warningDto.setSortList(visualWarning.getSortJson());
			R<QuerySqlParam> r = querySqlService.parseQuerySql(warningDto.getCubeDataId(), warningDto, getUser());
			if(!r.isSuccess()) { //校验参数是否可以正常解析SQL
				return R.fail(r.getMsg());
			}
			//解析使用到的字段ID列表
			Set<Integer> usedColumnIdList = querySqlService.parseUsedColumnIdList(warningDto);
			visualWarning.setColumnIdList(JSON.toJSONString(usedColumnIdList));
			boolean _ret = visualWarningService.saveOrUpdate(visualWarning);
			if(_ret && uptJob) {
				ReturnT<String> ret = _handleJob(visualWarning, getUser().getUserId(), jobId);
				log.info("智能预警|新增或修改xxljob任务|waringId={},result={}", visualWarning.getId(), ret);
				if (jobId == 0 && ReturnT.SUCCESS_CODE == ret.getCode()){
					//首次添加任务，需要把任务ID更新进去
					VisualWarning entity = new VisualWarning();
					entity.setId(visualWarning.getId());
					entity.setJobId(Integer.valueOf(ret.getContent()));
					visualWarningService.updateById(entity);
				}
			}
		} catch (Exception e) {
			log.error(e.getMessage(), e);
		}
		return R.data(visualWarning.getId());
	}*/

	/**
	 *	新增修改数据接口
	 * @return
	 * @throws Exception
	 */
	@PostMapping("/submit")
	@ApiOperation(value = "增加或更新智能预警器", httpMethod = "POST")
	public R<Integer> doSubmit(@RequestBody String value) throws Exception {
		String decoderStr = new String(Base64.getDecoder().decode(value), "utf-8");
		VisualWarning visualWarning = JSON.parseObject(decoderStr, VisualWarning.class);
		int jobId = 0; boolean uptJob = true;
		visualWarning.setUpdateTime(new Date());
		if(visualWarning.getId() == null) {
			visualWarning.setCreateUserId(getUser().getUserId());
			visualWarning.setCreateBy(getUser().getNickName());
			visualWarning.setCreateTime(new Date());
		}else {
			VisualWarning dbVisualWarning = visualWarningService.getById(visualWarning.getId());
			if(dbVisualWarning != null && dbVisualWarning.getJobId() != null) {
				jobId = dbVisualWarning.getJobId();
				uptJob = !visualWarning.getRequestFreq().equals(dbVisualWarning.getRequestFreq());
			}
		}
		try {
			WarningDto warningDto = new WarningDto();
			BeanUtils.copyProperties(visualWarning, warningDto);
			warningDto.setColumnList(visualWarning.getColumnJson());
			warningDto.setFilterList(visualWarning.getFilterJson());
			warningDto.setFilterRelation(visualWarning.getFilterRelationJson());
			warningDto.setSortList(visualWarning.getSortJson());
			R<QuerySqlParam> r = querySqlService.parseQuerySql(warningDto.getCubeDataId(), warningDto, getUser());
			if(!r.isSuccess()) { //校验参数是否可以正常解析SQL
				return R.fail(r.getMsg());
			}
			//解析使用到的字段ID列表
			Set<Integer> usedColumnIdList = querySqlService.parseUsedColumnIdList(warningDto);
			visualWarning.setColumnIdList(JSON.toJSONString(usedColumnIdList));
			boolean _ret = visualWarningService.saveOrUpdate(visualWarning);
			if(_ret && uptJob) {
				ReturnT<String> ret = _handleJob(visualWarning, getUser().getUserId(), jobId);
				log.info("智能预警|新增或修改xxljob任务|waringId={},result={}", visualWarning.getId(), ret);
				if (jobId == 0 && ReturnT.SUCCESS_CODE == ret.getCode()){
					//首次添加任务，需要把任务ID更新进去
					VisualWarning entity = new VisualWarning();
					entity.setId(visualWarning.getId());
					entity.setJobId(Integer.valueOf(ret.getContent()));
					visualWarningService.updateById(entity);
				}
			}
		} catch (Exception e) {
			log.error(e.getMessage(), e);
		}
		return R.data(visualWarning.getId());
	}

	/**
	 *
	 * @param visualWarning
	 * @return
	 */
	@PostMapping("/update")
	@ApiOperation(value = "根据ID更新智能预警器", httpMethod = "POST")
	public R<String> doUpdate(@RequestBody VisualWarning visualWarning){
		if(visualWarning == null || visualWarning.getId() == null) {
			return R.fail("id不能为空");
		}
		if(visualWarning.getEnableStatus() != null) {
			VisualWarning dbVisualWarning = visualWarningService.getById(visualWarning.getId());
			if(dbVisualWarning != null && dbVisualWarning.getJobId() != null) {
				if(visualWarning.getEnableStatus() == 1) { //开启
					ReturnT<String> returnT = adminBiz.jobStart(dbVisualWarning.getJobId());
					log.info("智能预警|[开启]xxljob任务|waringId={},result={}", visualWarning.getId(), returnT);
				}else { //关闭
					ReturnT<String> returnT = adminBiz.jobStop(dbVisualWarning.getJobId());
					log.info("智能预警|[关闭]xxljob任务|waringId={},result={}", visualWarning.getId(), returnT);
				}
			}
		}
		return R.status(visualWarningService.updateById(visualWarning));
	}



	/**
	 *	驳回之后回调接口
	 * @param busJson
	 * @return
	 * @throws Exception
	 */
	@PostMapping("/doReject")
	@ApiOperation(value = "驳回之后回调接口", httpMethod = "POST")
	public R<Integer> doReject(String busJson) throws Exception {
		String decoderStr = new String(Base64.getDecoder().decode(busJson), "utf-8");
		VisualWarning visualWarning = JSON.parseObject(decoderStr, VisualWarning.class);
		if(visualWarning.getId() != null) {
			VisualWarning dbVisualWarning = visualWarningService.getById(visualWarning.getId());
			dbVisualWarning.setApproveStatus(2);
			visualWarningService.updateById(dbVisualWarning);
		}
		return R.data(visualWarning.getId());
	}



	
//	/**
//	 *
//	 * @param visualWarning
//	 * @return
//	 */
//	@PostMapping("/update")
//	@ApiOperation(value = "根据ID更新智能预警器", httpMethod = "POST")
//	public R<String> update(@RequestBody VisualWarning visualWarning, HttpServletRequest request) {
//		if (visualWarning == null || visualWarning.getId() == null) {
//			return R.fail("id不能为空");
//		}
//		//修改审批状态
//		VisualWarning dbVisualWarning = visualWarningService.getById(visualWarning.getId());
//		dbVisualWarning.setApproveStatus(0);
//		visualWarningService.updateById(dbVisualWarning);
//		//绑定流程，发送http请求
//		Map<String, Object> paramMap = new HashMap<>();
//		paramMap.put("busId", visualWarning.getId());
//		paramMap.put("busJson", new String(Base64.getEncoder().encode(JSONObject.toJSONString(dbVisualWarning).getBytes())));
//		paramMap.put("dataJson", new String(Base64.getEncoder().encode(JSONObject.toJSONString(visualWarning).getBytes())));
//		paramMap.put("flowId", visualWarning.getFlowId());
//		paramMap.put("creatorId", getUser().getUserId());
//		paramMap.put("creatorName", getUser().getUserName());
//
//		//获取header
//		Map<String, String> headers = new HashMap<>();
//		headers.put("Authorization", "Basic c2FiZXI6c2FiZXJfc2VjcmV0");
//		headers.put("Blade-Auth", ServletUtil.getHeader(request, "Blade-Auth", "UTF-8"));
//
//		String resBody = HttpUtil.createPost("https://windata.platomix.net/flow-api/flow/bus/save").addHeaders(headers).form(paramMap).execute().body();
////		String resBody = HttpUtil.createPost("localhost:6464/flow/bus/save").addHeaders(headers).form(paramMap).execute().body();
//		log.info("resBody:" + resBody);
//		JSONObject result = JSONObject.parseObject(resBody);
//		if (result.getString("code").equals("200")) {
//			return R.success("正在等待审批，请稍等!");
//		} else {
//			return R.fail("流程创建失败!");
//		}


//
//			VisualWarning dbVisualWarning = visualWarningService.getById(visualWarning.getId());
//			if(dbVisualWarning != null && dbVisualWarning.getJobId() != null) {
//				if(visualWarning.getEnableStatus() == 1) { //开启
//					ReturnT<String> returnT = adminBiz.jobStart(dbVisualWarning.getJobId());
//					log.info("智能预警|[开启]xxljob任务|waringId={},result={}", visualWarning.getId(), returnT);
//				}else { //关闭
//					ReturnT<String> returnT = adminBiz.jobStop(dbVisualWarning.getJobId());
//					log.info("智能预警|[关闭]xxljob任务|waringId={},result={}", visualWarning.getId(), returnT);
//				}
//			}
//		}
//		return R.status(visualWarningService.updateById(visualWarning));

//	}

	/**
	 * 	执行修改预警器状态数据
	 * @param
	 * @return
	 */
	@PostMapping("/doUpdate")
	@ApiOperation(value = "执行修改预警器状态数据", httpMethod = "POST")
	public R<String> doUpdate(String busId) {
		VisualWarning dbVisualWarning = visualWarningService.getById(busId);
		dbVisualWarning.setApproveStatus(1);
		if(dbVisualWarning != null && dbVisualWarning.getJobId() != null) {
			if(dbVisualWarning.getEnableStatus() == 1) {
				dbVisualWarning.setEnableStatus(0);
				ReturnT<String> returnT = adminBiz.jobStop(dbVisualWarning.getJobId());
				log.info("智能预警|[关闭]xxljob任务|waringId={},result={}",busId, returnT);

			}else {
				dbVisualWarning.setEnableStatus(1);
				ReturnT<String> returnT = adminBiz.jobStart(dbVisualWarning.getJobId());
				log.info("智能预警|[开启]xxljob任务|waringId={},result={}", busId, returnT);
			}
		}
		return R.status(visualWarningService.updateById(dbVisualWarning));
	}

	/**
	 * 	删除数据接口
	 * @param id
	 * @return
	 */
	@PostMapping("/remove")
	@ApiOperation(value = "根据ID数组删除智能预警器(多个逗号分割)", httpMethod = "POST")
	public R<String> doRemove(@ApiParam(value = "主键", required = true) @RequestParam String ids) {
		return R.status(visualWarningService.removeByIds(Func.toIntList(ids)));
	}
	
//	/**
//	 * 	删除数据接口
//	 * @param id
//	 * @return
//	 */
//	@PostMapping("/remove")
//	@ApiOperation(value = "删除数据", httpMethod = "POST")
//	public R doRemove(@ApiParam(value = "主键", required = true) @RequestParam String id,
//					  @ApiParam(value = "流程id", required = true) @RequestParam String flowId,
//					  HttpServletRequest request) {
//		VisualWarning visualWarning = visualWarningService.getById(id);
//		if (visualWarning==null){
//			return R.fail("预警器不存在!");
//		}
//		visualWarning.setApproveStatus(0);//审核中
//		visualWarningService.updateById(visualWarning);
//		//绑定流程，发送http请求
//		Map<String, Object> paramMap =new HashMap<>();
//		paramMap.put("busId",id);
//		paramMap.put("busJson",new String(Base64.getEncoder().encode(JSONObject.toJSONString(visualWarning).getBytes())));
//		paramMap.put("dataJson","");
//		paramMap.put("flowId",flowId);
//		paramMap.put("creatorId",getUser().getUserId());
//		paramMap.put("creatorName",getUser().getUserName());
//
//		//获取header
//		Map<String, String> headers = new HashMap<>();
//		headers.put("Authorization","Basic c2FiZXI6c2FiZXJfc2VjcmV0");
//		headers.put("Blade-Auth", ServletUtil.getHeader(request,"Blade-Auth","UTF-8"));
//
//		String resBody = HttpUtil.createPost("https://windata.platomix.net/flow-api/flow/bus/save").addHeaders(headers).form(paramMap).execute().body();
////		String resBody = HttpUtil.createPost("localhost:6464/flow/bus/save").addHeaders(headers).form(paramMap).execute().body();
//		log.info("resBody:"+resBody);
//		JSONObject result = JSONObject.parseObject(resBody);
//		if (result.getString("code").equals("200")){
//			return R.success("正在等待审批，请稍等!");
//		}else {
//			return R.fail("流程创建失败!");
//		}
//
//
//	}
//
//	/**
//	 * 	执行删除数据
//	 * @param
//	 * @return
//	 */
//	@PostMapping("/doRemove")
//	@ApiOperation(value = "执行删除数据", httpMethod = "POST")
//	public R<String> doRemove(String busId) {
//		return R.status(visualWarningService.removeById(busId));
//	}
//
//
//	/**
//	 * 	删除操作审核拒绝
//	 * @param
//	 * @return
//	 */
//	@PostMapping("/doRemoveReject")
//	@ApiOperation(value = "执行删除数据", httpMethod = "POST")
//	public R<String> doRemoveReject(String busId) {
//		return R.status(visualWarningService.removeById(busId));
//	}

	/**
	 * 处理xxljob任务
	 * @param visualSubscribe
	 * @return
	 */
	private ReturnT<String> _handleJob(VisualWarning visualWarning, Long userId, int jobId) {
		JobInfo xxlJobInfo = new JobInfo();
		xxlJobInfo.setJobGroup(groupId);
		xxlJobInfo.setJobDesc("智能预警:"+visualWarning.getName());
		xxlJobInfo.setAuthor(userId.toString());
		xxlJobInfo.setScheduleType("FIX_RATE"); //调度类型：固定速度【以固定速度，触发任务调度；按照固定的间隔时间，周期性触发】
		xxlJobInfo.setScheduleConf(visualWarning.getRequestFreq().toString());
		xxlJobInfo.setMisfireStrategy("DO_NOTHING");  //调度过期策略：忽略【调度过期后，忽略过期的任务，从当前时间开始重新计算下次触发时间】
		xxlJobInfo.setExecutorRouteStrategy("ROUND"); //路由策略：轮询
		xxlJobInfo.setExecutorBlockStrategy("SERIAL_EXECUTION"); //阻塞处理策略：单机串行【调度请求进入单机执行器后，调度请求进入FIFO队列并以串行方式运行】
		xxlJobInfo.setExecutorHandler("smartWarningJobHandler");
		xxlJobInfo.setExecutorParam("" + visualWarning.getId());
		xxlJobInfo.setExecutorTimeout(100); //超时时间2分钟
		xxlJobInfo.setExecutorFailRetryCount(0);
		xxlJobInfo.setGlueUpdatetime(new Date());
		xxlJobInfo.setGlueType("BEAN");
		xxlJobInfo.setTriggerLastTime(0);
		xxlJobInfo.setTriggerNextTime(0);
		if(jobId == 0) { //新增
			xxlJobInfo.setAddTime(new Date());
			xxlJobInfo.setTriggerStatus(1);//运行
			return adminBiz.jobAdd(xxlJobInfo);
		}else {
			xxlJobInfo.setId(jobId);
			return adminBiz.jobUpdate(xxlJobInfo);
		}
	}
}