/**
 * 
 */
package org.springblade.visual.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.api.R;
import org.springblade.cube.enums.AggregationTypeEnum;
import org.springblade.cube.enums.ColumnAttrEnum;
import org.springblade.cube.enums.DimensionTypeEnum;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.service.CubeDataPermissionService;
import org.springblade.cube.service.CubeDataService;
import org.springblade.cube.utils.CubeConstants;
import org.springblade.cube.utils.CubeUtils;
import org.springblade.datasource.context.DialectContext;
import org.springblade.datasource.enums.DateFormatEnum;
import org.springblade.datasource.enums.DsTypeEnum;
import org.springblade.datasource.enums.FunctionMethodEnum;
import org.springblade.datasource.enums.NumberFormatEnum;
import org.springblade.datasource.enums.ViewColumnTypeEnum;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.service.IReportDatasourceService;
import org.springblade.usergroup.dto.DefineGroupFilter;
import org.springblade.usergroup.dto.DefineRuleDto;
import org.springblade.visual.dto.ComponentColumn;
import org.springblade.visual.dto.ComponentFilter;
import org.springblade.visual.dto.ComponentFilterCondition;
import org.springblade.visual.dto.CubePermParseDto;
import org.springblade.visual.dto.QuerySqlParam;
import org.springblade.visual.enums.ComponentTypeEnum;
import org.springblade.visual.enums.FilterConnectTimeEnum;
import org.springblade.visual.enums.FilterConnectTypeEnum;
import org.springblade.visual.utils.Tuple2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;

import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Slf4j
@Service
public class DefineGroupSqlService {
	
	@Autowired
	private CubeDataService cubeDataService;

	@Autowired
	private IReportDatasourceService reportDatasourceService;

	@Autowired
	private CubeDataPermissionService cubeDataPermissionService;
	
	@Autowired
	private DialectContext dialectContext;

	/**
	 * 常规组件 - 解析自定义人群查询SQL
	 * @return
	 */
	public R<QuerySqlParam> parseDefineGroupQuerySql(Integer cubeDataId, DefineRuleDto defineRuleDto, BladeUser bladeUser) {
		return parseDefineGroupQuerySql(cubeDataId, defineRuleDto, bladeUser, ComponentTypeEnum.GENERAL);
	}

	/**
	 * 解析自定义人群查询SQL
	 * @param cubeDataId
	 * @param defineRuleDto
	 * @param bladeUser
	 * @return
	 */
	public R<QuerySqlParam> parseDefineGroupQuerySql(Integer cubeDataId, DefineRuleDto defineRuleDto, BladeUser bladeUser, ComponentTypeEnum compType) {
		CubeData cubeData = cubeDataService.getById(cubeDataId);
		if(cubeData == null) {
			log.error("数据集不存在|id={}", cubeDataId);
			return R.fail("数据集不存在");
		}
		//2.查询数据源
		ReportDatasource reportDatasource = reportDatasourceService.getById(cubeData.getDatasourceId());
		if(reportDatasource == null) {
			log.error("数据源不存在|id={}", cubeData.getDatasourceId());
			return R.fail("数据源不存在");
		}
		DsTypeEnum dsTypeEnum = DsTypeEnum.resolve(reportDatasource.getDsType());
		//字段ID列表
		Set<Integer> columnIdList = new HashSet<Integer>();

		String fromSql = cubeData.getQuerySql();
		//处理电话号码字段
		List<ComponentColumn> phoneFieldList= JSON.parseArray(defineRuleDto.getPhoneField(), ComponentColumn.class);

		//构造Select模块
		List<ComponentColumn> columnList = JSON.parseArray(defineRuleDto.getColumns(), ComponentColumn.class);

		//将手机号字段放到最开始位置
		columnList.add(0,phoneFieldList.get(0));
		//初始化别名
		for(int i = 0; i < columnList.size(); i++) {
			ComponentColumn item = columnList.get(i);
			item.setColumnAliasName(CubeConstants.COLUMN_ALIAS_NAME_PREFIX + i);
			columnIdList.add(item.getCubeColumnId());
		}
		Map<Integer, ComponentColumn> columnMap = columnList.stream().collect(Collectors.toMap(ComponentColumn::getCubeColumnId, t -> t, (v1,v2) -> v1));
		List<String> selectSqlList = new ArrayList<String>();
		List<String> groupBySqlList = new ArrayList<String>();
		//主要用于排序时，不能使用带有%的字符串
		Map<Integer, String> originalNumberColumnMap = new HashMap<Integer, String>();
		//用于存储having的aggregationType
		Map<Integer, AggregationTypeEnum> havingMap = new HashMap<Integer, AggregationTypeEnum>();
		for(ComponentColumn column : columnList) {
			String columnName = "";
			if(compType == ComponentTypeEnum.DETAIL && StrUtil.containsAnyIgnoreCase(column.getColumnSql(), CubeConstants.AGGREGATION_FUNCTIONS)) {
				return R.fail("清单组件不能包含聚合函数，请检查数据集字段【"+column.getColumnTitle()+"】");
			}
			//如果清单组件并且包含聚合函数，字段不参与到select中
			String selectColumnName = columnName = column.getColumnSql();
			//聚合函数
			if(column.getColumnAttr()!= ColumnAttrEnum.VIRTUAL && column.getAggregationType() != null && column.getDimensionType() == DimensionTypeEnum.ZHIBIAO && compType != ComponentTypeEnum.DETAIL) {
				selectColumnName = columnName = String.format(column.getAggregationType().getMethod(), selectColumnName);
				havingMap.put(column.getCubeColumnId(),column.getAggregationType());
			}
			// 格式化日期
			if (column.getTimeDataFormat() != null
					&& column.getTimeDataFormat() != DateFormatEnum.yyyy_MM_ddhh$mm$ss
					&& column.getViewDataType() == ViewColumnTypeEnum.DATETIME) {
//				selectColumnName = columnName = CubeUtils.formatTimeColumn(column.getTimeDataFormat(), dsTypeEnum, selectColumnName);
				selectColumnName = columnName = dialectContext.getDialect(dsTypeEnum).formatDateTime(column.getTimeDataFormat(), selectColumnName);
			}
			// 格式化数字
			if (column.getNumberDataFormat() != null
					&& column.getViewDataType() == ViewColumnTypeEnum.NUMBER) {
				originalNumberColumnMap.put(column.getCubeColumnId(), selectColumnName);
				//groupby不需要百分号
				selectColumnName = columnName = String.format(column.getNumberDataFormat().getMethod(), selectColumnName);
				//select显示拼接百分号
//				selectColumnName = CubeUtils.percentConcat(column.getNumberDataFormat(), dsTypeEnum, selectColumnName);
				if(NumberFormatEnum.isPercentFormat(column.getNumberDataFormat())) {
					selectColumnName = dialectContext.getDialect(dsTypeEnum).formatMethodFunction(FunctionMethodEnum.NUMBER_JOINT_PERCENT, selectColumnName, column.getNumberDataFormat().getOracleFmFormat());
				}
			}
			selectSqlList.add(selectColumnName + " as \"" + column.getColumnAliasName() + "\"");
			//如果是维度，添加到groupBy
			if(column.getDimensionType() == DimensionTypeEnum.WEIDU && compType != ComponentTypeEnum.DETAIL) {
				//ORACLE的groupBy不能使用别名，clickhouse和mysql可行
				ComponentColumn componentColumn = columnMap.get(column.getCubeColumnId());
				if(componentColumn != null && (dsTypeEnum == DsTypeEnum.clickhouse || dsTypeEnum == DsTypeEnum.mysql)) {
					groupBySqlList.add(componentColumn.getColumnAliasName());
				}else {
					groupBySqlList.add(columnName);
				}
			}
		}
		StringBuilder strBuilder = new StringBuilder("select ").append(String.join(",", selectSqlList))
				.append(" from ").append(fromSql);

		List<String> whereSqlList = new ArrayList<>();
		//解析权限组全局SQL
		CubePermParseDto cubePermParseDto = cubeDataPermissionService.parseCubePermission(cubeDataId, bladeUser);
		if(StrUtil.isNotBlank(cubePermParseDto.getFilterSql())) {
			whereSqlList.add(cubePermParseDto.getFilterSql());
		}
		//构建where条件
		String whereSql="";
		if (StringUtils.isNotBlank(defineRuleDto.getFilter())){
			DefineGroupFilter filter = JSONObject.parseObject(defineRuleDto.getFilter(), DefineGroupFilter.class);
			List<ComponentFilter> filterList = JSON.parseArray(filter.getConditions(),ComponentFilter.class);
			filterList = Optional.ofNullable(filterList).orElse(new ArrayList<ComponentFilter>());
			//排除掉虚拟字段并且带聚合函数
			List<ComponentFilter> componentFilters =new ArrayList<>();
			for(ComponentFilter columnFilter : filterList) {
				ComponentColumn column = columnFilter.getColumn();
				if (column.getColumnAttr()!= ColumnAttrEnum.VIRTUAL && column.getAggregationType() != null){
					if (havingMap.containsKey(column.getCubeColumnId())){
						AggregationTypeEnum method = havingMap.get(column.getCubeColumnId());
						column.setAggregationType(method);
					}
					componentFilters.add(columnFilter);
				}
			}
			Tuple2<String, String> whereTuple = parseDefineFilters(componentFilters, filter.getRelation(),filter.getFilters(), dsTypeEnum);
			whereSql = whereTuple.getT1();
		}
		//构建having条件
		String havingSql="";
		if (StringUtils.isNotBlank(defineRuleDto.getHaving())){
			DefineGroupFilter having = JSONObject.parseObject(defineRuleDto.getHaving(), DefineGroupFilter.class);
			List<ComponentFilter> havingList = JSON.parseArray(having.getConditions(),ComponentFilter.class);
			havingList = Optional.ofNullable(havingList).orElse(new ArrayList<ComponentFilter>());
			Tuple2<String, String> havingTuple = parseDefineFilters(havingList, having.getRelation(),having.getFilters(), dsTypeEnum);
			havingSql = havingTuple.getT2();
		}
		//where子句
		if(StringUtils.isNotBlank(whereSql)) {
			whereSqlList.add(whereSql);
		}
		if(!whereSqlList.isEmpty()) {
			strBuilder.append(" where " + String.join(" AND ", whereSqlList));
		}
//		清单组件不需要groupBy和having
		if(compType != ComponentTypeEnum.DETAIL) {
			//groupby子句【不能使用别名】
			if(!groupBySqlList.isEmpty()) {
				strBuilder.append(" group by ").append(String.join(",", groupBySqlList));
			}
			//having子句
			if(StringUtils.isNotBlank(havingSql)) {
				strBuilder.append(" having " + havingSql);
			}
		}
//		log.info("=====>>querySQL:" + strBuilder.toString());
		if(CubeUtils.checkSqlInject(strBuilder.toString())) {
			return R.fail("有SQL注入风险，请检查");
		}
		return R.data(new QuerySqlParam(strBuilder.toString(), cubeData, reportDatasource, columnList, cubePermParseDto.getExcludeColumnIdList(),null,null,null));
	}
	
	/**
	 * 解析自定义人群过滤器
	 * @return
	 */
	public Tuple2<String, String> parseDefineFilters(List<ComponentFilter> filterList, FilterConnectTypeEnum relation,String filters, DsTypeEnum dsTypeEnum){
		String whereSql = "", havingSql = "";
		if(filterList != null && !filterList.isEmpty()) {

			//过滤关系
			if(relation != null) {
				List<String> havingSqlList = new ArrayList<String>();
				//1.构造单个条件集合
				List<String> whereSqlList = new ArrayList<String>();

				Tuple2<String, String> tuple = _buildConditions(filterList, relation, dsTypeEnum);
				if(StringUtils.isNotBlank(tuple.getT1())) {
					whereSqlList.add(tuple.getT1());
				}
				if(StringUtils.isNotBlank(tuple.getT2())){
					havingSqlList.add(tuple.getT2());
				}
				//2.构造多个条件集合
				List<DefineGroupFilter> componentFilterList =JSON.parseArray(filters,DefineGroupFilter.class);
				if (componentFilterList != null && componentFilterList.size()>0){
					for (DefineGroupFilter filter : componentFilterList){
						List<ComponentFilter> componentFilters = JSON.parseArray(filter.getConditions(),ComponentFilter.class);
						FilterConnectTypeEnum connectTypeEnum = filter.getRelation();
						if (componentFilters != null && componentFilters.size()>0  && connectTypeEnum!= null){
							Tuple2<String, String> tuple2 = _buildConditions(componentFilters, connectTypeEnum, dsTypeEnum);
							if(StringUtils.isNotBlank(tuple2.getT1())) {
								whereSqlList.add(tuple2.getT1());
							}
							if(StrUtil.isNotBlank(tuple.getT2())){
								havingSqlList.add(tuple.getT2());
							}
						}
					}
				}
				whereSql =_stringListJoin(whereSqlList, relation);
				havingSql=_stringListJoin(havingSqlList, relation);
			}else { //没有过滤关系
				Tuple2<String, String> whereHavingSqlTuple = _buildConditions(filterList, FilterConnectTypeEnum.AND, dsTypeEnum);
				if(whereHavingSqlTuple != null) {
					whereSql = whereHavingSqlTuple.getT1();
					havingSql = whereHavingSqlTuple.getT2();
				}
			}
		}
		return new Tuple2<String, String>(whereSql, havingSql);
	}
	
	/**
	 * 构造查询条件
	 * @param compFilterList
	 * @param relation
	 * @param dsTypeEnum
	 * @return
	 */
	private Tuple2<String, String> _buildConditions(List<ComponentFilter> compFilterList, FilterConnectTypeEnum relation, DsTypeEnum dsTypeEnum) {
		List<String> whereSqlList = new ArrayList<String>();
		List<String> havingSqlList = new ArrayList<String>();
		for(ComponentFilter columnFilter : compFilterList) {
			ComponentColumn column = columnFilter.getColumn();
			String columnName = column.getColumnSql();
			//2.计算过滤时机
			FilterConnectTimeEnum connectTime = columnFilter.getConnectTime();
			//HAVING需要把聚合函数参与进来[如果计算字段已存在聚合函数，不再添加]
			if(connectTime == FilterConnectTimeEnum.HAVING && column.getAggregationType() != null
					&& Optional.<Integer>ofNullable(column.getVirtualAggregationStatus()).orElse(0) == 0) {
				columnName = String.format(column.getAggregationType().getMethod(), columnName);
			}
			//格式化日期
			if (column.getTimeDataFormat() != null
//					&& column.getTimeDataFormat() != DateFormatEnum.yyyy_MM_ddhh$mm$ss
					&& column.getViewDataType() == ViewColumnTypeEnum.DATETIME) {
//				columnName = CubeUtils.formatTimeColumn(column.getTimeDataFormat(), dsTypeEnum, columnName);
				columnName = dialectContext.getDialect(dsTypeEnum).formatDateTime(column.getTimeDataFormat(), columnName);
			}
			//格式化数字
//			if (column.getNumberDataFormat() != null
//					&& column.getViewDataType() == ViewColumnTypeEnum.NUMBER) {
//				columnName = String.format(column.getNumberDataFormat().getMethod(), columnName);
//			}
			//判断columnName是否为空
			if(StrUtil.isEmpty(columnName)) {
				continue;
			}
			//3.计算过滤条件
			List<ComponentFilterCondition> conditions = columnFilter.getConditions();
			if(conditions != null && !conditions.isEmpty()) {
				List<String> conditionSqlList = new ArrayList<String>();
				//4.添加过滤条件
				for(ComponentFilterCondition condition : conditions) {
					String conditionSql = condition.getFunction().handle(columnName, column.getViewDataType(), condition.getValues(), dsTypeEnum);
					if(!StrUtil.isEmpty(conditionSql)) {
						conditionSqlList.add(conditionSql);
					}
				}
				if(!conditionSqlList.isEmpty()) {
					//不同的过滤时机添加到对应的集合
					if(connectTime == FilterConnectTimeEnum.HAVING) {
						havingSqlList.add(_stringListJoin(conditionSqlList, columnFilter.getConnectType()));
					}else {
						whereSqlList.add(_stringListJoin(conditionSqlList, columnFilter.getConnectType()));
					}
				}
			}
		}
		return new Tuple2<String, String>(_stringListJoin(whereSqlList, relation), _stringListJoin(havingSqlList, relation));
	}

	private String _stringListJoin(List<String> dataList, FilterConnectTypeEnum relation) {
		if(dataList == null || dataList.isEmpty()) {
			return "";
		}
		String startParenthesis = dataList.size() > 1 ? "(" : "";
		String endParenthesis = dataList.size() > 1 ? ")" : "";
		return startParenthesis + String.join(" " + relation.name() + " ", dataList) + endParenthesis;
	}
}
