/**
 *
 */
package org.springblade.visual.service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.cube.enums.DimensionTypeEnum;
import org.springblade.cube.enums.ParamTypeEnum;
import org.springblade.cube.enums.WeiduTypeEnum;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.model.CubeGlobalParam;
import org.springblade.cube.service.CubeDataPermissionService;
import org.springblade.cube.service.CubeDataService;
import org.springblade.cube.service.CubeGlobalParamService;
import org.springblade.cube.utils.CubeConstants;
import org.springblade.cube.utils.CubeUtils;
import org.springblade.datasource.context.DialectContext;
import org.springblade.datasource.enums.DateFormatEnum;
import org.springblade.datasource.enums.DsTypeEnum;
import org.springblade.datasource.enums.FunctionMethodEnum;
import org.springblade.datasource.enums.NumberFormatEnum;
import org.springblade.datasource.enums.ViewColumnTypeEnum;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.service.IReportDatasourceService;
import org.springblade.visual.dto.*;
import org.springblade.visual.enums.CommonFunctionEnum;
import org.springblade.visual.enums.ComponentTypeEnum;
import org.springblade.visual.enums.FilterConnectTimeEnum;
import org.springblade.visual.enums.FilterConnectTypeEnum;
import org.springblade.visual.enums.FilterTimeCalcEnum;
import org.springblade.visual.utils.DateCompleteSqlUtils;
import org.springblade.visual.utils.Tuple2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Tuple;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 */
@Slf4j
@Service
public class QuerySqlService {

    @Autowired
    private CubeDataService cubeDataService;

    @Autowired
    private IReportDatasourceService reportDatasourceService;

    @Autowired
    private CubeDataPermissionService cubeDataPermissionService;

    @Autowired
    private DialectContext dialectContext;
    
    @Autowired
    private CubeGlobalParamService cubeGlobalParamService;

    /**
     * 常规组件 - 解析查询SQL
     *
     * @param cubeDataId
     * @param previewDto
     * @return
     */
    public R<QuerySqlParam> parseQuerySql(Integer cubeDataId, PreviewBaseDto previewDto, BladeUser bladeUser) {
        return parseQuerySql(cubeDataId, previewDto, bladeUser, ComponentTypeEnum.GENERAL);
    }

    /**
     * 解析查询SQL
     *
     * @param cubeDataId
     * @param previewDto
     * @return
     */
    public R<QuerySqlParam> parseQuerySql(Integer cubeDataId, PreviewBaseDto previewDto, BladeUser bladeUser, ComponentTypeEnum compType) {
        CubeData cubeData = cubeDataService.getById(cubeDataId);
        if (cubeData == null) {
            log.error("数据集不存在|id={}", cubeDataId);
            return R.fail("数据集不存在");
        }
        //2.查询数据源
        ReportDatasource reportDatasource = reportDatasourceService.getById(cubeData.getDatasourceId());
        if (reportDatasource == null) {
            log.error("数据源不存在|id={}", cubeData.getDatasourceId());
            return R.fail("数据源不存在");
        }
        return parseQuerySql(cubeData, reportDatasource, previewDto, bladeUser, compType);
    }

    /**
     * 解析查询SQL
     *
     * @param cubeData
     * @param reportDatasource
     * @param previewDto
     * @param bladeUser
     * @param compType
     * @return
     */
    public R<QuerySqlParam> parseQuerySql(CubeData cubeData, ReportDatasource reportDatasource, PreviewBaseDto previewDto, BladeUser bladeUser, ComponentTypeEnum compType) {
        //构造全局变量
        Map<String, List<Object>> extraParams = buildGlobalParams(previewDto.getExtraParams(), bladeUser);
        DsTypeEnum dsTypeEnum = reportDatasource.getDsTypeSyl();

        String fromSql = cubeData.getQuerySql();
        //2.构造Select模块
        List<ComponentColumn> columnList = JSON.parseArray(previewDto.getColumnList(), ComponentColumn.class);
        //补全日期查询的时间列名
        ComponentColumn filterColum = null;
        //初始化别名
        for (int i = 0; i < columnList.size(); i++) {
            ComponentColumn item = columnList.get(i);
            String columnAliasName = CubeConstants.COLUMN_ALIAS_NAME_PREFIX + i;
            item.setColumnAliasName(dsTypeEnum.isFieldLowerCase() ? columnAliasName.toLowerCase() : columnAliasName);
            if (previewDto.getIsDateComplete() != null && StrUtil.isNotEmpty(previewDto.getFilterColum())) {
                if (previewDto.getIsDateComplete() && item.getColumnName().equals(previewDto.getFilterColum())) {
                    item.setIsDateComplete(true);
                    filterColum = item;
                } else {
                    item.setIsDateComplete(false);
                }
            } else {
                item.setIsDateComplete(false);
            }
        }
        Map<Integer, ComponentColumn> columnMap = columnList.stream().collect(Collectors.toMap(ComponentColumn::getCubeColumnId, t -> t, (v1, v2) -> v1));
        List<String> selectSqlList = new ArrayList<String>();
        List<String> groupBySqlList = new ArrayList<String>();
        //主要用于排序时，不能使用带有%的字符串
        Map<Integer, String> originalNumberColumnMap = new HashMap<Integer, String>();

        for (ComponentColumn column : columnList) {
            String columnName = "";
            //如果清单组件并且包含聚合函数，字段不参与到select中
            if (compType == ComponentTypeEnum.DETAIL && StrUtil.containsAnyIgnoreCase(column.getColumnSql(), CubeConstants.AGGREGATION_FUNCTIONS)) {
                return R.fail("清单组件不能包含聚合函数，请检查数据集字段【" + column.getColumnTitle() + "】");
            }
            String selectColumnName = columnName = CubeUtils.formatVirtualSqlFragment(column.getColumnSql(), extraParams);
            //聚合函数[且计算字段是不包含聚合函数的]
            if (column.getAggregationType() != null && column.getDimensionType() == DimensionTypeEnum.ZHIBIAO
                    && compType != ComponentTypeEnum.DETAIL && Optional.<Integer>ofNullable(column.getVirtualAggregationStatus()).orElse(0) == 0) {
                selectColumnName = columnName = String.format(column.getAggregationType().getMethod(), selectColumnName);
            }
            // 格式化日期
            if (column.getTimeDataFormat() != null
                    && column.getTimeDataFormat() != DateFormatEnum.yyyy_MM_ddhh$mm$ss
                    && column.getViewDataType() == ViewColumnTypeEnum.DATETIME) {
//						selectColumnName = columnName = CubeUtils.formatTimeColumn(column.getTimeDataFormat(), dsTypeEnum, selectColumnName);
                selectColumnName = columnName = dialectContext.getDialect(dsTypeEnum).formatDateTime(column.getTimeDataFormat(), selectColumnName);
            }
            // 格式化数字
            if (column.getNumberDataFormat() != null
                    && column.getViewDataType() == ViewColumnTypeEnum.NUMBER) {
                originalNumberColumnMap.put(column.getCubeColumnId(), selectColumnName);
//						//groupby不需要百分号
                selectColumnName = columnName = String.format(column.getNumberDataFormat().getMethod(), selectColumnName);
//						//select显示拼接百分号
//						selectColumnName = CubeUtils.percentConcat(column.getNumberDataFormat(), dsTypeEnum, selectColumnName);
                if (NumberFormatEnum.isPercentFormat(column.getNumberDataFormat())) {
                    selectColumnName = dialectContext.getDialect(dsTypeEnum).formatMethodFunction(FunctionMethodEnum.NUMBER_JOINT_PERCENT, selectColumnName, column.getNumberDataFormat().getOracleFmFormat());
                }
            }
            selectSqlList.add(selectColumnName + " as \"" + column.getColumnAliasName() + "\"");
            //如果是维度，添加到groupBy
            if (column.getDimensionType() == DimensionTypeEnum.WEIDU && compType != ComponentTypeEnum.DETAIL) {
                //ORACLE的groupBy不能使用别名，clickhouse和mysql可行
                ComponentColumn componentColumn = columnMap.get(column.getCubeColumnId());
                if (componentColumn != null && (dsTypeEnum == DsTypeEnum.clickhouse || dsTypeEnum == DsTypeEnum.mysql)) {
                    groupBySqlList.add(componentColumn.getColumnAliasName());
                } else {
                    groupBySqlList.add(columnName);
                }
            }
        }
        StringBuilder strBuilder = new StringBuilder("select ").append(String.join(",", selectSqlList)).append(" from ").append(fromSql);
        //解析where和having子句
        Tuple whereHavingSql = parseWhereHavingSql(cubeData, dsTypeEnum, previewDto, bladeUser, extraParams);
        String whereSql = whereHavingSql.get(0), havingSql = whereHavingSql.get(1);
        if (StrUtil.isNotBlank(whereSql)) {
            strBuilder.append(" where " + whereSql);
        }
        //清单组件不需要groupBy和having
        if (compType != ComponentTypeEnum.DETAIL) {
            //groupby子句
            if (!groupBySqlList.isEmpty()) {
                strBuilder.append(" group by ").append(String.join(",", groupBySqlList));
            }
            //having子句
            if (StrUtil.isNotBlank(havingSql)) {
                strBuilder.append(" having " + havingSql);
            }
        }
        //构造排序条件
        List<ComponentSort> sortList = Optional.ofNullable(JSON.parseArray(previewDto.getSortList(), ComponentSort.class)).orElse(new ArrayList<>());
        List<String> sortSqlList = new ArrayList<String>();
        for (ComponentSort sort : sortList) {
            ComponentColumn column = sort.getColumn();
            //获取别名
            ComponentColumn componentColumn = columnMap.get(column.getCubeColumnId());
            if (componentColumn != null) { //如果别名不在select中，不能放到orderBy中
                //如果排序字段带有%，使用原始字段
                String orgialColumnName = originalNumberColumnMap.get(column.getCubeColumnId());
                String columnName = StrUtil.isNotBlank(orgialColumnName) ? orgialColumnName : componentColumn.getColumnAliasName();
                sortSqlList.add(columnName + " " + sort.getOrderType().name());
            }
        }
        if (!sortSqlList.isEmpty()) {
            strBuilder.append(" order by ").append(String.join(",", sortSqlList));
        }
        if (CubeUtils.checkSqlInject(strBuilder.toString())) {
            log.info("sqlInject:warning|sql=" + strBuilder.toString());
            return R.fail("有SQL注入风险，请检查");
        }

        String sql = strBuilder.toString();
        Map<String, Object> map = null;
        if (filterColum != null) {
            map = getSql(filterColum, previewDto.getSelectTime(), strBuilder.toString(), columnList, reportDatasource);
            sql = map.get("sql").toString();
        }
        QuerySqlParam querySqlParam = new QuerySqlParam();
        querySqlParam.setQuerySql(sql);
        querySqlParam.setCubeData(cubeData);
        querySqlParam.setReportDatasource(reportDatasource);
        querySqlParam.setSelectColumnList(columnList);
        querySqlParam.setExcludeColumnIdList(whereHavingSql.get(2));
        if (CollectionUtil.isNotEmpty(map)) {
            List<String> selectTime = map.containsKey("dataTime") ? (List<String>) map.get("dataTime") : null;
            List<String> hourTime = map.containsKey("hourTime") ? (List<String>) map.get("hourTime") : null;
            String timeColum = map.containsKey("timeColum") ? map.get("timeColum").toString() : null;
            querySqlParam.setSelectTime(selectTime);
            querySqlParam.setHourTime(hourTime);
            querySqlParam.setTimeColum(timeColum);
        }
        return R.data(querySqlParam);
//		return R.data(new QuerySqlParam(sql, cubeData, reportDatasource, columnList, whereHavingSql.get(2)),null,null,null);
    }

    private Map<String, Object> getSql(ComponentColumn dataColum, List<Object> selectTime, String joinSql, List<ComponentColumn> columnList, ReportDatasource reportDatasource) {

        Map<String, Object> map = new HashMap<>();
        String table = "dim_date";
        String dateTimeColum = "YEAR";
        // 判断sql的时间参数是否为数字时间
        boolean bool = isNumberTime(selectTime);
        String sql = DateCompleteSqlUtils.getSqlByDatasource(reportDatasource);
        // 常规时间类型的处理方式
        if (!bool) {
            if (dataColum.getTimeDataFormat() != null) {
                // 判断使用哪个表
                if (dataColum.getTimeDataFormat().getValue() > 7) {
                    table = "dim_hour";
                    sql = DateCompleteSqlUtils.getHourSqlByDatasource(reportDatasource);
                }
                if (dataColum.getTimeDataFormat().getValue() >= 2 && dataColum.getTimeDataFormat().getValue() <= 4) {
                    dateTimeColum = "YEARMONTHID";
                } else if (dataColum.getTimeDataFormat().getValue() >= 5 && dataColum.getTimeDataFormat().getValue() <= 7) {
                    dateTimeColum = "YYYYMMDD";
                } else if (dataColum.getTimeDataFormat().getValue() >= 8) {
                    dateTimeColum = "YYYYMMDD";
                }
            }
        }

        // 获取字段的查询列表
        StringBuilder cloumStrs = new StringBuilder();
        for (ComponentColumn column : columnList) {
            if (!column.getColumnName().equals(dataColum.getColumnName())) {
                cloumStrs.append("b00.").append(column.getColumnAliasName()).append(",");
            }
        }

        // 获取sql的查询参数(分两种情况，一种情况是过滤器的时间范围，一种是过滤字段的时间范围)
        List<String> selectTimeList = new ArrayList<>();
        List<String> dataTimes = new ArrayList<>();
        if (selectTime.size() > 1) {
            for (Object o : selectTime) {
                // 如果当前的时间包含小时
                if (dataColum.getTimeDataFormat().getValue() >= 8) {
                    // 时间包含年月日小时
                    if (dataColum.getTimeDataFormat().getValue() >= 8 && dataColum.getTimeDataFormat().getValue() <= 10) {
                        if (o.toString().contains(":")) {
                            String[] split = o.toString().split(" ");
                            dataTimes.add(split[0].replace("[", "").replace("]", "")
                                    .replace("-", "").replace("/", ""));
                            String[] strings = split[1].split(":");
                            selectTimeList.add(Integer.valueOf(strings[0]).toString());
                        } else {
                            if (o.toString().length() > 2) {
                                selectTimeList.add(o.toString().trim().replace("[", "").replace("]", ""));
                            } else {
                                selectTimeList.add("0");
                            }
                        }
                    } else {
                        // 时间只包含小时
                        String[] split = o.toString().split(":");
                        selectTimeList.add(split[0]);
                    }
                } else {
                    // 当前时间为年月日
                    selectTimeList.add(o.toString().replace("[", "").replace("]", "")
                            .replace("-", "").replace("/", ""));
                }
            }
        } else {
            List<Object> objs = (List<Object>) selectTime.get(0);
            for (Object str : objs) {
                // 如果当前的时间包含小时
                if (dataColum.getTimeDataFormat().getValue() >= 8) {
                    if (dataColum.getTimeDataFormat().getValue() >= 8 && dataColum.getTimeDataFormat().getValue() <= 10) {
                        if (str.toString().contains(":")) {
                            String[] split = str.toString().split(" ");
                            if (split.length > 2) {
                                dataTimes.add(split[1].replace("[", "").replace("]", "")
                                        .replace("-", "").replace("/", ""));
                                String[] strings = split[2].split(":");
                                selectTimeList.add(Integer.valueOf(strings[0]).toString());
                            } else {
                                dataTimes.add(split[0].replace("[", "").replace("]", "")
                                        .replace("-", "").replace("/", ""));
                                String[] strings = split[1].split(":");
                                selectTimeList.add(Integer.valueOf(strings[0]).toString());
                            }
                        } else {
                            if (str.toString().length() > 2) {
                                selectTimeList.add(str.toString().trim().replace("[", "").replace("]", ""));
                            } else {
                                selectTimeList.add("0");
                            }
                        }
                    } else {
                        String[] split = str.toString().split(":");
                        selectTimeList.add(split[0]);
                    }
                } else {
                    // 当前时间为年月日
                    selectTimeList.add(str.toString().replace("[", "").replace("]", "")
                            .replace("-", "").replace("/", ""));
                }

            }
        }
        String s = selectTimeList.get(0);
        String s1 = selectTimeList.get(1);
        // 判断数字时间格式的查询列表名以及查询sql
        if (bool) {
            if (s.length() == 6 && (!s.contains("-") || !s.contains("/"))) {
                // 202306
                dateTimeColum = "YEARMONTHID";
            } else if (s.length() == 8 && (!s.contains("-") || !s.contains("/"))) {
                // 20230627
                dateTimeColum = "YYYYMMDD";
            } else if ((!s.contains("-") || !s.contains("/"))) {
                // 2023062715
                if (s.length() == 9 || s.length() == 10) {
                    dateTimeColum = "YYYYMMDD";
                    table = "dim_hour";
                    sql = DateCompleteSqlUtils.getNormalSqlByDatasource(reportDatasource);
                }
            }
        }
        String resultSql = "";
        if (bool && s.length() > 8) {
            resultSql = sql.replace("${0}", dataColum.getColumnAliasName())
                    .replace("${1}", cloumStrs.toString().substring(0, cloumStrs.length() - 1))
                    .replace("${2}", table)
                    .replace("${3}", joinSql)
                    .replace("${4}", s)
                    .replace("${5}", s1);
        } else {
            if (table.equals("dim_hour")) {
                resultSql = sql.replace("${0}", dataColum.getColumnAliasName())
                        .replace("${1}", cloumStrs.toString().substring(0, cloumStrs.length() - 1))
                        .replace("${2}", table)
                        .replace("${3}", joinSql);
                if (CollectionUtil.isNotEmpty(dataTimes)) {
                    String selectSql = " and a00.YYYYMMDD BETWEEN " + dataTimes.get(0) + " and " + dataTimes.get(1) + " ORDER BY";
                    resultSql = resultSql.replace("ORDER BY", selectSql);
                    map.put("dataTime", dataTimes);
                }
                map.put("timeColum", dataColum.getColumnAliasName());
                map.put("hourTime", selectTimeList);
            } else {
                resultSql = sql.replace("${0}", dateTimeColum)
                        .replace("${1}", dataColum.getColumnAliasName())
                        .replace("${2}", cloumStrs.toString().substring(0, cloumStrs.length() - 1))
                        .replace("${3}", joinSql)
                        .replace("${4}", s)
                        .replace("${5}", s1)
                        .replace("tableName", table);
            }
        }
        map.put("sql", resultSql);
        return map;
    }

    /**
     * 判断时间参数是否为数字日期
     */
    private boolean isNumberTime(List<Object> selectTime) {
        if (selectTime.size() > 1) {
            for (Object o : selectTime) {
                boolean numeric = isNumeric(o.toString().replace("[", "").replace("]", ""));
                if (!numeric) {
                    return false;
                }
            }
        } else {
            List<Object> objs = (List<Object>) selectTime.get(0);
            for (Object obj : objs) {
                boolean numeric = isNumeric(obj.toString().replace("[", "").replace("]", ""));
                if (!numeric) {
                    return false;
                }
            }
        }

        return true;
    }

    /**
     * 使用正则判断字符串是否为数字
     */
    public boolean isNumeric(String str) {
        Pattern pattern = Pattern.compile("-?\\d+(\\.\\d+)?");
        return pattern.matcher(str).matches();
    }

    public R<QuerySqlParam> parseLogicPageQuerySql(Integer cubeDataId, PreviewBaseDto previewDto, BladeUser bladeUser) {
        CubeData cubeData = cubeDataService.getById(cubeDataId);
        if (cubeData == null) {
            log.error("数据集不存在|id={}", cubeDataId);
            return R.fail("数据集不存在");
        }
        //2.查询数据源
        ReportDatasource reportDatasource = reportDatasourceService.getById(cubeData.getDatasourceId());
        if (reportDatasource == null) {
            log.error("数据源不存在|id={}", cubeData.getDatasourceId());
            return R.fail("数据源不存在");
        }
        //遍历维度
        Map<String, List<Object>> extraParams = buildGlobalParams(previewDto.getExtraParams(), bladeUser);
        DsTypeEnum dsTypeEnum = reportDatasource.getDsTypeSyl();

        //2.构造Select模块
        List<ComponentColumn> columnList = JSON.parseArray(previewDto.getColumnList(), ComponentColumn.class);
        //初始化别名
        for (int i = 0; i < columnList.size(); i++) {
            ComponentColumn item = columnList.get(i);
            item.setColumnAliasName(CubeConstants.COLUMN_ALIAS_NAME_PREFIX + i);
        }
        List<String> weiduColumnList = new ArrayList<>();
        List<String> groupList = new ArrayList<>(), orderList = new ArrayList<>();

        for (ComponentColumn column : columnList) {
            //行维度做展示操作
            if (column.getDimensionType() == DimensionTypeEnum.WEIDU && column.getWeiduType() == WeiduTypeEnum.ROW) {
                if (column.getTimeDataFormat() != null
                        && column.getViewDataType() == ViewColumnTypeEnum.DATETIME) {  //日期需要格式化
                    weiduColumnList.add(dialectContext.getDialect(dsTypeEnum).formatDateTime(column.getTimeDataFormat(), column.getColumnSql()) + " as \"" + column.getColumnAliasName() + "\"");
                } else {
                    weiduColumnList.add(column.getColumnSql() + " as \"" + column.getColumnAliasName() + "\"");
                }
                groupList.add(column.getColumnSql());
                orderList.add(column.getColumnAliasName());
            }
        }

        Tuple whereHavingSql = parseWhereHavingSql(cubeData, dsTypeEnum, previewDto, bladeUser, extraParams);
        String whereSql = whereHavingSql.get(0);

        String querySql = String.format("select %s from %s where 1=1 %s group by %s order by %s",
                String.join(",", weiduColumnList),
                cubeData.getQuerySql(),
                StrUtil.isNotBlank(whereSql) ? (" AND " + whereSql) : "",
                String.join(",", groupList),
                String.join(",", orderList));
        QuerySqlParam querySqlParam = new QuerySqlParam();
        querySqlParam.setQuerySql(querySql);
        querySqlParam.setCubeData(cubeData);
        querySqlParam.setReportDatasource(reportDatasource);
        querySqlParam.setSelectColumnList(columnList);
        querySqlParam.setExcludeColumnIdList(whereHavingSql.get(2));
        return R.data(querySqlParam);
    }

    /**
     * 解析使用到的字段ID列表
     *
     * @param previewDto
     * @return
     */
    public Set<Integer> parseUsedColumnIdList(PreviewBaseDto previewDto) {
        //字段ID列表
        Set<Integer> columnIdList = new HashSet<Integer>();
        //字段
        Optional.ofNullable(JSON.parseArray(previewDto.getColumnList(), ComponentColumn.class)).orElse(new ArrayList<>()).stream().map(ComponentColumn::getCubeColumnId).forEach(columnIdList::add);
        //过滤器字段
        Optional.ofNullable(JSON.parseArray(previewDto.getFilterList(), ComponentFilter.class)).orElse(new ArrayList<>()).stream().map(item -> item.getColumn().getCubeColumnId()).forEach(columnIdList::add);
        //排序字段
        Optional.ofNullable(JSON.parseArray(previewDto.getSortList(), ComponentSort.class)).orElse(new ArrayList<>()).stream().map(item -> item.getColumn().getCubeColumnId()).forEach(columnIdList::add);
        return columnIdList;
    }

    /**
     * 解析过滤器
     *
     * @param filterList
     * @param componentFilterRelation
     * @return
     */
    public Tuple2<String, String> parseFilters(List<ComponentFilter> filterList, ComponentFilterRelation componentFilterRelation, DsTypeEnum dsTypeEnum, Map<String, List<Object>> extraParams) {
        String whereSql = "", havingSql = "";
        if (CollectionUtil.isNotEmpty(filterList)) {
            //当外部调用该解析器时，判断extraParams是否为null
            extraParams = extraParams != null ? extraParams : CubeConstants.mergeSysUserParams(AuthUtil.getUser(), new HashMap<>());
            //过滤关系
            if (componentFilterRelation != null) {
                //filterRelation包含where和having的过滤关系，依次处理
                Tuple2<String, String> whereSqlTuple = _buildFilters(componentFilterRelation.getWhere(), filterList, dsTypeEnum, extraParams);
                if (whereSqlTuple != null) {
                    whereSql = whereSqlTuple.getT1();
                }
                //having处理
                Tuple2<String, String> havingSqlTuple = _buildFilters(componentFilterRelation.getHaving(), filterList, dsTypeEnum, extraParams);
                if (havingSqlTuple != null) {
                    havingSql = havingSqlTuple.getT2();
                }
                //构造非uniqId的条件集合【主要是全局筛选器，因此只需要处理where】
                List<ComponentFilter> dataList = filterList.stream().filter(f -> StrUtil.isEmpty(f.getUniqId())).collect(Collectors.toList());
                if (!dataList.isEmpty()) {
                    Tuple2<String, String> globalWhereSqlTuple = _buildConditions(dataList, FilterConnectTypeEnum.AND, dsTypeEnum, extraParams);
                    if (globalWhereSqlTuple != null && !"".equals(globalWhereSqlTuple.getT1())) {
                        whereSql += (StrUtil.isNotBlank(whereSql) ? " AND " : "") + globalWhereSqlTuple.getT1();
                    }
                }
            } else { //没有过滤关系
                Tuple2<String, String> whereHavingSqlTuple = _buildConditions(filterList, FilterConnectTypeEnum.AND, dsTypeEnum, extraParams);
                if (whereHavingSqlTuple != null) {
                    whereSql = whereHavingSqlTuple.getT1();
                    havingSql = whereHavingSqlTuple.getT2();
                }
            }
        }
        return new Tuple2<String, String>(whereSql, havingSql);
    }

    /**
     * 解析相对时间和标准时间
     *
     * @param filterTimeJson
     * @return
     */
    public List<ComponentFilter> parseFilterTime(String filterTimeJson, DsTypeEnum dsTypeEnum) {
        if (StrUtil.isEmpty(filterTimeJson)) {
            return null;
        }
        WarningFilterTime filterTime = JSON.parseObject(filterTimeJson, WarningFilterTime.class);
        //时间字段和相对时间对象 不能为空
        if (filterTime.getTimeColumn() == null || filterTime.getRelativeTime() == null) {
            return null;
        }
        if (filterTime.getCalcType() == FilterTimeCalcEnum.SPECIAL &&
                filterTime.getTimeColumn().getViewDataType() != ViewColumnTypeEnum.NUMBER) {
            log.warn("timeColumn is not a Number column!!");
            return null;
        }
        if (filterTime.getCalcType() == FilterTimeCalcEnum.NORMAL &&
                filterTime.getTimeColumn().getViewDataType() != ViewColumnTypeEnum.DATETIME) {
            log.warn("timeColumn is not a DateTime column!!");
            return null;
        }
        //计算出2个时间[开始时间和结束时间]
        WarningFilterRelativeTime relativeTime = filterTime.getRelativeTime();
        LocalDateTime startDateTime = relativeTime.getLeftModifier().getTime(relativeTime.getLeftVal(), filterTime.getTimeType());
        LocalDateTime endDateTime = relativeTime.getRightModifier().getTime(relativeTime.getRightVal(), filterTime.getTimeType());
        List<Object> values = new ArrayList<Object>();
        if (filterTime.getCalcType() == FilterTimeCalcEnum.NORMAL) {
            ComponentColumn timeColumn = filterTime.getTimeColumn();
            values.add(DateUtil.format(startDateTime, timeColumn.getTimeDataFormat().getFormat()));
            values.add(DateUtil.format(endDateTime, timeColumn.getTimeDataFormat().getFormat()));
        } else {
            values.add(filterTime.getTimeType().formatInteger(startDateTime));
            values.add(filterTime.getTimeType().formatInteger(endDateTime));
        }
        List<ComponentFilter> filterList = new ArrayList<ComponentFilter>();
        //构造时间字段过滤器
        ComponentFilter timeColumnFilter = new ComponentFilter(filterTime.getTimeColumn());
        timeColumnFilter.setConditions(Arrays.asList(new ComponentFilterCondition(CommonFunctionEnum.between, values)));
        filterList.add(timeColumnFilter);
        //如果时间计算方式为特殊时间，需要添加时间纬度过滤器
        if (filterTime.getCalcType() == FilterTimeCalcEnum.SPECIAL && filterTime.getTimeTypeColumn() != null) {
            ComponentFilter timeTypeColumnFilter = new ComponentFilter(filterTime.getTimeTypeColumn());
            timeTypeColumnFilter.setConditions(ListUtil.toList(new ComponentFilterCondition(CommonFunctionEnum.eq, Arrays.asList(filterTime.getTimeType().name()))));
            filterList.add(timeTypeColumnFilter);
        }
        return filterList;
    }

    public Tuple parseWhereHavingSql(CubeData cubeData, DsTypeEnum dsTypeEnum, PreviewBaseDto previewDto, BladeUser bladeUser, Map<String, List<Object>> extraParams) {
        List<String> whereSqlList = new ArrayList<>();
        //解析数据集全局过滤器
        if (StrUtil.isNotBlank(cubeData.getFilterJson())) {
            PreviewBaseDto cubePreviewDto = JSON.parseObject(cubeData.getFilterJson(), PreviewBaseDto.class);
            //解析SQL
            Tuple2<String, String> whereHavingSqlTuple = parseFilters(JSON.parseArray(cubePreviewDto.getFilterList(), ComponentFilter.class),
                    JSON.parseObject(cubePreviewDto.getFilterRelation(), ComponentFilterRelation.class), dsTypeEnum, extraParams);
            if (StrUtil.isNotBlank(whereHavingSqlTuple.getT1())) {
                whereSqlList.add(whereHavingSqlTuple.getT1());
            }
        }
        //解析当前登录用户的行/列权限
        CubePermParseDto cubePermParseDto = cubeDataPermissionService.parseCubePermission(cubeData.getId(), bladeUser);
        if (StrUtil.isNotBlank(cubePermParseDto.getFilterSql())) {
            whereSqlList.add(cubePermParseDto.getFilterSql());
        }
        //构建过滤条件
        List<ComponentFilter> filterList = Optional.ofNullable(JSON.parseArray(previewDto.getFilterList(), ComponentFilter.class)).orElse(new ArrayList<ComponentFilter>());
        //执行前置操作，是否有时间JSON，进行查询操作
        List<ComponentFilter> filterTimeList = parseFilterTime(previewDto.getFilterTimeJson(), dsTypeEnum);
        if (filterTimeList != null) {
            filterList.addAll(filterTimeList);
        }
        //构造冗余过滤条件
        List<ComponentFilter> extFilterList = Optional.ofNullable(JSON.parseArray(previewDto.getExtFilterJson(), ComponentFilter.class)).orElse(new ArrayList<ComponentFilter>());
        if (CollectionUtil.isNotEmpty(extFilterList)) {
            filterList.addAll(extFilterList);
        }
        //解析过滤器以及过滤器关系
        ComponentFilterRelation componentFilterRelation = JSON.parseObject(previewDto.getFilterRelation(), ComponentFilterRelation.class);
        Tuple2<String, String> whereHavingSqlTuple = parseFilters(filterList, componentFilterRelation, dsTypeEnum, extraParams);
        String whereSql = whereHavingSqlTuple.getT1(), havingSql = whereHavingSqlTuple.getT2();
        //where子句
        if (StrUtil.isNotBlank(whereSql)) {
            whereSqlList.add(whereSql);
        }
        return new Tuple(String.join(" AND ", whereSqlList), havingSql, cubePermParseDto.getExcludeColumnIdList());
    }

    /**
     * 构建Where子句和Having子句
     *
     * @param compFilterDto
     */
    private Tuple2<String, String> _buildFilters(ComponentFilterDto compFilterDto, List<ComponentFilter> filterList, DsTypeEnum dsTypeEnum, Map<String, List<Object>> extraParams) {
//		if((compFilterDto == null
//				|| compFilterDto.getConditions() == null && compFilterDto.getFilters() == null)
//				|| (compFilterDto.getConditions().isEmpty() && compFilterDto.getFilters().isEmpty())
//				|| compFilterDto.getRelation() == null) {
//			return null;
//		}
        if (compFilterDto == null || compFilterDto.getRelation() == null
                || (CollectionUtil.isEmpty(compFilterDto.getConditions()) && CollectionUtil.isEmpty(compFilterDto.getFilters()))) {
            return null;
        }
        List<String> havingSqlList = new ArrayList<String>();
        //1.构造单个条件集合
        List<String> whereSqlList = new ArrayList<String>();
        if (compFilterDto.getConditions() != null && !compFilterDto.getConditions().isEmpty()) {
            List<String> uniqIdList = compFilterDto.getConditions().stream().filter(f -> StrUtil.isNotBlank(f.getUniqId())).map(ComponentFilterItem::getUniqId).collect(Collectors.toList());
            List<ComponentFilter> filterDataList = filterList.stream().filter(f -> uniqIdList.contains(f.getUniqId())).collect(Collectors.toList());
            Tuple2<String, String> tuple = _buildConditions(filterDataList, compFilterDto.getRelation(), dsTypeEnum, extraParams);
            if (StrUtil.isNotBlank(tuple.getT1())) {
                whereSqlList.add(tuple.getT1());
            }
            if (StrUtil.isNotBlank(tuple.getT2())) {
                havingSqlList.add(tuple.getT2());
            }
        }
        //2.构造多个条件集合
        if (CollectionUtil.isNotEmpty(compFilterDto.getFilters())) {
            for (ComponentFilterPlus filterPlus : compFilterDto.getFilters()) {
                List<String> uniqIdList = filterPlus.getConditions().stream().filter(f -> StrUtil.isNotBlank(f.getUniqId())).map(ComponentFilterItem::getUniqId).collect(Collectors.toList());
                List<ComponentFilter> conds = filterList.stream().filter(f -> uniqIdList.contains(f.getUniqId())).collect(Collectors.toList());
                FilterConnectTypeEnum relation = filterPlus.getRelation();
                if (conds != null && relation != null && !conds.isEmpty()) {
                    Tuple2<String, String> tuple = _buildConditions(conds, relation, dsTypeEnum, extraParams);
                    if (StrUtil.isNotBlank(tuple.getT1())) {
                        whereSqlList.add(tuple.getT1());
                    }
                    if (StrUtil.isNotBlank(tuple.getT2())) {
                        havingSqlList.add(tuple.getT2());
                    }
                }
            }
        }
        return new Tuple2<String, String>(_stringListJoin(whereSqlList, compFilterDto.getRelation()), _stringListJoin(havingSqlList, compFilterDto.getRelation()));
    }

    /**
     * 构造查询条件
     *
     * @param compFilterList
     * @param relation
     * @param dsTypeEnum
     * @return
     */
    private Tuple2<String, String> _buildConditions(List<ComponentFilter> compFilterList, FilterConnectTypeEnum relation, DsTypeEnum dsTypeEnum, Map<String, List<Object>> extraParams) {
        List<String> whereSqlList = new ArrayList<String>();
        List<String> havingSqlList = new ArrayList<String>();
        for (ComponentFilter columnFilter : compFilterList) {
            ComponentColumn column = columnFilter.getColumn();
            String columnName = CubeUtils.formatVirtualSqlFragment(column.getColumnSql(), extraParams);
            //计算过滤时机
            FilterConnectTimeEnum connectTime = columnFilter.getConnectTime();
            //HAVING需要把聚合函数参与进来[如果计算字段已存在聚合函数，不再添加]
            if (connectTime == FilterConnectTimeEnum.HAVING && column.getAggregationType() != null
                    && Optional.<Integer>ofNullable(column.getVirtualAggregationStatus()).orElse(0) == 0) {
                columnName = String.format(column.getAggregationType().getMethod(), columnName);
            }
            //格式化日期 【日期类型，返回页面时jackson自动解析为yyyy-MM-dd HH:mm:ss，其他日期类型格式需要转换】
            if (column.getTimeDataFormat() != null
//					&& column.getTimeDataFormat() != DateFormatEnum.yyyy_MM_ddhh$mm$ss
                    && column.getViewDataType() == ViewColumnTypeEnum.DATETIME) {
//				columnName = CubeUtils.formatTimeColumn(column.getTimeDataFormat(), dsTypeEnum, columnName);
                columnName = dialectContext.getDialect(dsTypeEnum).formatDateTime(column.getTimeDataFormat(), columnName);
            }
            //判断columnName是否为空
            if (StrUtil.isEmpty(columnName)) {
                continue;
            }
            //计算过滤条件
            List<ComponentFilterCondition> conditions = columnFilter.getConditions();
            if (conditions != null && !conditions.isEmpty()) {
                //合并日期变量
                CubeConstants.mergeSysCurrentTimeParams(column.getTimeDataFormat(), extraParams);
                List<String> conditionSqlList = new ArrayList<String>();
                //添加过滤条件
                for (ComponentFilterCondition condition : conditions) {
                    //替换变量参数
                    if (StrUtil.isNotBlank(condition.getParamKey())) {
                        condition.setValues(extraParams.get(condition.getParamKey()));
                    }
                    //如果为空，跳过进入下一个条件
                    if (CollectionUtil.isEmpty(condition.getValues())) {
                        continue;
                    }
                    String conditionSql = condition.getFunction().handle(columnName, column.getViewDataType(), condition.getValues(), dsTypeEnum);
                    if (!StrUtil.isEmpty(conditionSql)) {
                        conditionSqlList.add(conditionSql);
                    }
                }
                if (!conditionSqlList.isEmpty()) {
                    //不同的过滤时机添加到对应的集合
                    if (connectTime == FilterConnectTimeEnum.HAVING) {
                        havingSqlList.add(_stringListJoin(conditionSqlList, columnFilter.getConnectType()));
                    } else {
                        whereSqlList.add(_stringListJoin(conditionSqlList, columnFilter.getConnectType()));
                    }
                }
            }
        }
        return new Tuple2<String, String>(_stringListJoin(whereSqlList, relation), _stringListJoin(havingSqlList, relation));
    }

    /**
     * 构造全局变量参数
     * @param extraParams
     * @return
     */
    public Map<String, List<Object>> buildGlobalParams(Map<String, List<Object>> extraParams, BladeUser bladeUser){
    	Map<String, List<Object>> _extraParams = new HashMap<>();
    	if(bladeUser != null) { //兼容处理：智能预警没有bladeUser情况
    		//添加tenantId查询条件，因为服务化接口会调用该方法
        	List<CubeGlobalParam> gloablParamList = cubeGlobalParamService.list(Wrappers.<CubeGlobalParam>lambdaQuery()
        			.eq(CubeGlobalParam::getParamType, ParamTypeEnum.BIZ).eq(CubeGlobalParam::getTenantId, bladeUser.getTenantId()));
    		extraParams = Optional.ofNullable(extraParams).orElse(new HashMap<>());
        	if(CollectionUtil.isNotEmpty(gloablParamList)) {
    			Map<String, String> _mapData = gloablParamList.stream().collect(Collectors.toMap(CubeGlobalParam::getParamKey, t -> Optional.ofNullable(t.getDefaultValue()).orElse(""), (a, b) -> b));
    			for(Map.Entry<String, String> entry : _mapData.entrySet()) {
    				List<Object> paramValue = extraParams.get(entry.getKey());
    				if(StrUtil.isNotBlank(entry.getValue()) || CollectionUtil.isNotEmpty(paramValue)) {
    					//优先使用paramValue的值
    					_extraParams.put(entry.getKey(), CollectionUtil.isNotEmpty(paramValue) ? paramValue : ListUtil.toList(entry.getValue()));
    				}
    			}
    		}
    	}
    	return CubeConstants.mergeAllSysParams(bladeUser, DateFormatEnum.yyyy_MM_ddhh$mm$ss, _extraParams);
    }
    
    /**
     * SQL片段拼接
     *
     * @param dataList
     * @param relation
     * @return
     */
    private String _stringListJoin(List<String> dataList, FilterConnectTypeEnum relation) {
        if (dataList == null || dataList.isEmpty()) {
            return "";
        }
        String startParenthesis = dataList.size() > 1 ? "(" : "";
        String endParenthesis = dataList.size() > 1 ? ")" : "";
        return startParenthesis + String.join(" " + relation.name() + " ", dataList) + endParenthesis;
    }

    /**
     * 拼接桑基图sql
     */
    public String splicingSql(PreviewBaseDto previewDto, BladeUser user,ReportDatasource reportDatasource,Integer cubeDataId) {
        String sql = "select b as COL_ALIAS_0, count(a) as COL_ALIAS_1 from ( $sql ) S group by b order by COL_ALIAS_1 desc";
        DsTypeEnum dsTypeEnum = reportDatasource.getDsTypeSyl();
        // 根据不同的sql拼接不同的韦恩图查询sql
        String strBuilder = getSqlByDsType(dsTypeEnum);
        CubeData cubeData = cubeDataService.getById(cubeDataId);
        //构造全局变量
        Map<String, List<Object>> extraParams = buildGlobalParams(previewDto.getExtraParams(), user);

        //2.构造Select模块
        List<ComponentColumn> columnList = JSON.parseArray(previewDto.getColumnList(), ComponentColumn.class);
        Map<Integer, ComponentColumn> columnMap = columnList.stream().collect(Collectors.toMap(ComponentColumn::getCubeColumnId, t -> t, (v1, v2) -> v1));
        //主要用于排序时，不能使用带有%的字符串
        Map<Integer, String> originalNumberColumnMap = new HashMap<>();
        int num = 0;
        String tableAliasName = "";
        for (ComponentColumn column : columnList) {
            // 替换字段
            strBuilder = strBuilder.replace("$table",column.getTableName()).replace("col_" + num,column.getColumnName());
            tableAliasName = column.getTableAliasName();
            num++;
        }
        //解析where和having子句
        Tuple whereHavingSql = parseWhereHavingSql(cubeData, dsTypeEnum, previewDto, user, extraParams);
        String whereSql = whereHavingSql.get(0);
        if (StrUtil.isNotBlank(whereSql)) {
            if (StrUtil.isNotEmpty(tableAliasName)) {
                strBuilder = strBuilder.replace("$where", "where " + whereSql.replace(tableAliasName+".",""));
            } else {
                strBuilder = strBuilder.replace("$where", "where " + whereSql);
            }

        } else {
            strBuilder = strBuilder.replace("$where", " ");
        }
        //构造排序条件
        List<ComponentSort> sortList = Optional.ofNullable(JSON.parseArray(previewDto.getSortList(), ComponentSort.class)).orElse(new ArrayList<>());
        List<String> sortSqlList = new ArrayList<String>();
        for (ComponentSort sort : sortList) {
            ComponentColumn column = sort.getColumn();
            //获取别名
            ComponentColumn componentColumn = columnMap.get(column.getCubeColumnId());
            if (componentColumn != null) { //如果别名不在select中，不能放到orderBy中
                //如果排序字段带有%，使用原始字段
                String orgialColumnName = originalNumberColumnMap.get(column.getCubeColumnId());
                String columnName = StrUtil.isNotBlank(orgialColumnName) ? orgialColumnName : componentColumn.getColumnAliasName();
                sortSqlList.add(columnName + " " + sort.getOrderType().name());
            }
        }
        if (!sortSqlList.isEmpty()) {
            strBuilder = strBuilder + " order by " + (String.join(",", sortSqlList));
        }
        if (CubeUtils.checkSqlInject(strBuilder.toString())) {
            log.info("sqlInject:warning|sql=" + strBuilder);
            return null;
        }
        if (DsTypeEnum.oracle.getValue().equals(dsTypeEnum.getValue())) {
            sql = "select dbms_lob.substr(b,2000) as COL_ALIAS_0, count(a) as COL_ALIAS_1 from ( $sql ) S group by dbms_lob.substr(b,2000) order by COL_ALIAS_1 desc";
        }
        return sql.replace("$sql",strBuilder);
    }

    private String getSqlByDsType(DsTypeEnum dsTypeEnum) {
        // 默认为mysql
        String strBuilder = "select col_1 as a, group_concat(distinct col_0)  as b from $table $where group by col_1";
        if (DsTypeEnum.oracle.getValue().equals(dsTypeEnum.getValue())) { // oracle
//            strBuilder = "SELECT col_1 as a, LISTAGG(col_0, ',') WITHIN GROUP (ORDER BY col_0) AS b FROM $table $where  GROUP BY col_1";
            strBuilder = "SELECT col_1 as a, xmlagg(xmlparse(content col_0 || ',' wellformed) order by col_0).getclobval() AS b " +
                    "FROM $table $where  GROUP BY col_1";
        } else if (DsTypeEnum.sqlserver.getValue().equals(dsTypeEnum.getValue())) { // sqlserver
            strBuilder = "SELECT T.COL_0 as a,\n" +
                    "       STUFF((SELECT ',' + A.col_1\n" +
                    "              FROM $table A\n" +
                    "              WHERE A.col_0 = T.COL_0\n" +
                    "              group by A.COL_1\n" +
                    "              FOR XML PATH ( '' )), 1, 1, '') AS b\n" +
                    "FROM $table T\n" +
                    "$where\n" +
                    "GROUP BY T.COL_0";
        } else if (DsTypeEnum.postgresql.getValue().equals(dsTypeEnum.getValue())) { // pg
            strBuilder = "SELECT COL_0 as a, string_agg(distinct COL_1,',') AS b FROM $table $where GROUP BY COL_0";
        } else if (DsTypeEnum.clickhouse.getValue().equals(dsTypeEnum.getValue())) { // clickhouse
            strBuilder = "SELECT COL_0 as a, arrayStringConcat(groupArray(COL_1), ',') AS b FROM $table $where GROUP BY COL_0";
        } else if (DsTypeEnum.kingbase.getValue().equals(dsTypeEnum.getValue())) { // 人大金仓
            strBuilder = "SELECT col_0 as a, GROUP_CONCAT(col_1) as b FROM $table $where GROUP BY col_1";
        }

        return strBuilder;
    }
}
