package org.springblade.visual.service;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springblade.core.secure.BladeUser;
import org.springblade.visual.dto.VisualComponentHistoryRecordDto;
import org.springblade.visual.entity.VisualComponentHistoryRecord;
import org.springblade.visual.mapper.VisualComponentHistoryRecordMapper;
import org.springblade.visual.utils.MD5Util;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;


@Service
public class VisualComponentHistoryRecordService extends ServiceImpl<VisualComponentHistoryRecordMapper, VisualComponentHistoryRecord> {

    /**
     * 保存
     * @param historyRecord
     * @return
     */
    public Boolean saveComponentHistoryRecord(VisualComponentHistoryRecord historyRecord, BladeUser user) {
        boolean result;
        //首先对查询条件加组件key进行MD5加密
        String settingInfo = historyRecord.getSettingInfo();
        String md5 = MD5Util.md5(settingInfo);
        historyRecord.setSettingInfoMd5(md5);
        historyRecord.setCreateTime(LocalDateTime.now());
        // 如果存在id则进行更新操作
        if (historyRecord.getId() != null) {
            result = this.updateById(historyRecord);
        } else {
            // 首先检验MD5加密是否一致 是则更新否则新加
            VisualComponentHistoryRecord record = this.getOne(Wrappers.<VisualComponentHistoryRecord>lambdaQuery()
                    .eq(VisualComponentHistoryRecord::getSettingInfoMd5, md5)
                    .eq(VisualComponentHistoryRecord::getOperationUserId, user.getUserId()));
            if (record != null && record.getId() > 0){
                // 如果存在相同的记录则进行更新
                historyRecord.setId(record.getId());
                result = this.updateById(historyRecord);
            } else {
                historyRecord.setOperationUserId(user.getUserId());
                historyRecord.setOperationUserName(user.getNickName());
                // 添加
                result = this.save(historyRecord);
            }
        }
        return result;
    }

    /**
     * 分页查询组件历史记录
     * @param record
     * @param limit
     * @return
     */
    public List<VisualComponentHistoryRecord> getComponentHistoryRecordList(VisualComponentHistoryRecordDto record, Integer limit) {

        return baseMapper.getComponentHistoryRecordList(record,limit);
    }
}
