/**
 * 
 */
package org.springblade.visual.service;

import java.util.Date;
import java.util.List;
import java.util.Set;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.service.CubeDataService;
import org.springblade.visual.dto.ComponentColumn;
import org.springblade.visual.dto.ComponentDto;
import org.springblade.visual.entity.VisualComponent;
import org.springblade.visual.enums.ComponentUseTypeEnum;
import org.springblade.visual.mapper.VisualComponentMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Slf4j
@Service
public class VisualComponentService extends ServiceImpl<VisualComponentMapper, VisualComponent> {
	
	@Autowired
	private QuerySqlService querySqlService;
	
	@Autowired
	private CubeDataService cubeDataService;
	
	/**
	 * @param visualId
	 * @param componentDtoList
	 */
	@Transactional
	public void batchSaveComponent(Integer visualId, List<ComponentDto> componentDtoList, BladeUser bladeUser) {
		for(ComponentDto componentDto : componentDtoList) {
			VisualComponent component = componentDto.getComponent();
			if(component.getId() == null) { //新增
				component.setCreateTime(new Date());
				component.setCreateBy(SecureUtil.getNickName());
				component.setCreateUserId(SecureUtil.getUserId());
			}
			component.setVisualId(visualId);
			component.setColumnJson(componentDto.getColumnList());
			component.setFilterJson(componentDto.getFilterList());
			component.setFilterRelationJson(componentDto.getFilterRelation());
			component.setSortJson(componentDto.getSortList());
			 //当且仅当组件类型是Cube并且有字段信息解析SQL（有可能是个纯前端的组件或者是API类型组件）
			if(!StrUtil.isEmpty(componentDto.getColumnList())
					&& component.getUseType() == ComponentUseTypeEnum.CUBE) {
//				//解析SQL存储【注意：组件存储的SQL暂时没有用；使用时调用preview接口，当时拼接】
//				R<QuerySqlParam> r = querySqlService.parseQuerySql(component.getCubeDataId(), componentDto, bladeUser);
//				if(!r.isSuccess()) {
//					log.warn("获取QuerySQL失败|message=" + r.getMsg());
//					continue;
//				}
//				QuerySqlParam querySqlParam = r.getData();
//				component.setDatasourceId(querySqlParam.getReportDatasource().getId());
//				component.setQuerySql(querySqlParam.getQuerySql());
//				//重设columnJson，主要为了设置字段别名
//				component.setColumnJson(JSON.toJSONString(querySqlParam.getSelectColumnList()));
				LambdaQueryWrapper<CubeData> queryWrapper = Wrappers.<CubeData>lambdaQuery().select(CubeData::getDatasourceId).eq(CubeData::getId, component.getCubeDataId());
				Integer datasourceId = cubeDataService.getObj(queryWrapper, t -> Integer.valueOf(t.toString()));
				//设置数据源ID
				component.setDatasourceId(datasourceId);
				
				//解析使用到的字段ID列表
				Set<Integer> usedColumnIdList = querySqlService.parseUsedColumnIdList(componentDto);
				//如果extJson中存在zuanList字段，也需要放到columnId列表中
				if(StrUtil.isNotBlank(component.getExtJson())) {
					try {
						JSONObject tempObject = JSON.parseObject(component.getExtJson());
						if(tempObject.containsKey("zuanList")) {
							JSON.parseArray(tempObject.getString("zuanList"), ComponentColumn.class).stream().map(ComponentColumn::getCubeColumnId).forEach(usedColumnIdList::add);;
						}
					} catch (Exception e) {
						log.error(e.getMessage(), e);
					}
				}
				//设置使用到的字段ID列表JSON
				component.setColumnIdList(JSON.toJSONString(usedColumnIdList));
			}
			this.saveOrUpdate(component);
		}
	}
}
