package org.springblade.visual.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springblade.core.secure.BladeUser;
import org.springblade.report.util.EncryptionUtil;
import org.springblade.visual.dto.VisualLoginFreeLinkPermissionDto;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualLoginFreeLink;
import org.springblade.visual.entity.VisualLoginFreeLinkPermission;
import org.springblade.visual.entity.VisualPowerPointPermission;
import org.springblade.visual.mapper.VisualLoginFreeLinkMapper;
import org.springblade.visual.utils.Number62;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.support.SimpleTriggerContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
public class VisualLoginFreeLinkService extends ServiceImpl<VisualLoginFreeLinkMapper, VisualLoginFreeLink> {


    @Autowired
    private VisualShortCodeGeneratorService codeGeneratorService;

    @Autowired
    private VisualLoginFreeLinkPermissionService visualLoginFreeLinkPermissionService;

//    @Value("${blade.loginFreeDomain}")
//    private String domain;

    @Transactional(rollbackFor = Exception.class)
    public Integer saveOrUpdateLoginFreeLink(VisualLoginFreeLinkPermissionDto linkPermissionDto, VisualData visualData,
                                             BladeUser user) {

        VisualLoginFreeLink visualLoginFreeLink = BeanUtil.copyProperties(linkPermissionDto, VisualLoginFreeLink.class);
        if (visualLoginFreeLink.getId() == null) {
            visualLoginFreeLink.setCreateBy(user.getNickName());
            visualLoginFreeLink.setCreateTime(new Date());
            visualLoginFreeLink.setCreateUserId(user.getUserId());
            visualLoginFreeLink.setStatus(1);
            visualLoginFreeLink.setVisualDataTitle(visualData.getTitle());
            if (visualLoginFreeLink.getLinkType() == 0) {//免登录
                visualLoginFreeLink.setShortLink("/third/" + getCode());
            } else {//账号匹配
                visualLoginFreeLink.setShortLink("/open/" + getCode());
            }
            visualLoginFreeLink.setRealUrl(String.format("%s/%s", "/newbi/fullscreen", visualData.getId()));
            this.save(visualLoginFreeLink);

            bindPermGroup(linkPermissionDto, user, visualLoginFreeLink);
        } else {
            VisualLoginFreeLink loginFreeLink = this.getById(visualLoginFreeLink.getId());
            if (visualLoginFreeLink.getLinkType() == 0 && (visualLoginFreeLink.getLinkType() != loginFreeLink.getLinkType())) {
                //账号匹配改为免登录
                loginFreeLink.setShortLink(loginFreeLink.getShortLink().replace("open", "third"));
            }
            if (visualLoginFreeLink.getLinkType() == 1 && (visualLoginFreeLink.getLinkType() != loginFreeLink.getLinkType())) {
                //免登录改为账号匹配
                loginFreeLink.setShortLink(loginFreeLink.getShortLink().replace("third", "open"));
            }
            loginFreeLink.setValidDate(visualLoginFreeLink.getValidDate());
            loginFreeLink.setUserName(visualLoginFreeLink.getUserName());
            loginFreeLink.setVisualDataId(visualLoginFreeLink.getVisualDataId());
            loginFreeLink.setVisualDataTitle(visualLoginFreeLink.getVisualDataTitle());
            loginFreeLink.setStatus(visualLoginFreeLink.getStatus());
            loginFreeLink.setLinkType(visualLoginFreeLink.getLinkType());
            loginFreeLink.setPurpose(visualLoginFreeLink.getPurpose());
            loginFreeLink.setPublicStatus(visualLoginFreeLink.getPublicStatus());
            loginFreeLink.setRealUrl(String.format("%s/%s", "/newbi/fullscreen", visualData.getId()));
            loginFreeLink.setUpdateBy(user.getNickName());
            loginFreeLink.setUpdateTime(new Date());
            loginFreeLink.setUpdateUserId(user.getUserId());
            loginFreeLink.setPass(visualLoginFreeLink.getPass());
            this.updateById(loginFreeLink);
        }

        return visualLoginFreeLink.getId();
    }

    /**
     * 绑定权限组
     *
     * @param linkPermissionDto
     * @param user
     * @param visualLoginFreeLink
     */
    private void bindPermGroup(VisualLoginFreeLinkPermissionDto linkPermissionDto, BladeUser user, VisualLoginFreeLink visualLoginFreeLink) {
        // 同步保存权限组关联信息
        if (visualLoginFreeLink.getId() != null && CollectionUtil.isNotEmpty(linkPermissionDto.getDataPermIdList())) {
            List<VisualLoginFreeLinkPermission> linkPermissions = new ArrayList<>();
            linkPermissionDto.getDataPermIdList().forEach(t -> {
                VisualLoginFreeLinkPermission linkPermission = VisualLoginFreeLinkPermission.builder()
                        .linkId(visualLoginFreeLink.getId())
                        .dataPermId(t)
                        .createBy(user.getNickName())
                        .createUserId(user.getUserId())
                        .createTime(LocalDateTime.now())
                        .build();
                linkPermissions.add(linkPermission);
            });

            visualLoginFreeLinkPermissionService.saveBatch(linkPermissions);
        }
    }

    /**
     * 生成短链接编码
     */
    private String getCode() {
        Long codeNum = codeGeneratorService.getCodeNum();
        String shortCode = Number62.encode(codeNum.longValue());
        return shortCode;
    }

    /**
     * 根据ID删除免登陆链接
     *
     * @param ids
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean removeByIdList(List<Integer> ids) {
        //删除免登录链接和权限组的关系
        visualLoginFreeLinkPermissionService.remove(Wrappers.<VisualLoginFreeLinkPermission>lambdaQuery()
                .in(VisualLoginFreeLinkPermission::getLinkId, ids));
        return this.removeByIds(ids);
    }
}
