package org.springblade.visual.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.thread.ExecutorBuilder;
import cn.hutool.core.thread.ThreadFactoryBuilder;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import org.springblade.core.tool.api.R;
import org.springblade.cube.model.*;
import org.springblade.cube.service.*;
import org.springblade.cube.utils.CubeConstants;
import org.springblade.datasource.context.DialectContext;
import org.springblade.datasource.enums.DsTypeEnum;
import org.springblade.datasource.model.TableColumnDto;
import org.springblade.datasource.support.DynamicDataSourceSupport;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.service.IReportDatasourceService;
import org.springblade.system.client.MenuClient;
import org.springblade.system.client.model.BladeMenuModel;
import org.springblade.visual.dto.ComponentFilter;
import org.springblade.visual.dto.ComponentFilterRelation;
import org.springblade.visual.dto.PreviewBaseDto;
import org.springblade.visual.entity.*;
import org.springblade.visual.utils.Tuple2;
import org.springblade.visual.vo.CopyCubeDataVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

@Service
@Slf4j
public class VisualPublicSourceInitService {

    @Autowired
    private VisualDataService visualDataService;

    @Autowired
    private IReportDatasourceService reportDatasourceService;

    @Autowired
    private CubeDataService cubeDataService;

    @Autowired
    private CubeTableColumnService cubeTableColumnService;

    @Autowired
    private CubeDataPermissionService cubeDataPermissionService;

    @Autowired
    private CubeSqlViewService cubeSqlViewService;

    @Autowired
    private VisualComponentService visualComponentService;

    @Autowired
    private VisualClassifyService visualClassifyService;

    @Autowired
    private CubeWorkspaceService cubeWorkspaceService;

    @Autowired
    private DynamicDataSourceSupport dynamicDataSourceSupport;

    @Autowired
    private VisualPowerPointCategoryService visualPowerPointCategoryService;

    @Autowired
    private VisualPowerPointInfoService visualPowerPointInfoService;

    @Autowired
    private MenuClient menuClient;


    @Value("${init.edit}")
    private int isEdit;

    @Transactional(rollbackFor = Exception.class)
    public Boolean initSource(String tenantId) {
        try {
            // 查询数据集
            List<CubeData> cubeList = cubeDataService.list(Wrappers.<CubeData>lambdaQuery()
                    .eq(CubeData::getTenantId, "111111"));
            if (CollectionUtil.isEmpty(cubeList)) {
                return false;
            }
            List<Integer> cubeIds = cubeList.stream().map(CubeData::getId).collect(Collectors.toList());

            List<CubeData> cubeDataList = cubeDataService.list(Wrappers.<CubeData>lambdaQuery()
                    .eq(CubeData::getTenantId, tenantId)
                    .in(CubeData::getOldCubeDataId, cubeIds));
            List<CubeData> copyCubeList = new LinkedList<>();
            if (CollectionUtil.isNotEmpty(cubeDataList)) {
                Map<Integer, List<CubeData>> cubeMap = cubeDataList.stream().collect(Collectors.groupingBy(CubeData::getOldCubeDataId));
                for (CubeData data : cubeList) {
                    if (CollectionUtil.isEmpty(cubeMap.get(data.getId()))) {
                        copyCubeList.add(data);
                    }
                }
            } else {
                copyCubeList.addAll(cubeList);
            }

            // 查询大屏
            List<VisualData> dataList = visualDataService.list(Wrappers.<VisualData>lambdaQuery()
                    .eq(VisualData::getTenantId, "111111"));
            List<Integer> visualIds = dataList.stream().map(VisualData::getId).collect(Collectors.toList());
            List<VisualData> visualDatas = visualDataService.list(Wrappers.<VisualData>lambdaQuery()
                    .eq(VisualData::getTenantId, tenantId)
                    .in(VisualData::getRefVisualId, visualIds));
            List<VisualData> copyVisualList = new LinkedList<>();
            if (CollectionUtil.isNotEmpty(visualDatas)) {
                Map<Integer, List<VisualData>> visualMap = visualDatas.stream().collect(Collectors.groupingBy(VisualData::getRefVisualId));
                for (VisualData visualData : dataList) {
                    if (CollectionUtil.isEmpty(visualMap.get(visualData.getId()))) {
                        copyVisualList.add(visualData);
                    }
                }
            } else {
                copyVisualList.addAll(dataList);
            }

            // 查询幻灯片
            List<VisualPowerPointInfo> powerList = visualPowerPointInfoService.list(Wrappers.<VisualPowerPointInfo>lambdaQuery()
                    .eq(VisualPowerPointInfo::getTenantId, "111111"));
            List<VisualPowerPointInfo> copyPowerList = new LinkedList<>();
            if (CollectionUtil.isNotEmpty(powerList)) {
                List<Integer> powerIds = powerList.stream().map(VisualPowerPointInfo::getId).collect(Collectors.toList());
                List<VisualPowerPointInfo> list = visualPowerPointInfoService.list(Wrappers.<VisualPowerPointInfo>lambdaQuery()
                        .in(VisualPowerPointInfo::getId, powerIds)
                        .eq(VisualPowerPointInfo::getTenantId, tenantId));
                if (CollectionUtil.isNotEmpty(list)) {
                    Map<Integer, List<VisualPowerPointInfo>> listMap = list.stream().collect(Collectors.groupingBy(VisualPowerPointInfo::getRefPowerId));
                    for (VisualPowerPointInfo pointInfo : powerList) {
                        if (CollectionUtil.isEmpty(listMap.get(pointInfo.getId()))) {
                            copyPowerList.add(pointInfo);
                        }
                    }
                } else {
                    copyPowerList.addAll(powerList);
                }
            }

            // 查询数据源
            ReportDatasource reportDatasource = reportDatasourceService.getOne(Wrappers.<ReportDatasource>lambdaQuery()
                    .eq(ReportDatasource::getTenantId, "111111"));
            int datasourceId = reportDatasource.getId();
            ReportDatasource oldDatasource = BeanUtil.toBean(reportDatasource, ReportDatasource.class);
            ReportDatasource newReportDatasource = null;
            if (CollectionUtil.isNotEmpty(cubeDataList)) {
                newReportDatasource = reportDatasourceService.getOne(Wrappers.<ReportDatasource>lambdaQuery()
                        .eq(ReportDatasource::getTenantId, tenantId)
                        .eq(ReportDatasource::getId, cubeDataList.get(0).getDatasourceId()));
            }
            if (newReportDatasource == null || newReportDatasource.getId() < 0) {
                reportDatasource.setId(null);
                reportDatasource.setTenantId(tenantId);
                reportDatasource.setIsEdit(isEdit);
                reportDatasource.setPoolName(UUID.randomUUID().toString().replaceAll("-", "").substring(0, 10));
                dynamicDataSourceSupport.addDs(reportDatasource);
                reportDatasourceService.save(reportDatasource);
            } else {
                reportDatasource.setId(newReportDatasource.getId());
                reportDatasource.setTenantId(newReportDatasource.getTenantId());
            }

            // 根据数据源 id 查询视图
            List<CubeSqlView> copySqlViews = new LinkedList<>();
            List<CubeSqlView> cubeSqlViews = cubeSqlViewService.getByDatasourceId(datasourceId);
            if (CollectionUtil.isNotEmpty(cubeSqlViews)) {
                List<CubeSqlView> sqlViews = cubeSqlViewService.getByDatasourceId(reportDatasource.getId());
                if (CollectionUtil.isNotEmpty(sqlViews)) {
                    for (CubeSqlView sqlView : cubeSqlViews) {
                        boolean bool = true;
                        for (CubeSqlView cubeSqlView : sqlViews) {
                            if (sqlView.getViewName().equals(cubeSqlView.getViewName())
                                    && sqlView.getSqlName().equals(cubeSqlView.getSqlName())) {
                                bool = false;
                                break;
                            }
                        }
                        if (bool) {
                            copySqlViews.add(sqlView);
                        }
                    }
                } else {
                    copySqlViews.addAll(cubeSqlViews);
                }
                if (CollectionUtil.isNotEmpty(copySqlViews)) {
                    copySqlViews.forEach(view -> {
                        view.setId(null);
                        view.setDatasourceId(reportDatasource.getId());
                        view.setTenantId(tenantId);
                    });
                }
            }
            cubeSqlViewService.saveBatch(copySqlViews);

            // 如果大屏和数据集待拷贝的集合为空则说明没有需要初始化的数据直接返回即可
            if (CollectionUtil.isEmpty(copyVisualList) && CollectionUtil.isEmpty(copyCubeList)) {
                return true;
            }

            List<CubeTableColumn> cubeTableColumns = new LinkedList<>();
            if (CollectionUtil.isNotEmpty(copyCubeList)) {
                // 查询数据集表
                List<Integer> copyCubeIds = copyCubeList.stream().map(CubeData::getId).collect(Collectors.toList());
                cubeTableColumns = cubeTableColumnService.list(Wrappers.<CubeTableColumn>lambdaQuery()
                        .eq(CubeTableColumn::getTenantId, "111111")
                        .in(CubeTableColumn::getCubeDataId, copyCubeIds));
            }

            // 查询大屏组件
            List<Integer> copyVisualIds = copyVisualList.stream().map(VisualData::getId).collect(Collectors.toList());
            List<VisualComponent> visualComponents = visualComponentService.list(Wrappers.<VisualComponent>lambdaQuery()
                    .in(VisualComponent::getVisualId, copyVisualIds));

            // 查询大屏分类
            VisualClassify classify = visualClassifyService.getOne(Wrappers.<VisualClassify>lambdaQuery()
                    .eq(VisualClassify::getTenantId, tenantId)
                    .eq(VisualClassify::getCategoryValue, "初始化"));
            if (classify == null || classify.getId() < 0) {
                classify = new VisualClassify();
                classify.setParentId(0);
                classify.setCategoryValue("初始化");
                classify.setCreateTime(new Date());
                classify.setTenantId(tenantId);
                classify.setCreateUserId(0L);
                classify.setSortNum(1);
                visualClassifyService.save(classify);
            }
            // 查询数据集分类
            CubeWorkspace workspace = cubeWorkspaceService.getOne(Wrappers.<CubeWorkspace>lambdaQuery()
                    .eq(CubeWorkspace::getTenantId, tenantId)
                    .eq(CubeWorkspace::getWsName, "初始化"));
            if (workspace == null || workspace.getId() < 0) {
                workspace = new CubeWorkspace();
                workspace.setParentId(0);
                workspace.setWsName("初始化");
                workspace.setCreateBy("admin");
                workspace.setSortNum(1);
                workspace.setCreateTime(new Date());
                workspace.setTenantId(tenantId);
                workspace.setCreateUserId(0L);
                cubeWorkspaceService.save(workspace);
            }

            List<CubeTableColumn> tableColumns = new ArrayList<>();
            List<VisualComponent> components = new ArrayList<>();
            List<CubeDataPermission> entityList = new ArrayList<>();
            List<CubeData> cubes = new LinkedList<>();
            List<VisualData> visualDataList = new LinkedList<>();
            List<CubeData> cubeDatas = new ArrayList<>();
            // 拷贝数据集
            AtomicInteger i = new AtomicInteger(1);
            if (CollectionUtil.isNotEmpty(copyCubeList)) {
                for (CubeData data : copyCubeList) {
                    CubeData bean = BeanUtil.toBean(data, CubeData.class);
                    copyCube(tenantId, reportDatasource, bean, tableColumns, entityList, cubeTableColumns, cubes, i, workspace);
                    cubeDatas.add(bean);
                }
            }
            cubeDataService.saveBatch(cubes);
            for (int i1 = 0; i1 < cubes.size(); i1++) {
                int finalI = i1 + 1;
                tableColumns.forEach(colum -> {
                    if (colum.getCubeDataId() == finalI) {
                        colum.setCubeDataId(cubes.get(finalI - 1).getId());
                    }
                });
                if (CollectionUtil.isNotEmpty(entityList)) {
                    entityList.forEach(per -> {
                        if (per.getCubeDataId() == finalI) {
                            per.setCubeDataId(cubes.get(finalI - 1).getId());
                        }
                    });
                }
            }
            cubeTableColumnService.saveBatch(tableColumns);
            AtomicInteger j = new AtomicInteger(1);
            if (CollectionUtil.isNotEmpty(copyVisualList)) {
                for (VisualData visualData : copyVisualList) {
                    copyVisualData(tenantId, visualData, copyCubeList, cubeDatas, components, visualComponents, tableColumns, visualDataList, j,
                            classify, oldDatasource, reportDatasource);
                }
            }
            cubeDataPermissionService.saveBatch(entityList);
            visualDataService.saveBatch(visualDataList);
            for (int i1 = 0; i1 < visualDataList.size(); i1++) {
                int finalI = i1 + 1;
                components.forEach(component -> {
                    if (component.getVisualId() == finalI) {
                        component.setVisualId(visualDataList.get(finalI - 1).getId());
                    }
                });
            }
            visualComponentService.saveBatch(components);

            List<VisualPowerPointInfo> powerPointInfos = syncPowerPoint(copyPowerList, tenantId, visualDataList);

            syncMenu(visualDataList, powerPointInfos, tenantId);
        } catch (Exception e) {
            throw new RuntimeException("初始化报错了",e);
        }
        return false;
    }

    /**
     * 初始化幻灯片
     */
    private List<VisualPowerPointInfo> syncPowerPoint(List<VisualPowerPointInfo> copyPowerList, String tenantId, List<VisualData> visualDataList) {
        // 先查看幻灯片初始化分类是否有被创建
        VisualPowerPointCategory category = visualPowerPointCategoryService.getOne(Wrappers.<VisualPowerPointCategory>lambdaQuery()
                .eq(VisualPowerPointCategory::getCateName, "初始化")
                .eq(VisualPowerPointCategory::getTenantId, tenantId));
        if (category == null || category.getId() < 0) {
            category = new VisualPowerPointCategory();
            category.setCateName("初始化");
            category.setTenantId(tenantId);
            category.setSortNum(0);
            category.setParentId(0);
            category.setCreateTime(LocalDateTime.now());
            category.setCreateUserId(0L);
            visualPowerPointCategoryService.save(category);
        }
        //
        // 设置幻灯片
        List<VisualPowerPointInfo> powerPointInfos = new LinkedList<>();
        for (VisualPowerPointInfo pointInfo : copyPowerList) {
            Integer id = pointInfo.getId();
            pointInfo.setCateId(category.getId());
            pointInfo.setTenantId(tenantId);
            pointInfo.setRefPowerId(id);
            pointInfo.setId(null);
            pointInfo.setCreateTime(LocalDateTime.now());
            pointInfo.setCreateUserId(0L);
            pointInfo.setUpdateBy("admin");
            pointInfo.setUpdateTime(LocalDateTime.now());
            List<String> visuslIdList = new LinkedList<>();
            if (CollectionUtil.isNotEmpty(pointInfo.getVisualIds())) {
                List<String> visualIds = pointInfo.getVisualIds();
                Map<Integer, List<VisualData>> map = visualDataList.stream().collect(Collectors.groupingBy(VisualData::getRefVisualId));
                for (String visualId : visualIds) {
                    if (CollectionUtil.isNotEmpty(map.get(Integer.parseInt(visualId)))) {
                        visuslIdList.add(map.get(Integer.parseInt(visualId)).get(0).getId().toString());
                    }
                }
            }
            pointInfo.setVisualIds(visuslIdList);
            powerPointInfos.add(pointInfo);
        }
        visualPowerPointInfoService.saveBatch(powerPointInfos);
        // 更新幻灯片大屏的幻灯片id
        Map<Integer, List<VisualData>> collect = visualDataList.stream()
                .filter(visualData -> visualData.getPowerPointId() != null)
                .collect(Collectors.groupingBy(VisualData::getPowerPointId));
        List<VisualData> visualDatas = new ArrayList<>();
        for (VisualPowerPointInfo powerPointInfo : powerPointInfos) {
            if (CollectionUtil.isNotEmpty(collect.get(powerPointInfo.getRefPowerId()))) {
                List<VisualData> visualData = collect.get(powerPointInfo.getRefPowerId());
                visualData.forEach(visual -> {
                    visual.setPowerPointId(powerPointInfo.getId());
                    visualDatas.add(visual);
                });
            }
        }
        if (CollectionUtil.isNotEmpty(visualDatas)) {
            visualDataService.updateBatchById(visualDatas);
        }

        return powerPointInfos;
    }

    /**
     * 初始化菜单
     */
    private void syncMenu(List<VisualData> visualDataList, List<VisualPowerPointInfo> powerPointInfos, String tenantId) {
        Map<Integer, List<VisualData>> map;
        Map<Integer, List<VisualPowerPointInfo>> powerMap;
        if (CollectionUtil.isNotEmpty(visualDataList) || CollectionUtil.isNotEmpty(powerPointInfos)) {
            map = visualDataList.stream().collect(Collectors.groupingBy(VisualData::getRefVisualId));
            powerMap = powerPointInfos.stream().collect(Collectors.groupingBy(VisualPowerPointInfo::getRefPowerId));
        } else {
            return;
        }
        List<BladeMenuModel> initMenus = menuClient.getByTenantId("111111");
        if (CollectionUtil.isNotEmpty(initMenus)) {
            List<BladeMenuModel> menus = menuClient.getByTenantId(tenantId);
            List<BladeMenuModel> menuModels = new LinkedList<>();
            if (CollectionUtil.isNotEmpty(menus)) {
                for (BladeMenuModel initMenu : initMenus) {
                    boolean bool = true;
                    for (BladeMenuModel menu : menus) {
                        if (initMenu.getCode().equals(menu.getCode()) &&
                                initMenu.getName().equals(menu.getName()) &&
                                initMenu.getAlias().equals(menu.getAlias())) {
                            bool = false;
                            break;
                        }
                    }
                    if (bool) {
                        String path = initMenu.getPath();
                        String newPath = exchangePath(path, map, powerMap);
                        initMenu.setPath(newPath);
                        initMenu.setTenantId(tenantId);
                        menuModels.add(initMenu);
                    }
                }
            } else {
                for (BladeMenuModel initMenu : initMenus) {
                    String path = initMenu.getPath();
                    String newPath = exchangePath(path, map, powerMap);
                    initMenu.setPath(newPath);
                    initMenu.setTenantId(tenantId);
                    menuModels.add(initMenu);
                }
            }
            menuModels.forEach(m -> {
                Long menuId = m.getId();
                m.setId(null);
                Long parentId = menuClient.save(m);
                menuModels.forEach(menu -> {
                    if (menu.getParentId().equals(menuId)) {
                        menu.setParentId(parentId);
                    }
                });
            });
        }
    }

    private String exchangePath(String path, Map<Integer, List<VisualData>> map, Map<Integer, List<VisualPowerPointInfo>> powerMap) {
        String newPath = path;
        String[] split = path.split("/");
        if (!path.toLowerCase().contains("powerpoint")) {
            String id = split[split.length - 1];
            if (StrUtil.isNumeric(id)) {
                if (CollectionUtil.isNotEmpty(map.get(Integer.valueOf(id)))) {
                    newPath = path.replace(id, map.get(Integer.valueOf(id)).get(0).getId().toString());
                }
            }
        } else {
            String id = split[split.length - 2];
            if (StrUtil.isNumeric(id)) {
                newPath = path.replace(id, powerMap.get(Integer.valueOf(id)).get(0).getId().toString());
            }
        }

        return newPath;
    }

    private void copyCube(String tenantId, ReportDatasource datasource, CubeData cubeData,
                          List<CubeTableColumn> tableColumns, List<CubeDataPermission> entityList,
                          List<CubeTableColumn> cubeTableColumns, List<CubeData> cubes, AtomicInteger num,
                          CubeWorkspace workspace) {
        int cubeId = num.getAndAdd(1);
        cubeData.setTenantId(tenantId);
        isSave(datasource, cubeData, null, tenantId, tableColumns, entityList, cubeTableColumns, cubes, cubeId, workspace);
    }

    private boolean isSave(ReportDatasource reportDatasource, CubeData cubeData, CubeSqlView view, String tenantId,
                           List<CubeTableColumn> tableColumns, List<CubeDataPermission> entityList, List<CubeTableColumn> cubeTableColumn,
                           List<CubeData> cubes, int cubeId, CubeWorkspace workspace) {
        Map<Integer, List<CubeTableColumn>> map = cubeTableColumn.stream().collect(Collectors.groupingBy(CubeTableColumn::getCubeDataId));
        // 查询数据集的列的集合
        List<CubeTableColumn> cubeTableColumns = map.get(cubeData.getId());
        // 查询数据集的权限组
//        List<CubeDataPermission> cubeDataPermissions = cubeDataPermissionService.list(Wrappers.<CubeDataPermission>lambdaQuery()
//                .eq(CubeDataPermission::getCubeDataId, cubeData.getId()));
        List<CubeDataPermission> cubeDataPermissions = null;
        Integer id = cubeData.getId();
        // 数据集数据复制
        cubeData.setOldCubeDataId(id);
        cubeData.setId(null);
        cubeData.setCreateTime(new Date());
        cubeData.setCreateBy("管理员");
        cubeData.setCreateUserId(0L);
        cubeData.setWorkspaceId(workspace.getId());
        cubeData.setDatasourceId(reportDatasource.getId());
        cubeData.setIsEdit(isEdit);
        cubes.add(cubeData);
        // 复制数据集的列
        if (CollectionUtil.isNotEmpty(cubeTableColumns)) {
            for (CubeTableColumn tableColumn : cubeTableColumns) {
                tableColumn.setCubeDataId(cubeId);
                if (tableColumn.getId() != null && tableColumn.getId() > 0) {
                    // 因为是新增所以要将原来的id置为空
                    tableColumn.setId(null);
                    tableColumn.setTenantId(tenantId);
                    tableColumns.add(tableColumn);
                }
            }
        }
        // 复制权限组
        if (CollectionUtil.isNotEmpty(cubeDataPermissions)) {
            for (CubeDataPermission cubeDataPerm : cubeDataPermissions) {
                cubeDataPerm.setId(null);
                cubeDataPerm.setCreateUserId(0L);
                cubeDataPerm.setCreateTime(new Date());
                cubeDataPerm.setCubeDataId(cubeId);
                // 替换行过滤器
                String filterJson = cubeDataService.exchangeFilterJson(tableColumns, cubeDataPerm.getFilterJson());
                cubeDataPerm.setFilterJson(filterJson);
                // 替换列过滤器
                String excludeColumnIds = cubeDataService.exchangeExcludeColumnIds(tableColumns, cubeDataPerm.getExcludeColumnIds());
                cubeDataPerm.setExcludeColumnIds(excludeColumnIds);
                // 设置过滤sql
//                DsTypeEnum dsTypeEnum = DsTypeEnum.resolve(reportDatasource.getDsType());
//                PreviewBaseDto previewDto = JSON.parseObject(filterJson, PreviewBaseDto.class);
//                if (previewDto != null) {
//                    //解析SQL
//                    Tuple2<String, String> whereHavingSqlTuple = querySqlService.parseFilters(JSON.parseArray(previewDto.getFilterList(), ComponentFilter.class),
//                            JSON.parseObject(previewDto.getFilterRelation(), ComponentFilterRelation.class), dsTypeEnum, previewDto.getExtraParams());
//                    cubeDataPerm.setFilterSql(whereHavingSqlTuple.getT1());
//                }
                entityList.add(cubeDataPerm);
            }
        }
        return true;
    }

    private void copyVisualData(String tenantId, VisualData visualData, List<CubeData> cubeDataList, List<CubeData> list, List<VisualComponent> visualComponentList,
                                List<VisualComponent> visualComponent, List<CubeTableColumn> tableColumns, List<VisualData> visualDataList, AtomicInteger num,
                                VisualClassify classify, ReportDatasource oldDatasource, ReportDatasource reportDatasource) {
        int visualId = num.getAndAdd(1);
        Map<Integer, List<VisualComponent>> map = visualComponent.stream().collect(Collectors.groupingBy(VisualComponent::getVisualId));
        Map<Integer, List<CubeTableColumn>> columMap = tableColumns.stream().collect(Collectors.groupingBy(CubeTableColumn::getCubeDataId));
        int visualDataId = visualData.getId();
        // 根据大屏 id 查询组件信息
        List<VisualComponent> visualComponents = map.get(visualData.getId());
        List<VisualComponent> components = new ArrayList<>();
        List<Integer> cubeDataIdList = new LinkedList<>();
        Map<Integer, List<VisualComponent>> componentMap = new HashMap<>();
        Map<Integer, List<VisualComponent>> listMap = new HashMap<>();

        if (CollectionUtil.isNotEmpty(visualComponents)) {
            visualComponents.forEach(m -> {
                if (m.getCubeDataId() != null && m.getCubeDataId() > 0) {
                    components.add(m);
                }
            });
            // 以前的数据集 id
            cubeDataIdList = components.stream().map(VisualComponent::getCubeDataId).collect(Collectors.toList());
            // 以前的自己拿
            listMap = components.stream().collect(Collectors.groupingBy(VisualComponent::getCubeDataId));
            // 以前的组件
            componentMap = visualComponents.stream().collect(Collectors.groupingBy(VisualComponent::getId));
        }

        List<CubeData> cubeData1 = new ArrayList<>();
        for (CubeData cubeData : cubeDataList) {
            for (Integer id : cubeDataIdList) {
                if (id.equals(cubeData.getId())) {
                    cubeData1.add(cubeData);
                }
            }
        }
        // 新的数据集id集合
        List<Integer> cubeDataIds = new ArrayList<>();
        List<CubeData> cubeDatas = new ArrayList<>();
        List<CopyCubeDataVo> copyCubeDataVoList = new ArrayList<>();
        for (CubeData data : list) {
            for (Integer cubeId : cubeDataIdList) {
                if (data.getOldCubeDataId().equals(cubeId)) {
                    cubeDatas.add(data);
                    cubeDataIds.add(data.getId());
                }
            }
            if (CollectionUtil.isNotEmpty(listMap.get(data.getOldCubeDataId()))) {
                for (VisualComponent component : listMap.get(data.getOldCubeDataId())) {
                    CopyCubeDataVo cubeDataVo = new CopyCubeDataVo();
                    cubeDataVo.setNewCubeDataId(data.getId());
                    cubeDataVo.setComponentId(component.getId());
                    copyCubeDataVoList.add(cubeDataVo);
                }
            }
        }

        visualData.setTenantId(tenantId);
        visualData.setId(null);
        visualData.setCreateDept(0L);
        visualData.setCreateUser(0L);
        visualData.setCreateTime(LocalDateTime.now());
        visualData.setUpdateUser(0L);
        visualData.setUpdateTime(LocalDateTime.now());
        visualData.setIsEdit(isEdit);
        visualData.setRefVisualId(visualDataId);
        visualData.setCategory(classify.getId());
//        boolean save = visualDataService.save(visualData);
        visualDataList.add(visualData);
        Map<Integer, List<CubeData>> cubeDataMap = cubeDatas.stream().collect(Collectors.groupingBy(CubeData::getId));

        for (CopyCubeDataVo cubeDataVo : copyCubeDataVoList) {
            // 如果需要复制的数据集为空则需要判断一下当前
            if (CollectionUtil.isEmpty(cubeDataMap) || CollectionUtil.isEmpty(cubeDataMap.get(cubeDataVo.getNewCubeDataId()))) {
                continue;
            }
            CubeData newCubeData = cubeDataMap.get(cubeDataVo.getNewCubeDataId()).get(0);
            if (newCubeData == null) {
                throw new RuntimeException("数据集不存在");
            }
            // 获取组件信息
            VisualComponent component = componentMap.get(cubeDataVo.getComponentId()).get(0);
            // 删除map中使用到的组件防止后续操作重复组件信息
            componentMap.remove(cubeDataVo.getComponentId());
            component.setId(null);
            component.setVisualId(visualId);
            component.setCubeDataId(newCubeData.getId());
            component.setDatasourceId(newCubeData.getDatasourceId());
            component.setCreateTime(new Date());
            component.setCreateUserId(0L);
            component.setCreateBy("管理员");

            List<CubeTableColumn> newCubeTableColumns = columMap.get(newCubeData.getId());

            // 复制过滤器json
            String filterJson = component.getFilterJson();
            String copyFilterJson = visualDataService.exchangeFilterJsn(filterJson, newCubeTableColumns);
            component.setFilterJson(copyFilterJson);
            // 复制字段json
            String columnJson = component.getColumnJson();
            String copyColumnJson = visualDataService.exchangeCloumnJson(columnJson, newCubeTableColumns);
            component.setColumnJson(copyColumnJson);
            // 复制字段id集合
            String columnIdList = component.getColumnIdList();
            if (StrUtil.isNotEmpty(columnIdList) && !columnIdList.equals("[]") && CollectionUtil.isNotEmpty(newCubeTableColumns)) {
                String copyColumnIdList = visualDataService.exchangeColumnIdList(columnIdList, newCubeTableColumns);
                component.setColumnIdList(copyColumnIdList);
            }
            // 复制extJson
            String extJson = component.getExtJson();
            String copyExtJson = visualDataService.exchangeExtJson(extJson, newCubeTableColumns);
            component.setExtJson(copyExtJson);
            // 复制 sortJosn
            String sortJson = component.getSortJson();
            String copySortJson = visualDataService.exchangeSortJson(sortJson, newCubeTableColumns);
            component.setSortJson(copySortJson);
            visualComponentList.add(component);
        }
        // 如果组件map不为空则表明有的组件并没有被复制
        if (CollectionUtil.isNotEmpty(componentMap)) {
            componentMap.forEach((k, v) -> {
                VisualComponent component = v.get(0);
                component.setId(null);
                component.setVisualId(visualId);
                component.setCreateTime(new Date());
                component.setCreateUserId(0L);
                component.setCreateBy("管理员");
                if (CollectionUtil.isNotEmpty(cubeDataIds)) {
                    List<CubeTableColumn> newCubeTableColumns = new LinkedList<>();
                    for (Integer cubeDataId : cubeDataIds) {
                        if (CollectionUtil.isNotEmpty(columMap) && CollectionUtil.isNotEmpty(columMap.get(cubeDataId))) {
                            newCubeTableColumns.addAll(columMap.get(cubeDataId));
                        }
                    }
                    // 对没有复制的组件信息再次进行过滤筛选，
                    if (StrUtil.isEmpty(component.getColumnJson())) {
                        String cloumnJson = visualDataService.exchangeCloumnJson(component.getColumnJson(), newCubeTableColumns);
                        component.setColumnJson(cloumnJson);
                    }
                    if (StrUtil.isNotEmpty(component.getFilterJson())) {
                        String filterJsn = visualDataService.exchangeFilterJsn(component.getFilterJson(), newCubeTableColumns);
                        component.setFilterJson(filterJsn);
                    }
                    if (StrUtil.isNotEmpty(component.getExtJson())) {
                        String extJson = visualDataService.exchangeExtJson(component.getExtJson(), newCubeTableColumns);
                        component.setExtJson(extJson);
                    }
                }
                visualComponentList.add(component);
            });
        }
    }
}
