package org.springblade.visual.service;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.model.CubeTableColumn;
import org.springblade.cube.service.CubeDataService;
import org.springblade.cube.service.CubeTableColumnService;
import org.springblade.datasource.context.DialectContext;
import org.springblade.datasource.model.TableColumnDto;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.service.IReportDatasourceService;
import org.springblade.visual.dto.ComponentDto;
import org.springblade.visual.dto.TemplateDto;
import org.springblade.visual.entity.*;
import org.springblade.visual.enums.VisualTypeEnum;
import org.springblade.visual.mapper.VisualTemplateMapper;
import org.springblade.visual.vo.CopyCubeDataVo;
import org.springblade.visual.vo.CopyVisualDataCubeDataVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

@Service
public class VisualTemplateService extends ServiceImpl<VisualTemplateMapper, VisualTemplate> {

    @Autowired
    private VisualComponentService visualComponentService;

    @Autowired
    private CubeDataService cubeDataService;

    @Autowired
    private CubeTableColumnService cubeTableColumnService;

    @Autowired
    private IReportDatasourceService reportDatasourceService;

    @Autowired
    private DialectContext dialectContext;

    public Integer doSaveOrUpdate(TemplateDto templateDto, BladeUser user) {
        VisualTemplate template = templateDto.getTemplate();
        Long userId = SecureUtil.getUserId();
        String deptId = SecureUtil.getDeptId();
        if(template.getId() == null) { //新增
            template.setCreateTime(LocalDateTime.now());
            template.setCreateUser(userId);
            template.setCreateDept(Func.toLong(deptId, 0L));
        }else {
            template.setUpdateTime(LocalDateTime.now());
            template.setUpdateUser(userId);
        }
        template.setTenantId(user.getTenantId());
        //1.新增或修改
        boolean ret = this.saveOrUpdate(template);
        if(ret) {
            List<ComponentDto> componentList = templateDto.getComponentList();
            //2.新增组件和组件字段
            visualComponentService.batchSaveComponent(template.getId(), componentList, user);

            //3.仅添加大屏时：权限组处理【前端在编辑时，不显示权限组选择】
//            if(CollectionUtil.isNotEmpty(templateDto.getDataPermIdList())) {
//                // 保存公共大屏与权限组的关系
//                if (visualDto.getVisualData().getVisualDataType() == 0) {
//                    List<VisualDataPermission> dataList = new ArrayList<>(visualDto.getDataPermIdList().size());
//                    visualDto.getDataPermIdList().forEach(t -> {
//                        VisualDataPermission entity = new VisualDataPermission();
//                        entity.setCreateBy(bladeUser.getNickName());
//                        entity.setCreateTime(new Date());
//                        entity.setCreateUserId(bladeUser.getUserId());
//                        entity.setDataPermId(t);
//                        entity.setVisualId(visualData.getId());
//                        dataList.add(entity);
//                    });
//                    visualDataPermissionService.saveBatch(dataList);
//                } else {
//                    // 保存自助报表与权限组的关系
//                    List<VisualSelfReportPermission> dataList = new ArrayList<>(visualDto.getDataPermIdList().size());
//                    visualDto.getDataPermIdList().forEach(t -> {
//                        VisualSelfReportPermission entity = new VisualSelfReportPermission();
//                        entity.setCreateBy(bladeUser.getNickName());
//                        entity.setCreateTime(new Date());
//                        entity.setCreateUserId(bladeUser.getUserId());
//                        entity.setDataPermId(t);
//                        entity.setSelfReportId(visualData.getId());
//                        dataList.add(entity);
//                    });
//                    visualSelfReportPermissionService.saveBatch(dataList);
//                }
//            }
        }
        return template.getId();
    }

    /**
     * 删除模板信息
     */
    public void doRemove(List<Integer> idList) {
        baseMapper.deleteBatchIds(idList);
        visualComponentService.remove(Wrappers.<VisualComponent>lambdaQuery().in(VisualComponent::getVisualId, idList));
//        List<VisualTemplate> templates = baseMapper.selectBatchIds(idList);
//        Map<Integer, List<VisualTemplate>> visualDataMap = templates.stream().collect(Collectors.groupingBy(VisualTemplate::getVisualDataType));
//        if (CollectionUtil.isNotEmpty(visualDataMap)){
//            if (CollectionUtil.isNotEmpty(visualDataMap.get(0))){
//                // 删除大屏与权限组的关系
//                List<Integer> ids = visualDataMap.get(0).stream().map(VisualData::getId).collect(Collectors.toList());
//                visualDataPermissionService.remove(Wrappers.<VisualDataPermission>lambdaQuery().in(VisualDataPermission::getVisualId, ids));
//            } else if (CollectionUtil.isNotEmpty(visualDataMap.get(1))){
//                // 删除自助报表与权限组的关系
//                List<Integer> ids = visualDataMap.get(1).stream().map(VisualData::getId).collect(Collectors.toList());
//                visualSelfReportPermissionService.remove(Wrappers.<VisualSelfReportPermission>lambdaQuery().in(VisualSelfReportPermission::getSelfReportId,ids));
//            }
//        }
    }

    /**
     * 复制模板信息
     */
    public VisualTemplate doCopy(Integer templateId, Integer visualCategoryId, Integer reportInfoId,
                                Integer powerPointId, VisualTypeEnum visualType, BladeUser bladeUser, String titlePrefix,String tenantId) {

        //批量添加大屏
        VisualTemplate visualTemplate = this.getById(templateId);
        List<VisualComponent> visualComponentList = new ArrayList<>();
        if (visualTemplate != null){
            final int oldVisualId = visualTemplate.getId();
            int ret = saveVisualData(visualTemplate,visualCategoryId,reportInfoId,powerPointId,visualType,bladeUser,titlePrefix,tenantId);
            if(ret > 0) { //插入成功
                //查询旧组件列表
                List<VisualComponent> componetList = visualComponentService.list(Wrappers.<VisualComponent>lambdaQuery().eq(VisualComponent::getVisualId, oldVisualId));
                if(componetList != null && !componetList.isEmpty()) {
                    componetList.forEach(f -> {
                        f.setId(null);
                        f.setVisualId(visualTemplate.getId());
                        if (StrUtil.isNotEmpty(tenantId)) {
                            f.setCreateBy("管理员");
                            f.setCreateUserId(0L);
                        } else {
                            f.setCreateBy(bladeUser.getNickName());
                            f.setCreateUserId(bladeUser.getUserId());
                        }
                        f.setCreateTime(new Date());

                    });
                    visualComponentList.addAll(componetList);
                }
            }
        }
        //批量添加组件列表
        if(CollectionUtil.isNotEmpty(visualComponentList)) {
            visualComponentService.saveBatch(visualComponentList);
        }
        return visualTemplate;
    }

    private int saveVisualData(VisualTemplate item,Integer visualCategoryId, Integer reportInfoId,
                               Integer powerPointId, VisualTypeEnum visualType, BladeUser bladeUser, String titlePrefix,String tenantId){
        final int oldVisualId = item.getId();
        item.setId(null);
        String name = titlePrefix + item.getTitle();
        if (StrUtil.isNotEmpty(name) && !name.equals("[副本]")){
            name = titlePrefix;
        }
        if (StrUtil.isNotEmpty(tenantId)){
            item.setTenantId(tenantId);
        }
        item.setTitle(name);
        item.setRefVisualId(oldVisualId);
        item.setVisualType(visualType);
        item.setCreateTime(LocalDateTime.now());
        item.setHistoryReportId(reportInfoId);
        if (StrUtil.isNotEmpty(tenantId)) {
            item.setCreateUser(0L);
            item.setUpdateUser(0L);
            item.setCreateDept(0L);
        } else {
            item.setCreateUser(bladeUser.getUserId());
            item.setUpdateUser(bladeUser.getUserId());
            item.setCreateDept(Func.toLong(bladeUser.getDeptId()));
        }
        item.setUpdateTime(LocalDateTime.now());
        item.setOrderIndex(9999);
        if(visualCategoryId != null) {
            item.setCategory(visualCategoryId);
        }
        if(powerPointId != null) {
            item.setPowerPointId(powerPointId);
        }
        return baseMapper.insert(item);
    }

    public R<Integer> doCopyCube(CopyVisualDataCubeDataVo cubeData, BladeUser bladeUser) {
        VisualTemplate template = this.getById(cubeData.getVisualId());
        final int templateId = template.getId();
        List<VisualComponent> visualComponentList = new ArrayList<>();
        VisualTypeEnum visualType = cubeData.getVisualType();
        if (cubeData.getVisualType() == null){
            visualType = VisualTypeEnum.GENERAL;
        }
        int ref = saveVisualData(template, cubeData.getNewCategoryId(), null, null, visualType, bladeUser, cubeData.getVisualName(),cubeData.getTenantId());
        Map<Integer, List<VisualComponent>> componentMap = null;
        if (ref > 0){
            List<CopyCubeDataVo> cubeDataList = cubeData.getCubeDataList();
            // 新的数据集id集合
            List<Integer> cubeDataIds = cubeDataList.stream().map(CopyCubeDataVo::getNewCubeDataId).collect(Collectors.toList());
            // 根据大屏id查询组件信息
            List<VisualComponent> visualComponents = visualComponentService.list(Wrappers.<VisualComponent>lambdaQuery()
                    .eq(VisualComponent::getVisualId,templateId));
            List<Integer> cubeDataIdLIst = visualComponents.stream().map(VisualComponent::getCubeDataId).collect(Collectors.toList());
            // 查询组件的数据集
            List<CubeData> cubeData1 = cubeDataService.getBaseMapper().selectBatchIds(cubeDataIdLIst);
            Map<Integer, List<CubeData>> oldCubeDataMap = cubeData1.stream().collect(Collectors.groupingBy(CubeData::getId));

            componentMap = visualComponents.stream().collect(Collectors.groupingBy(VisualComponent::getId));
            // 查询新的数据集
            List<CubeData> cubeDatas;
            // 如果租户id不为空，则需要根据租户id查询数据集
            if (StrUtil.isNotEmpty(cubeData.getTenantId())) {
                cubeDatas = cubeDataService.list(Wrappers.<CubeData>lambdaQuery()
                        .eq(CubeData::getTenantId,cubeData.getTenantId())
                        .in(CubeData::getId,cubeDataIds));
            } else {
                cubeDatas = cubeDataService.getBaseMapper().selectBatchIds(cubeDataIds);
            }
            Map<Integer, List<CubeData>> cubeDataMap = cubeDatas.stream().collect(Collectors.groupingBy(CubeData::getId));

            for (CopyCubeDataVo cubeDataVo : cubeDataList) {
                // 如果需要复制的数据集为空则需要判断一下当前
                if (CollectionUtil.isEmpty(cubeDataMap) || CollectionUtil.isEmpty(cubeDataMap.get(cubeDataVo.getNewCubeDataId()))){
                    continue;
                }
                CubeData newCubeData = cubeDataMap.get(cubeDataVo.getNewCubeDataId()).get(0);
                if (newCubeData == null){
                    return R.fail("id为"+cubeDataVo.getNewCubeDataId()+"的数据集不存在");
                }
                // 获取组件信息
                VisualComponent component = componentMap.get(cubeDataVo.getComponentId()).get(0);
                // 删除map中使用到的组件防止后续操作重复组件信息
                componentMap.remove(cubeDataVo.getComponentId());
                CubeData oldCubeData = oldCubeDataMap.get(component.getCubeDataId()).get(0);
                //
                List<String> tableColum = getTableColum(getReportDatasource(newCubeData.getDatasourceId()), getTableName(newCubeData.getCubeTableNames()));
                List<String> oldTableColum = getTableColum(getReportDatasource(oldCubeData.getDatasourceId()), getTableName(oldCubeData.getCubeTableNames()));
                oldTableColum.removeAll(tableColum);
                if (CollectionUtil.isNotEmpty(oldTableColum)){
                    StringBuilder msg = new StringBuilder();
                    for (String s : oldTableColum) {
                        msg.append(s).append(",");
                    }
                    return R.fail("组件"+component.getName()+"复制的数据集"+newCubeData.getCubeName()+"数据不全,缺少以下字段"+msg);
                }
                component.setId(null);
                component.setVisualId(template.getId());
                component.setCubeDataId(newCubeData.getId());
                component.setDatasourceId(newCubeData.getDatasourceId());
                component.setCreateTime(new Date());
                if (StrUtil.isNotEmpty(cubeData.getTenantId())) {
                    component.setCreateUserId(0L);
                    component.setCreateBy("管理员");
                } else {
                    component.setCreateUserId(bladeUser.getUserId());
                    component.setCreateBy(bladeUser.getNickName());
                }
                List<CubeTableColumn> newCubeTableColumns;
                if (StrUtil.isNotEmpty(cubeData.getTenantId())) {
                    newCubeTableColumns = cubeTableColumnService.list(Wrappers.<CubeTableColumn>lambdaQuery()
                            .eq(CubeTableColumn::getCubeDataId, newCubeData.getId())
                            .eq(CubeTableColumn::getTenantId,cubeData.getTenantId()));
                } else {
                    newCubeTableColumns = cubeTableColumnService.list(Wrappers.<CubeTableColumn>lambdaQuery()
                            .eq(CubeTableColumn::getCubeDataId, newCubeData.getId()));
                }
                // 复制过滤器json
                String filterJson = component.getFilterJson();
                String copyFilterJson = exchangeFilterJsn(filterJson,newCubeTableColumns);
                component.setFilterJson(copyFilterJson);
                // 复制字段json
                String columnJson = component.getColumnJson();
                String copyColumnJson = exchangeCloumnJson(columnJson,newCubeTableColumns);
                component.setColumnJson(copyColumnJson);
                // 复制字段id集合
                String columnIdList = component.getColumnIdList();
                String copyColumnIdList = exchangeColumnIdList(columnIdList,newCubeTableColumns);
                component.setColumnIdList(copyColumnIdList);
                // 复制extJson
                String extJson = component.getExtJson();
                String copyExtJson = exchangeExtJson(extJson,newCubeTableColumns);
                component.setExtJson(copyExtJson);
                // 复制 sortJosn
                String sortJson = component.getSortJson();
                String copySortJson = exchangeSortJson(sortJson,newCubeTableColumns);
                component.setSortJson(copySortJson);
                visualComponentList.add(component);
            }
            // 如果组件map不为空则表明有的组件并没有被复制
            if (CollectionUtil.isNotEmpty(componentMap)){
                componentMap.forEach((k,v) -> {
                    VisualComponent component = v.get(0);
                    component.setId(null);
                    component.setVisualId(template.getId());
                    component.setCreateTime(new Date());
                    if (StrUtil.isNotEmpty(cubeData.getTenantId())) {
                        component.setCreateUserId(0L);
                        component.setCreateBy("管理员");
                    } else {
                        component.setCreateUserId(bladeUser.getUserId());
                        component.setCreateBy(bladeUser.getNickName());
                    }
                    List<CubeTableColumn> newCubeTableColumns;
                    if (StrUtil.isNotEmpty(cubeData.getTenantId())) {
                        newCubeTableColumns = cubeTableColumnService.list(Wrappers.<CubeTableColumn>lambdaQuery()
                                .in(CubeTableColumn::getCubeDataId, cubeDataIds)
                                .eq(CubeTableColumn::getTenantId,cubeData.getTenantId()));
                    } else {
                        newCubeTableColumns = cubeTableColumnService.list(Wrappers.<CubeTableColumn>lambdaQuery()
                                .in(CubeTableColumn::getCubeDataId, cubeDataIds));
                    }
                    // 对没有复制的组件信息再次进行过滤筛选，
                    if (StrUtil.isEmpty(component.getColumnJson())) {
                        String cloumnJson = exchangeCloumnJson(component.getColumnJson(), newCubeTableColumns);
                        component.setColumnJson(cloumnJson);
                    }
                    if (StrUtil.isNotEmpty(component.getFilterJson())) {
                        String filterJsn = exchangeFilterJsn(component.getFilterJson(), newCubeTableColumns);
                        component.setFilterJson(filterJsn);
                    }
                    if (StrUtil.isNotEmpty(component.getExtJson())) {
                        String extJson = exchangeExtJson(component.getExtJson(), newCubeTableColumns);
                        component.setExtJson(extJson);
                    }
                    visualComponentList.add(component);
                });
            }
            if (CollectionUtil.isNotEmpty(visualComponentList)) {
                visualComponentService.saveBatch(visualComponentList);
            }
        }

        return R.success("操作成功");
    }

    public List<String> getTableColum(ReportDatasource reportDatasource, String tableName) {
        List<TableColumnDto> reportColumnList = dialectContext.getDialect(reportDatasource.getDsTypeSyl()).listTableColumnsByTableViewName(reportDatasource, tableName);

        return reportColumnList.stream().map(TableColumnDto::getColumnName).collect(Collectors.toList());
    }

    public ReportDatasource getReportDatasource(Integer datasourceId) {
        return reportDatasourceService.getById(datasourceId);
    }

    public String getTableName(String str){
        JSONArray jsonArray = JSONUtil.parseArray(str);
        JSONObject json = JSONUtil.parseObj(jsonArray.get(0));
        String tableName = null;
        if (json.containsKey("tableName")){
            tableName = json.getStr("tableName");
        }
        return tableName;
    }

    /**
     * 修改过滤器cubeColumnId
     */
    public String exchangeFilterJsn(String filterJson,List<CubeTableColumn> newCubeTableColumns) {
        if (StrUtil.isEmpty(filterJson)){
            return null;
        }
        JSONArray arrays = new JSONArray();
        JSONArray jsonArray = JSONUtil.parseArray(filterJson);
        for (Object column : jsonArray) {
            JSONObject json = JSONUtil.parseObj(column);
            if (!json.containsKey("column")){
                arrays.add(json);
                continue;
            }
            String columnComment = StrUtil.isNotEmpty(json.getStr("columnComment")) ? json.getStr("columnComment") : "";
            String columnName = StrUtil.isNotEmpty(json.getStr("columnName")) ? json.getStr("columnName") : "";
            String tableName = StrUtil.isNotEmpty(json.getStr("tableName")) ? json.getStr("tableName") : "";
            if (CollectionUtil.isNotEmpty(newCubeTableColumns)) {
                for (CubeTableColumn tableColumn : newCubeTableColumns) {
                    String tColumnComment = StrUtil.isNotEmpty(tableColumn.getColumnComment()) ? tableColumn.getColumnComment() : "";
                    String tColumnName = StrUtil.isNotEmpty(tableColumn.getColumnName()) ? tableColumn.getColumnName() : "";
                    String tTableName = StrUtil.isNotEmpty(tableColumn.getTableName()) ? tableColumn.getTableName() : "";
                    if (tColumnComment.equals(columnComment)
                            && tColumnName.equals(columnName)
                            && tTableName.equals(tableName)) {
                        json.set("cubeColumnId", tableColumn.getId());
                    }
                }
            }
            arrays.add(json);
        }

        return JSONUtil.toJsonStr(arrays);
    }

    public String exchangeCloumnJson(String columnJson, List<CubeTableColumn> newCubeTableColumns) {
        if (StrUtil.isEmpty(columnJson)){
            return null;
        }
        JSONArray arrays = new JSONArray();
        JSONArray jsonArray = JSONUtil.parseArray(columnJson);
        for (Object column : jsonArray) {
            JSONObject json = JSONUtil.parseObj(column);
            String columnComment = StrUtil.isNotEmpty(json.getStr("columnComment")) ? json.getStr("columnComment") : "";
            String columnName = StrUtil.isNotEmpty(json.getStr("columnName")) ? json.getStr("columnName") : "";
            String tableName = StrUtil.isNotEmpty(json.getStr("tableName")) ? json.getStr("tableName") : "";
            if (CollectionUtil.isNotEmpty(newCubeTableColumns)) {
                for (CubeTableColumn tableColumn : newCubeTableColumns) {
                    String tColumnComment = StrUtil.isNotEmpty(tableColumn.getColumnComment()) ? tableColumn.getColumnComment() : "";
                    String tColumnName = StrUtil.isNotEmpty(tableColumn.getColumnName()) ? tableColumn.getColumnName() : "";
                    String tTableName = StrUtil.isNotEmpty(tableColumn.getTableName()) ? tableColumn.getTableName() : "";
                    if (tColumnComment.equals(columnComment)
                            && tColumnName.equals(columnName)
                            && tTableName.equals(tableName)) {
                        json.set("cubeColumnId", tableColumn.getId());
                    }
                }
            }
            arrays.add(json);
        }

        return JSONUtil.toJsonStr(arrays);
    }

    public String exchangeColumnIdList(String columnIdList, List<CubeTableColumn> newCubeTableColumns) {
        if (StrUtil.isEmpty(columnIdList)){
            return null;
        }
        JSONArray arrays = new JSONArray();
        JSONArray jsonArray = JSONUtil.parseArray(columnIdList);
        List<Integer> ids = new ArrayList<>();
        for (Object column : jsonArray) {
            int i = Integer.parseInt(column.toString());
            ids.add(i);
        }
        List<CubeTableColumn> cubeTableColumns = cubeTableColumnService.getBaseMapper().selectBatchIds(ids);
        if (CollectionUtil.isNotEmpty(newCubeTableColumns)) {
            for (CubeTableColumn newCubeTableColumn : newCubeTableColumns) {
                for (CubeTableColumn cubeTableColumn : cubeTableColumns) {
                    String newColumnComment = StrUtil.isNotEmpty(newCubeTableColumn.getColumnComment()) ? newCubeTableColumn.getColumnComment() : "";
                    String newColumnName = StrUtil.isNotEmpty(newCubeTableColumn.getColumnName()) ? newCubeTableColumn.getColumnName() : "";
                    String newTableName = StrUtil.isNotEmpty(newCubeTableColumn.getTableName()) ? newCubeTableColumn.getTableName() : "";
                    String columnComment = StrUtil.isNotEmpty(cubeTableColumn.getColumnComment()) ? cubeTableColumn.getColumnComment() : "";
                    String columnName = StrUtil.isNotEmpty(cubeTableColumn.getColumnName()) ? cubeTableColumn.getColumnName() : "";
                    String tableName = StrUtil.isNotEmpty(cubeTableColumn.getTableName()) ? cubeTableColumn.getTableName() : "";
                    if (newColumnComment.equals(columnComment)
                            && newColumnName.equals(columnName)
                            && newTableName.equals(tableName)) {
                        arrays.add(newCubeTableColumn.getId());
                    }
                }
            }
        }

        return JSONUtil.toJsonStr(arrays);
    }

    public String exchangeExtJson(String extJson, List<CubeTableColumn> newCubeTableColumns) {
        if (StrUtil.isEmpty(extJson)){
            return null;
        }
        JSONArray array = new JSONArray();
        JSONObject json = JSONUtil.parseObj(extJson);
        if (json.containsKey("filterList")){
            JSONArray jsonArray = JSONUtil.parseArray(json.get("filterList"));
            for (Object o : jsonArray) {
                JSONObject filterList = JSONUtil.parseObj(o);
                String filterType = filterList.getStr("filterType");
                if (filterType.equals("normal")) {
                    exchageNormal(newCubeTableColumns, filterList);
                } else if (filterType.equals("tree")) {
                    exchangeTree(newCubeTableColumns, filterList);
                } else {
                    exchangeTime(newCubeTableColumns, filterList);
                }
                array.add(filterList);
            }
            json.set("filterList",array);
            return json.toString();
        } else {
            return json.toString();
        }
    }

    public void exchangeTime(List<CubeTableColumn> newCubeTableColumns, JSONObject filterList) {
        String chartList = filterList.getStr("chartList");
        JSONArray chartListArray = JSONUtil.parseArray(chartList);
        JSONArray array = new JSONArray();
        for (Object obj : chartListArray) {
            JSONObject entries = JSONUtil.parseObj(obj);
            if (entries.containsKey("tfield")){
                JSONObject tfield = JSONUtil.parseObj(entries.getStr("tfield"));
                String columnComment = StrUtil.isNotEmpty(tfield.getStr("columnComment")) ? tfield.getStr("columnComment") : "";
                String columnName = StrUtil.isNotEmpty(tfield.getStr("columnName")) ? tfield.getStr("columnName") : "";
                String tableName = StrUtil.isNotEmpty(tfield.getStr("tableName")) ? tfield.getStr("tableName") : "";
                for (CubeTableColumn newCubeTableColumn : newCubeTableColumns) {
                    String newColumnComment = StrUtil.isNotEmpty(newCubeTableColumn.getColumnComment()) ? newCubeTableColumn.getColumnComment() : "";
                    String newColumnName = StrUtil.isNotEmpty(newCubeTableColumn.getColumnName()) ? newCubeTableColumn.getColumnName() : "";
                    String newTableName = StrUtil.isNotEmpty(newCubeTableColumn.getTableName()) ? newCubeTableColumn.getTableName() : "";
                    if (newColumnComment.equals(columnComment)
                            && newColumnName.equals(columnName)
                            && newTableName.equals(tableName)) {
                        entries.set("cubeDataId",newCubeTableColumn.getCubeDataId());
                        entries.set("tid",newCubeTableColumn.getId());
                        tfield.set("id",newCubeTableColumn.getId());
                        tfield.set("cubeDataId",newCubeTableColumn.getCubeDataId());
                        entries.set("tfield",tfield);
                    }
                }
            } else {
                JSONObject vfield = JSONUtil.parseObj(entries.getStr("vfield"));
                String columnComment = StrUtil.isNotEmpty(vfield.getStr("columnComment")) ? vfield.getStr("columnComment") : "";
                String columnName = StrUtil.isNotEmpty(vfield.getStr("columnName")) ? vfield.getStr("columnName") : "";
                String tableName = StrUtil.isNotEmpty(vfield.getStr("tableName")) ? vfield.getStr("tableName") : "";
                for (CubeTableColumn newCubeTableColumn : newCubeTableColumns) {
                    String newColumnComment = StrUtil.isNotEmpty(newCubeTableColumn.getColumnComment()) ? newCubeTableColumn.getColumnComment() : "";
                    String newColumnName = StrUtil.isNotEmpty(newCubeTableColumn.getColumnName()) ? newCubeTableColumn.getColumnName() : "";
                    String newTableName = StrUtil.isNotEmpty(newCubeTableColumn.getTableName()) ? newCubeTableColumn.getTableName() : "";
                    if (newColumnComment.equals(columnComment)
                            && newColumnName.equals(columnName)
                            && newTableName.equals(tableName)) {
                        entries.set("vid",newCubeTableColumn.getId());
                        vfield.set("id",newCubeTableColumn.getId());
                        vfield.set("cubeDataId",newCubeTableColumn.getCubeDataId());
                        entries.set("tfield",vfield);
                    }
                }
            }
            array.add(entries);
        }
        // 保存最后的结果
        filterList.set("chartList",array);
    }

    public void exchangeTree(List<CubeTableColumn> newCubeTableColumns, JSONObject filterList) {
        String cubeList = filterList.getStr("cubeList");
        JSONArray jsonArray = JSONUtil.parseArray(cubeList);
        JSONArray array1 = new JSONArray();
        for (Object obj : jsonArray) {
            JSONObject json = JSONUtil.parseObj(obj);
            JSONArray fields = JSONUtil.parseArray(json.getStr("fields"));
            JSONArray newFields = new JSONArray();
            for (Object field : fields) {
                boolean bool = true;
                JSONObject entries = JSONUtil.parseObj(field);
                String columnComment = StrUtil.isNotEmpty(entries.getStr("columnComment")) ? entries.getStr("columnComment") : "";
                String columnName = StrUtil.isNotEmpty(entries.getStr("columnName")) ? entries.getStr("columnName") : "";
                String tableName = StrUtil.isNotEmpty(entries.getStr("tableName")) ? entries.getStr("tableName") : "";
                for (CubeTableColumn newCubeTableColumn : newCubeTableColumns) {
                    String newColumnComment = StrUtil.isNotEmpty(newCubeTableColumn.getColumnComment()) ? newCubeTableColumn.getColumnComment() : "";
                    String newColumnName = StrUtil.isNotEmpty(newCubeTableColumn.getColumnName()) ? newCubeTableColumn.getColumnName() : "";
                    String newTableName = StrUtil.isNotEmpty(newCubeTableColumn.getTableName()) ? newCubeTableColumn.getTableName() : "";
                    if (newColumnComment.equals(columnComment)
                            && newColumnName.equals(columnName)
                            && newTableName.equals(tableName)) {
                        // 数据集id
                        json.set("id",newCubeTableColumn.getCubeDataId());
                        // 数据集id
                        entries.set("cubeDataId",newCubeTableColumn.getCubeDataId());
                        // 数据集字段id
                        entries.set("id",newCubeTableColumn.getId());
                        newFields.add(entries);
                        bool = false;
                    }
                }
                if (!bool) {
                    newFields.add(entries);
                }
            }
            json.set("fields",newFields);
            array1.set(json);
        }

        filterList.set("cubeList",array1);
    }

    public void exchageNormal(List<CubeTableColumn> newCubeTableColumns, JSONObject filterList) {
        JSONArray chartList = JSONUtil.parseArray(filterList.get("chartList"));
        JSONArray array1 = new JSONArray();
        for (Object o1 : chartList) {
            JSONObject chart = JSONUtil.parseObj(o1);
            String table = chart.getStr("tableName");
            String columnName = chart.getStr("columnName");
            for (CubeTableColumn newCubeTableColumn : newCubeTableColumns) {
                if (newCubeTableColumn.getTableName().equals(table)
                        && newCubeTableColumn.getColumnName().equals(columnName)){
                    chart.set("cubeDataId",newCubeTableColumn.getCubeDataId());
                    if (chart.containsKey("fieldId")) {
                        chart.set("fieldId",newCubeTableColumn.getId());
                    }
                    JSONObject obj = JSONUtil.parseObj(chart.getStr("field"));
                    obj.set("cubeDataId",newCubeTableColumn.getId());
                    obj.set("id",newCubeTableColumn.getId());
                    chart.set("field",obj);
                }
            }
            array1.add(chart);
        }
        if (array1.size() < 1){
            filterList.set("chartList",chartList);
        } else {
            filterList.set("chartList",array1);
        }
    }

    public String exchangeSortJson(String sortJson, List<CubeTableColumn> newCubeTableColumns) {
        if (StrUtil.isEmpty(sortJson)) {
            return null;
        }
        JSONArray array = new JSONArray();
        JSONArray sortList = JSONUtil.parseArray(sortJson);
        for (Object o : sortList) {
            JSONObject json = JSONUtil.parseObj(o.toString());
            JSONObject column = JSONUtil.parseObj(json.get("column"));
            String columnName = StrUtil.isNotEmpty(column.getStr("columnName")) ? column.getStr("columnName") : "";
            String columnTitle = StrUtil.isNotEmpty(column.getStr("columnTitle")) ? column.getStr("columnTitle") : "";
            String tableName = StrUtil.isNotEmpty(column.getStr("tableName")) ? column.getStr("tableName") : "";


            for (CubeTableColumn tableColumn : newCubeTableColumns) {
                String oldColumName = StrUtil.isNotEmpty(tableColumn.getColumnName()) ? tableColumn.getColumnName() : "";
                String oldColumnTitle = StrUtil.isNotEmpty(tableColumn.getColumnTitle()) ? tableColumn.getColumnTitle() : "";
                String oldTableName = StrUtil.isNotEmpty(tableColumn.getTableName()) ? tableColumn.getTableName() : "";
                if (oldColumName.equals(columnName) && oldColumnTitle.equals(columnTitle)) {
                    column.set("cubeColumnId",tableColumn.getId());
                    json.set("column",column);
                } else if (oldTableName.equals(tableName) && oldColumName.equals(columnName)) {
                    column.set("cubeColumnId",tableColumn.getId());
                    json.set("column",column);
                }
            }
            array.add(json);
        }

        return array.toString();
    }
}
