/**
 * 
 */
package org.springblade.visual.task;

import java.io.File;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import cn.hutool.crypto.digest.MD5;
import cn.hutool.http.HttpUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.core.tool.utils.StringUtil;
import org.springblade.cube.service.DynamicTableService;
import org.springblade.message.app.TianXiangYunMsgTemplate;
import org.springblade.message.enums.MsgType;
import org.springblade.message.model.AppMsgInfo;
import org.springblade.message.model.AppMsgRequest;
import org.springblade.message.model.EmailInfo;
import org.springblade.message.model.EmailMsgRequest;
import org.springblade.message.model.MsgResponse;
import org.springblade.message.model.SmsInfo;
import org.springblade.message.model.SmsMsgRequest;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.subscribe.entity.VisualSubscribeNotifyConf;
import org.springblade.subscribe.enums.DataType;
import org.springblade.subscribe.enums.NotifyType;
import org.springblade.subscribe.service.VisualSubscribeNotifyConfService;
import org.springblade.subscribe.task.handler.VisualDataPdfHandler;
import org.springblade.subscribe.util.SortUtils;
import org.springblade.system.client.SmsClient;
import org.springblade.system.client.SubscribeNotifyConfClient;
import org.springblade.system.client.UserClient;
import org.springblade.system.client.model.SmsModel;
import org.springblade.system.client.model.UserModel;
import org.springblade.system.client.model.VisualSubscribeNotifyConfModel;
import org.springblade.visual.dto.ComponentColumn;
import org.springblade.visual.dto.QuerySqlParam;
import org.springblade.visual.dto.WarningDto;
import org.springblade.visual.dto.WarningWayDto;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualWarning;
import org.springblade.visual.entity.VisualWarningRecord;
import org.springblade.visual.entity.VisualWarningRecordUser;
import org.springblade.visual.enums.WarningWayEnum;
import org.springblade.visual.service.QuerySqlService;
import org.springblade.visual.service.VisualWarningRecordService;
import org.springblade.visual.service.VisualWarningRecordUserService;
import org.springblade.visual.service.VisualWarningService;
import org.springblade.visual.task.unicom.UnicomSmsSender;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.thread.ThreadFactoryBuilder;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.spring.SpringUtil;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Slf4j
@Component
public class SmartWarningJobHandler {
	
	@Autowired
	private DynamicTableService dynamicTableService;

	@Autowired
	private VisualWarningService visualWarningService;

	@Autowired
	private VisualWarningRecordService visualWarningRecordService;

	@Autowired
	private VisualWarningRecordUserService visualWarningRecordUserService;

	@Autowired
	private SubscribeNotifyConfClient subscribeNotifyConfClient;
	
	@Autowired
	private VisualDataPdfHandler visualDataPdfHandler;

	@Autowired
	private QuerySqlService querySqlService;

	@Autowired
	private UserClient userClient;
	
	@Autowired
	private SmsClient smsClient;
	
	@Value("${blade.unicom:false}")
	private boolean isUnicom;


	@Value("${blade.domain}")
	private String domain;


	final static int THREAD_NUM = 50;
	final ExecutorService threadPool = new ThreadPoolExecutor(
			THREAD_NUM, 
			THREAD_NUM, 
			5,
			TimeUnit.MINUTES,
			new ArrayBlockingQueue<>(3000), 
			new ThreadFactoryBuilder().setDaemon(false).setNamePrefix("SmartWarning-Pool-").build(),
			new ThreadPoolExecutor.DiscardPolicy());
	
	/**
	 * @return
	 */
	@SneakyThrows
	@XxlJob("smartWarningJobHandler")
	public boolean warnJobHandler() {
		VisualWarning visualWarning = null;
		try {
			//获取参数
			String param = XxlJobHelper.getJobParam();
			visualWarning = visualWarningService.getById(Func.toInt(param, 0));
			if(visualWarning == null) {
				throw new RuntimeException("未查询到预警信息");
			}
			XxlJobHelper.log("开始处理预警任务|warnName={}", visualWarning.getName());
			long oneTime = System.currentTimeMillis();
			// 获取字段ID列表存储
			WarningDto warningDto = new WarningDto();
			BeanUtils.copyProperties(visualWarning, warningDto);
			warningDto.setColumnList(visualWarning.getColumnJson());
			warningDto.setFilterList(visualWarning.getFilterJson());
			warningDto.setFilterRelation(visualWarning.getFilterRelationJson());
			R<QuerySqlParam> r = querySqlService.parseQuerySql(warningDto.getCubeDataId(), warningDto, null);
			if (!r.isSuccess()) {
				log.warn("解析SQL出错|warningId={},message={}", visualWarning.getId(), r.getMsg());
				XxlJobHelper.log("解析SQL出错|message={}", r.getMsg());
				throw new RuntimeException("解析SQL出错");
			}
			QuerySqlParam querySqlParam = r.getData();
			ReportDatasource reportDatasource = querySqlParam.getReportDatasource();
			String querySql = querySqlParam.getQuerySql();
			Map<String, String> fieldDict = querySqlParam.getSelectColumnList().stream()
					.collect(Collectors.toMap(ComponentColumn::getColumnAliasName, ComponentColumn::getColumnTitle));
			long twoTime = System.currentTimeMillis();
			List<Map<String, Object>> resultList = dynamicTableService.selectList(reportDatasource, querySql);
			long threeTime = System.currentTimeMillis();
			if (CollectionUtil.isNotEmpty(resultList)) { // 出现预警
				JSONObject resutData = new JSONObject();
				resutData.put("field", fieldDict);
				resutData.put("dataList", resultList);

				//格式化消息内容
				Map<String, Object> jsonObject = resultList.get(0);
				Map<String, Object> dataMap = new HashMap<>();
				jsonObject.entrySet().stream().filter(item -> fieldDict.containsKey(item.getKey())).forEach(entry -> dataMap.put(fieldDict.get(entry.getKey()), entry.getValue()));
				//格式化占位符获取消息体完整内容
				String content = StrUtil.isNotEmpty(visualWarning.getMessageDynamicContent()) ? _parser(visualWarning.getMessageDynamicContent(), dataMap) : null;
				visualWarning.setFormatContent(content);
				
				// 插入预警记录
				VisualWarningRecord warningRecord = new VisualWarningRecord();
				warningRecord.setCreateTime(new Date());
				warningRecord.setWarnData(resutData.toJSONString());
				warningRecord.setWarningId(visualWarning.getId());
				warningRecord.setWarningName(visualWarning.getName());
				warningRecord.setWarnMessageWay(visualWarning.getWarnMessageWay());
				warningRecord.setWarnMessage(content);
				warningRecord.setTenantId(visualWarning.getTenantId());
				visualWarningRecordService.save(warningRecord);
				
				// 根据选定的通知方式开始推送
				List<Long> userIdList = Func.toLongList(visualWarning.getWarnUserIds());
				List<UserModel> userList = userClient.listByIds(userIdList);
				if(CollectionUtil.isEmpty(userList)) {
					XxlJobHelper.log("未查询到用户信息");
					throw new RuntimeException("未查询到用户信息");
				}
				//根据选定的通知方式，开始推送消息
				List<WarningWayDto> checkedWays = JSON.parseArray(visualWarning.getWarningWays(),WarningWayDto.class);
				List<CompletableFuture<Void>> tasks = new ArrayList<>();
				
				//开启多线程推送消息
				if (checkedWays != null && checkedWays.size()>0){
					for (WarningWayDto wayDto : checkedWays){
						if (wayDto.getWarningWayEnum()==WarningWayEnum.SMS){
							CompletableFuture<Void> task = _sendSmsMsg(userList, visualWarning, fieldDict, dataMap);
							if(task != null) {
								tasks.add(task);
							}
						}
					}
				}

				if(checkedWays.stream().anyMatch(item -> item.getWarningWayEnum() == WarningWayEnum.SMS)) { //SMS消息
					CompletableFuture<Void> task = _sendSmsMsg(userList, visualWarning, fieldDict, dataMap);
					if(task != null) {
						tasks.add(task);
					}
				}
				if(checkedWays.stream().anyMatch(item -> item.getWarningWayEnum() != WarningWayEnum.SMS)) { //非SMS消息，走文件和文本发送
					tasks.addAll(_sendFileMsg(userList, checkedWays, visualWarning));
				}
				
				if(!tasks.isEmpty()) {
					long startTime = System.currentTimeMillis();
					try { //线程执行的超时时间为2分钟
						CompletableFuture.allOf(tasks.stream().toArray(CompletableFuture[]::new)).get(2, TimeUnit.MINUTES);
					} catch (Exception e) {
						log.error(e.getMessage(), e);
					} finally {
						log.info("执行消息推送耗时：{}s", (System.currentTimeMillis() - startTime) / 1000.f);
					}
				}
				
				// 保存预警人和预警记录的中间表
				List<VisualWarningRecordUser> entityList = new ArrayList<VisualWarningRecordUser>();
				for(Long userId : userIdList) {
					entityList.add(new VisualWarningRecordUser(userId, visualWarning.getId(), warningRecord.getId()));
				}
				visualWarningRecordUserService.saveBatch(entityList);
			}
			log.info("完成一次预警任务|warnName={},warnStatus={},parseSqlCost={}ms,queryDataCost={}ms,totalCost={}ms",
					visualWarning.getName(), resultList != null && !resultList.isEmpty(), twoTime - oneTime,
					threeTime - twoTime, System.currentTimeMillis() - oneTime);
		}catch(Exception e) {
			log.error(e.getMessage(), e);
			XxlJobHelper.log(e);
			return XxlJobHelper.handleFail();
		}finally {
			if(visualWarning != null) { //更新下一次任务的时间戳
				long nextLoopTs = System.currentTimeMillis() + visualWarning.getRequestFreq() * 1000;
				VisualWarning entity = new VisualWarning();
				entity.setId(visualWarning.getId());
				entity.setNextLoopTs(nextLoopTs);
				visualWarningService.updateById(entity);
			}
		}
		return XxlJobHelper.handleSuccess();
	}
	
	/**
	 * 发送短信消息
	 * @param userList
	 * @param visualWarning
	 */
	private CompletableFuture<Void> _sendSmsMsg(List<UserModel> userList, VisualWarning visualWarning, Map<String, String> fieldDict, Map<String, Object> dataMap) {
		List<String> phones = userList.stream().map(UserModel::getPhone).filter(StrUtil::isNotEmpty).collect(Collectors.toList());
		if(CollectionUtil.isEmpty(phones)) {
			XxlJobHelper.log("未查询到有效的手机号");
			log.warn("未查询到有效的手机号");
			return null;
		}
		return CompletableFuture.runAsync(() -> {
			if(isUnicom) {//特殊操作：联通短信走另外一套逻辑
				UnicomSmsSender.sendSms(visualWarning.getFormatContent(), phones.stream().toArray(String[]::new));
			}else if(visualWarning.getSmsId() != null && StrUtil.isNotEmpty(visualWarning.getSmsParams())){//标准短信策略
				SmsModel smsModel = smsClient.getById(visualWarning.getSmsId());
				if(smsModel == null) {
					XxlJobHelper.log("未查询到有效的短信模版|smsid="+visualWarning.getSmsId());
					log.warn("未查询到有效的短信模版|smsid="+visualWarning.getSmsId());
				}else {
					//{code: "订购量"}   dataMap={"订购量": 1000}
					JSONObject smsParam = JSON.parseObject(visualWarning.getSmsParams());
					//请求到第三方短信参数Map
					Map<String, String> smsDataMap = new HashMap<>();
					smsParam.forEach((key, value) -> smsDataMap.put(key, Optional.ofNullable(dataMap.get(value)).orElse("").toString()));
					log.info("短信请求参数:" + smsDataMap);
					XxlJobHelper.log("短信请求参数:" + smsDataMap);
					MsgType msgType = null;
					if(smsModel.getCategoryVal() == 0) { //阿里云
						msgType = MsgType.smsAliyun;
					}else if(smsModel.getCategoryVal() == 1) { //七牛云
						msgType = MsgType.smsQiniu;
					}else {
						msgType = MsgType.smsTencent;
					}
					SmsInfo smsInfo = SmsInfo.builder().accessKey(smsModel.getAccessKey()).secretKey(smsModel.getSecretKey()).signName(smsModel.getSignName())
							.regionId(smsModel.getRegionId()).appId(smsModel.getAppId()).templateId(smsModel.getTemplateId()).build();
					SmsMsgRequest smsMsgRequest = SmsMsgRequest.builder().msgType(msgType).smsInfo(smsInfo).phones(phones).params(smsDataMap).build();
					MsgResponse msgResponse = SpringUtil.getBean(msgType.getClazz()).sendSmsMessage(smsMsgRequest);
					XxlJobHelper.log("【"+msgType.getMessage()+"】发送完成|res="+JSON.toJSONString(msgResponse));
				}
			}else {
				XxlJobHelper.log("短信推送方式有错误|smsid="+visualWarning.getSmsId());
				log.warn("短信推送方式有错误");
			}
		}, threadPool);
	}
	
	/**
	 * 发送文件和文本消息
	 * @param userList
	 * @param visualWarning
	 * @throws Exception 
	 */
	private List<CompletableFuture<Void>> _sendFileMsg(List<UserModel> userList, List<WarningWayDto> warningWayList, VisualWarning visualWarning) throws Exception {
		List<String> phones = userList.stream().map(UserModel::getPhone).filter(StrUtil::isNotEmpty).collect(Collectors.toList());
		List<String> emails = userList.stream().map(UserModel::getEmail).filter(StrUtil::isNotEmpty).collect(Collectors.toList());
		boolean notProcessEmail = warningWayList.stream().anyMatch(item -> item.getWarningWayEnum() == WarningWayEnum.EMAIL) && CollectionUtil.isEmpty(emails);
		boolean notProcessApp = warningWayList.stream().anyMatch(item -> item.getWarningWayEnum() != WarningWayEnum.EMAIL) && CollectionUtil.isEmpty(phones);
		if(notProcessEmail && notProcessApp) {
			return ListUtil.empty();
		}
		//获取大屏ID
		String visualDataJson = visualWarning.getVisualDataJson();
		List<CompletableFuture<Void>> tasks = new ArrayList<>();
		List<VisualData> visualDataList = JSON.parseArray(visualDataJson, VisualData.class);
		if(CollectionUtil.isNotEmpty(visualDataList)) {
			List<Integer> visualIdList = visualDataList.stream().map(VisualData::getId).collect(Collectors.toList());
			userList.forEach(x->{
				List<String> imageUrlList = new ArrayList<>();
				File _file = null;
				//获取大屏快照图片列表
				List<String> _list = visualDataPdfHandler.getVisualScreenShotImageUrl(x.getId(), visualWarning.getCreateBy(), visualIdList, DataType.visual);
				if(_list != null) {
					imageUrlList.addAll(_list);
				}
				//邮件不需要文件
				if(CollectionUtil.isNotEmpty(imageUrlList) && warningWayList.stream().anyMatch(item -> item.getWarningWayEnum() != WarningWayEnum.EMAIL)) {
					try {
						_file = visualDataPdfHandler.buildImagePdf(imageUrlList, visualWarning.getName());
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
				//构造请求参数
				for(WarningWayDto warningWay : warningWayList) {
					File final_file = _file;
					tasks.add(CompletableFuture.runAsync(() -> {
						if(warningWay.getWarningWayEnum() == WarningWayEnum.DINGTALK) {
							List<VisualSubscribeNotifyConfModel> list = subscribeNotifyConfClient.newNotifyConfList(StringUtil.isBlank(warningWay.getTenantId())?"0":warningWay.getTenantId(),NotifyType.dingtalk.getValue());
							if(CollectionUtil.isNotEmpty(list)) {
								Map<String, String> map = list.stream().collect(Collectors.toMap(VisualSubscribeNotifyConfModel::getAttriKey, VisualSubscribeNotifyConfModel::getAttriValue));
								AppMsgInfo appMsgInfo = AppMsgInfo.builder().agentId(map.get("agent_id")).appId(map.get("app_key")).appSecret(map.get("app_secret")).build();
								AppMsgRequest appMsgRequest = AppMsgRequest.builder().file(final_file).phones(Arrays.asList(x.getPhone())).appMsgInfo(appMsgInfo)
										.title(visualWarning.getName()).body(visualWarning.getFormatContent()).msgType(MsgType.appDingtalk).build();
								MsgResponse msgResponse = SpringUtil.getBean(MsgType.appDingtalk.getClazz()).sendAppMessage(appMsgRequest);
								XxlJobHelper.log("【"+NotifyType.dingtalk.getMessage()+"】发送完成|res="+JSON.toJSONString(msgResponse));
							}
						}else if(warningWay.getWarningWayEnum() == WarningWayEnum.FEISHU) {
							List<VisualSubscribeNotifyConfModel> list = subscribeNotifyConfClient.newNotifyConfList(StringUtil.isBlank(warningWay.getTenantId())?"0":warningWay.getTenantId(),NotifyType.feishu.getValue());
							if(CollectionUtil.isNotEmpty(list)) {
								Map<String, String> map = list.stream().collect(Collectors.toMap(VisualSubscribeNotifyConfModel::getAttriKey, VisualSubscribeNotifyConfModel::getAttriValue));
								AppMsgInfo appMsgInfo = AppMsgInfo.builder().appId(map.get("app_id")).appSecret(map.get("app_secret")).build();
								AppMsgRequest appMsgRequest = AppMsgRequest.builder().file(final_file).phones(Arrays.asList(x.getPhone())).appMsgInfo(appMsgInfo)
										.title(visualWarning.getName()).body(visualWarning.getFormatContent()).msgType(MsgType.appFeishu).build();
								MsgResponse msgResponse = SpringUtil.getBean(MsgType.appFeishu.getClazz()).sendAppMessage(appMsgRequest);
								XxlJobHelper.log("【"+NotifyType.feishu.getMessage()+"】发送完成|res="+JSON.toJSONString(msgResponse));
							}
						}else if(warningWay.getWarningWayEnum() == WarningWayEnum.QYWECHAT) {

							List<VisualSubscribeNotifyConfModel> list = subscribeNotifyConfClient.newNotifyConfList(StringUtil.isBlank(warningWay.getTenantId())?"0":warningWay.getTenantId(),NotifyType.qywechat.getValue());
							if(CollectionUtil.isNotEmpty(list)) {
								Map<String, String> map = list.stream().collect(Collectors.toMap(VisualSubscribeNotifyConfModel::getAttriKey, VisualSubscribeNotifyConfModel::getAttriValue));
								AppMsgInfo appMsgInfo = AppMsgInfo.builder().agentId(map.get("agent_id")).corpSecret(map.get("corp_secret")).appId(map.get("corp_id")).appSecret(map.get("app_secret")).build();
								AppMsgRequest appMsgRequest = AppMsgRequest.builder().file(final_file).phones(Arrays.asList(x.getPhone())).appMsgInfo(appMsgInfo)
										.title(visualWarning.getName()).body(visualWarning.getFormatContent()).msgType(MsgType.appQywechat).build();
								MsgResponse msgResponse = SpringUtil.getBean(MsgType.appQywechat.getClazz()).sendAppMessage(appMsgRequest);
								XxlJobHelper.log("【"+NotifyType.qywechat.getMessage()+"】发送完成|res="+JSON.toJSONString(msgResponse));
							}
						}else if(warningWay.getWarningWayEnum() == WarningWayEnum.EMAIL) {
							String htmlContent = visualWarning.getFormatContent();
							if(CollectionUtil.isNotEmpty(imageUrlList)) {
								htmlContent = htmlContent + "<p>&nbsp;</p>" + imageUrlList.stream().map(t -> String.format("<img src=\"%s\">", t)).collect(Collectors.joining("</br>"));
							}
							String subject = visualWarning.getName() + " " + DateUtil.format(new Date(), "yyyyMMdd");
							List<VisualSubscribeNotifyConfModel> list = subscribeNotifyConfClient.newNotifyConfList(StringUtil.isBlank(warningWay.getTenantId())?"0":warningWay.getTenantId(),NotifyType.email.getValue());
							if(CollectionUtil.isNotEmpty(list)) {
								Map<String, String> emailMap = list.stream().collect(Collectors.toMap(VisualSubscribeNotifyConfModel::getAttriKey, VisualSubscribeNotifyConfModel::getAttriValue));
								EmailInfo emailInfo = EmailInfo.builder().smtpHost(emailMap.get("smtp")).userName(emailMap.get("username")).passWord(emailMap.get("password")).build();
								EmailMsgRequest emailMsgRequest = EmailMsgRequest.builder().body(htmlContent).title(subject).msgType(MsgType.email).emails(Arrays.asList(x.getEmail())).emailInfo(emailInfo).build();
								//发送邮件
								MsgResponse msgResponse = SpringUtil.getBean(MsgType.email.getClazz()).sendEmailMessage(emailMsgRequest);
								XxlJobHelper.log("【"+NotifyType.email.getMessage()+"】发送完成|res="+JSON.toJSONString(msgResponse));
							}
						}else if(warningWay.getWarningWayEnum() == WarningWayEnum.TIANXIANGYUN) {
							List<VisualSubscribeNotifyConfModel> list = subscribeNotifyConfClient.newNotifyConfList(StringUtil.isBlank(warningWay.getTenantId())?"0":warningWay.getTenantId(),NotifyType.tianxiangyun.getValue());
							if(CollectionUtil.isNotEmpty(list)) {
								Map<String, String> map = list.stream().collect(Collectors.toMap(VisualSubscribeNotifyConfModel::getAttriKey, VisualSubscribeNotifyConfModel::getAttriValue));
								TianXiangYunMsgTemplate tianXiangYunMsgTemplate=new TianXiangYunMsgTemplate();
								TianXiangYunMsgTemplate.TianXiangYunMsgInfo msgInfo =tianXiangYunMsgTemplate.new TianXiangYunMsgInfo();
								map.put("file",domain+"/uploads/pdf/"+ LocalDate.now().getYear()+"/"+String.format(Locale.US,"%tB",LocalDate.now()).toLowerCase()+"/"+ final_file.getName());
								map.put("title",visualWarning.getName());
								msgInfo.setParamMap(map);
								msgInfo.setAccounts(Arrays.asList(x.getAccount()));
								MsgResponse msgResponse = null;
								try {
									msgResponse = msgInfo.sendFileMessage();
								} catch (Exception e) {
									e.printStackTrace();
								}
								XxlJobHelper.log("【"+NotifyType.tianxiangyun.getMessage()+"】发送完成|res="+JSON.toJSONString(msgResponse));
							}
						}
					}, threadPool));
				}
			});
		}
		return tasks;
	}
	
	/**
	 * 解析数据
	 * @param tplContent
	 * @param params
	 * @param pattern
	 * @return
	 */
	final static Pattern FORMAT_AT_PATTERN = Pattern.compile("@.*?@");
	private static String _parser(String tplContent, Map<String, ?> params) {
		Matcher matcher = FORMAT_AT_PATTERN.matcher(tplContent);
		while (matcher.find()) {
			String group = matcher.group();
			String key = group.substring(1, group.length() - 1);
			if (params.containsKey(key)) {
				tplContent = tplContent.replace(group, params.get(key).toString());
			}
		}
		return tplContent;
	}
}
