/**
 * 
 */
package org.springblade.visual.task.unicom;

import java.util.Map;
import java.util.concurrent.TimeUnit;

import lombok.extern.slf4j.Slf4j;
import okhttp3.HttpUrl;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;

/**
 * @author wookvn
 *
 */
@Slf4j
public class HttpKit {

	final static OkHttpClient httpClient = new OkHttpClient.Builder()
			// 信任所有ssl证书
			// .sslSocketFactory(createSSLSocketFactory(), x509tm)
			// 信任所有主机
			.hostnameVerifier((hostname, session) -> true)
			// 设置读取超时时间
			.readTimeout(2000, TimeUnit.MILLISECONDS)
			// 设置写的超时时间
			.writeTimeout(2000, TimeUnit.MILLISECONDS)
			// 设置连接的超时时间
			.connectTimeout(2000, TimeUnit.MILLISECONDS)
			// 关闭重试
			.retryOnConnectionFailure(false)
			// 禁止重定向
			.followRedirects(false)
			.build();

	/**
	 * GET请求
	 * 
	 * @param url
	 * @param params
	 * @param timeout
	 * @return
	 */
	public static String get(String url, Map<String, String> params) {
		try {
			Request.Builder requestBuilder = new Request.Builder();
			HttpUrl.Builder uriBuilder = HttpUrl.parse(url).newBuilder();
			params.forEach(uriBuilder::addQueryParameter);
			Response response = httpClient.newCall(requestBuilder.url(uriBuilder.build()).method("GET", null).build()).execute();
			return response.body().string();
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			return null;
		}
	}
}
